function axes_alignment(r,c)

%
% aligns axes objects based on either number of rows or on number of columns.
%

fig = get(0,'CurrentFigure');
toolbar = get(fig,'UserData');
myaxes = get(toolbar.print,'UserData');
num_axes = size(myaxes,2);
if r>0;
   r = min(r,num_axes);
   c = ceil(num_axes/r);
elseif c>0;
   c = min(c,num_axes);
   r = ceil(num_axes/c);
end;

for i=1:num_axes;
   old_units{i} = get(myaxes(1,i),'Units');
   set(myaxes(1,i),'Units','normalized');
end;

%
% determine new positions of plots by creating a hidden figure with r*c axes objects
% and then reading off their positions
%

new_fig = figure('Units',get(fig,'Units'), ...
   'Position',get(fig,'Position'), ...
   'MenuBar','none', ...
   'ToolBar','none', ...
   'Visible','off');

for i=1:r*c;
   subplot(r,c,i);
   set(gca,'Units','normalized');
   ax(i).position = get(gca,'Position');
end;

delete(new_fig);

%for i=1:num_axes;
%   set(myaxes(1,i),'Position',ax(i).position);
%end;

for i=1:r;
   for j=1:c;
      num1 = i+(r*(j-1));
      num2 = j+(c*(i-1));
      if (num1<=num_axes)&(num2<=num_axes);
         set(myaxes(1,num1),'Position',ax(num2).position);
      end;
   end;
end;

%
% return old axes units
%

for i=1:num_axes;
   set(myaxes(1,i),'Units',old_units{i});
end;

%
% end of axes_alignment.m
%
