function [nyblomT,nyblomcorr,e_parm] = boot_nyblom_partial(Z0,Z1,Z2,r,p,q1,d,T0,np,real_T)

%
% function for bootstrapping Nyblom test with Psi set at full sample estimate
%

curr_ini = find_inisettings;

[n T] = size(Z0);

%
% Ensuring that all input variables are set
%

if nargin<7;
   T1 = round(2*real_T/3);
else;
   if T0<0;
      T1 = round(2*real_T/3);
   elseif T0>0.95;
      T1 = round(2*real_T/3);
   else;
      T1 = round(real_T*T0);
   end;
end;

if T1<=size(Z1,1)+size(Z2,1);
   T1 = size(Z1,1)+size(Z2,1)+1;
end;

if nargin<8;
   np = r+round(r*(n-r)/n)+n*(p-1)+d;
end;

if T1>T;
   nyblomT = [];
   nyblomcorr = [];
   return;
end;

%
% compute full sample estimates
%

[S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,trace_test] = mlcointbasic(Z0,Z1,Z2,r);

alpha = S_01*beta/(beta'*S_11*beta);
if isempty(Z2)==0;
   Psi = (M_02/M_22)-(alpha*beta'*(M_12/M_22));
   epsilon = Z0-(alpha*beta'*Z1)-(Psi*Z2);
else;
   Psi = [];
   epsilon = Z0-(alpha*beta'*Z1);
end;
Omega = (1/T)*epsilon*epsilon';

if isempty(Z2)==0;
   R0 = Z0-(M_02*(M_22\Z2));
   R1 = Z1-(M_12*(M_22\Z2));
else;
   R0 = Z0;
   R1 = Z1;
end;

%
% if full sample estimates of the parameters are requested
%

if nargout>2;
   e_parm.alpha = alpha;
   e_parm.beta = beta;
   e_parm.Psi = Psi;
   e_parm.Omega = Omega;
   e_parm.epsilon = epsilon;
end;

%
% material for Nyblom test
%

if r<n;
   beta_orth = null(beta(1:n+q1,1:r)');
   qp = size(beta,1)-n-q1;
   if size(beta,1)==n+q1;
      c = beta;
      c_orth = beta_orth;
   else;
      c = beta(1:n+q1,:);
      c_orth = beta_orth(1:n+q1,:);
      c = [c;zeros(qp,r)];
      c_orth = [[c_orth zeros(n,qp)];[zeros(qp,n-r) eye(qp)]];
   end;
   if curr_ini.taylor_nyblom==1;
      c_bar = c/(c'*c);
      c_bar_orth = c_orth*inverse(c_orth'*c_orth);
      betaT = beta/(c_bar'*beta);
      alpha_cT = alpha*beta'*c_bar;
      M_T = (1/T)*c_orth'*S_11*c_orth;
      V_T = alpha_cT'*(Omega\alpha_cT);
   else;
      inv_Omega = eye(size(Omega,1))/Omega;
      J_T = kron(alpha'*inv_Omega*alpha,c_orth'*S_11*c_orth);
      alpha_T = alpha;
      Omega_T = Omega;
      beta_T = beta(:,1:r);
   end;
end;

beta = beta(1:n+q1,1:r);

nyblomT = zeros(1,T-T1+1);
nyblomcorr = zeros(1,T-T1+1);

t = T-1;
while t>=T1;
   R0t = R0(:,1:t);
   R1t = R1(:,1:t);
   S_00 = (1/t)*R0t*R0t';
   S_11 = (1/t)*R1t*R1t';
   S_01 = (1/t)*R0t*R1t';
   S_10 = (1/t)*R1t*R0t';
   if curr_ini.taylor_nyblom==1;
      %
      % Solving the eigenvalue problem
      %
      % Step 1: Diagonalize S_11
      %
      [W,rho] = eig(S_11);
      rho = diag(rho,0);
      [rho2,AscInd] = sort(abs(rho));
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      for i=1:nr;
         DescInd(i)=AscInd(nr+1-i);
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      rho = rho(DescInd);
      sqrho = sqrt(rho);
      rho = diag(rho,0);
      sqrho = diag(sqrho,0);
      W = W(:,DescInd);
      %
      % Step 2: Reformulating eigenvalue problem to
      %         calculating the eigenvalues for
      %
      %         adjS_11 = sqS_11*S_10*inverse(S_00)*S_01*sqS_11
      %
      sqS_11 = W*(sqrho\W');
      adjS_11 = sqS_11*S_10*(S_00\S_01)*sqS_11;
      [U,lambda] = eig(adjS_11);
      lambda = diag(lambda,0);
      [lambda2,AscInd] = sort(abs(lambda));
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      for i=1:nr;
         DescInd(i)=AscInd(nr+1-i);
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      lambda = lambda(DescInd);
      U = U(:,DescInd);
      %
      % Renormalize U such that V'S_11V=I
      %
      V = sqS_11*U;
      %
      beta = V(:,1:r);
   end;
   %
   if r<n;
      if curr_ini.taylor_nyblom==1;
         betat = beta;
         M_t = c_orth'*S_11*c_orth;
         beta_ct = betat/(c_bar'*betat);
         q_t = c_bar_orth'*(beta_ct-betaT);
         nyblomt = V_T*q_t'*M_t*(M_T\M_t)*q_t;
         nyblomt = sum(diag(nyblomt));
         nyblomT(1,t-T1+1) = (t/T)*(t/T)*nyblomt(1,1);
         nyblomcorr(1,t-T1+1) = ((T-np)/T)*(t/T)*(t/T)*nyblomt(1,1);
      else;
         inv_Omega_T = eye(size(Omega_T,1))/Omega_T;
         S_t = (vec(alpha_T))'*kron(eye(r),inv_Omega_T*(S_01-alpha_T*beta_T'*S_11)*c_orth);
         inv_J_T = eye(size(J_T,1))/J_T;
         Q_t = (t/T)*(t/T)*T*S_t*inv_J_T*S_t';
         nyblomT(1,t-T1+1) = Q_t;
         nyblomcorr(1,t-T1+1) = ((T-np)/T)*Q_t;
      end;
   end;
   %
   t = t-1;
end;

%
% end of boot_nyblom_partial.m
%
