%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating eigenvalues            %
%          of the companion matrix using restricted beta         %
%                                                                %
%          Function: companionrest                               %
%                                                                %
%          Input:                                                %
%                       alpha: nxr matrix                        %
%                       Psi: nxnreg matrix                       %
%                       mybeta: nq1xr matrix                     %
%                       p: lag length                            %
%                       n: number of endogenous variables        %
%                       d: number of deterministic variables     %
%                                                                %
%          Output:                                               %
%          	            eigenvalues: nx1 vector of eigenvalues   %
%                           of the companion matrix              %
%                                                                %
%          This version:        June 1, 2004                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function eigenvalues = companionrest(alpha,Psi,mybeta,p,n,d)

if isempty(mybeta)==0;
   Pi = alpha*mybeta(1:n,:)';
else;
   Pi = zeros(n,n);
end;
if p>=2;
   Phi = Psi(:,d+1:d+n*(p-1));
end;

ip = zeros(p,p);
i = 1;
while i<=p;
   j = 1;
   while j<=p;
      if j>=i;
         ip(j,i)=1;
      end;
      j = j+1;
   end;
   i = i+1;
end;

%
% Computing levels VAR coefficients from
% VECM coefficients
%

ip = kron(ip,eye(n));
if p>=2;
   M = [(Pi+eye(n)) -Phi]/ip;
   M = [M;[eye(n*(p-1)) zeros(n*(p-1),n)]];
else;
   M = (Pi+eye(n))/ip;
end;

eigenvalues = eig(M);

%
% end of function companionrest.m
%
