%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for selecting the forecast horizon  %
%                                                                %
%          Function: forecast_horizon                            %
%                                                                %
%          Input:                                                %
%                       mylabel: dialog name                     %
%                       freq: data frequency                     %
%                       T: number of observations                %
%                       focus: yes or no (yes by default)        %
%                                                                %
%          Output:                                               %
%                       fh: integer                              %
%                                                                %
%          This version:        May 3, 2006                      %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function fh = forecast_horizon(mylabel,freq,T,focus)

curr_ini = find_inisettings;

if nargin<4;
   focus = 'ok';
else;
   if ischar(focus)==1;
      focus = lower(focus);
      if (strcmp(focus,'ok')==0)&(strcmp(focus,'cancel')==0);
         focus = 'ok';
      end;
   else;
      focus = 'ok';
   end;
end;

if strcmp(freq,'q')==1;
   freq_str = 'quarter';
   min_horizon = 20;
   fh_value = 4*curr_ini.forecasthorizon;
elseif strcmp(freq,'m')==1;
   freq_str = 'month';
   min_horizon = 60;
   fh_value = 12*curr_ini.forecasthorizon;
else;
   freq_str = 'year';
   min_horizon = 5;
   fh_value = curr_ini.forecasthorizon;
end;

for i=1:min(round(T/4),min_horizon);
   if i==1;
      fh_str = ['1 ' freq_str];
   else;
      fh_str = [fh_str '|' num2str(i,'%0.0f') ' ' freq_str 's'];
   end;
end;

if fh_value>min(round(T/4),min_horizon);
   fh_value = min(round(T/4),min_horizon);
end;

mycons = getstoredimages;
if curr_ini.use_toolbar_bgcolor==1;
   bgcolor = (1/255)*(round(255*get(0,'defaultuicontrolbackgroundcolor'))-[8 4 8]);
   bgcolor(1) = max(0,bgcolor(1));
   bgcolor(2) = max(0,bgcolor(2));
   bgcolor(3) = max(0,bgcolor(3));
else;
   bgcolor = get(0,'defaultuicontrolbackgroundcolor');
end;
%
mylogo = mycons.forecastbig_image;
%
mylogo = maketrans(mylogo,[160 160 164],bgcolor);
mylogo = mirrorimage(mylogo);

scrsz = get(0,'ScreenSize');

histdlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'FileName','forecast_horizon.m', ...
   'MenuBar','none', ...
   'Units','pixels', ...
   'Position',[(scrsz(3)-450)/2 (scrsz(4)-120)/2 450 120], ...
   'Tag','ForecastHorizon', ...
   'Resize','off', ...
   'Visible','off', ...
   'Name',mylabel, ...
   'NumberTitle','off', ...
   'WindowStyle','modal', ...
   'CloseRequestFcn','closegui', ...
   'ToolBar','none');

if matlabnumber>=7.0;
   set(histdlg,'DockControl','off');
end;

axesbox([2 2 448 110],'Forecast Horizon',82,[0.5 1]);

imageaxes = axes('Units','pixels', ...
   'Box','on', ...
   'TickLength',[0 0], ...
   'CameraUpVector',[0 1 0], ...
   'Color',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[400 60 39 39], ...
   'XTickLabelMode','manual', ...
   'YTickLabelMode','manual', ...
   'Tag','AxisLogo', ...
   'XColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'YColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'ZColor',get(0,'defaultuicontrolbackgroundcolor'));

image('CData',mylogo,'Clipping','off','Parent',imageaxes);

uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[50 60 120 20], ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Style','text', ...
   'String','Select Forecast Horizon:', ...
   'HorizontalAlignment','left', ...
   'Tag','ForecastHorizonText');

histcontrol.fh = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[170 64 180 20], ...
   'Style','popupmenu', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'String',fh_str, ...
   'HorizontalAlignment','center', ...
   'Tag','ForecastHorizon',...
   'Value',fh_value);

if strcmp(focus,'ok')==1;
   id_str = ' 2';
else;
   id_str = ' 1';
end;

histcontrol.ok = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[149.5 12 75 25], ...
   'String','OK', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'UserData','waiting', ...
   'Tag','YesButton');

histcontrol.cancel = uicontrol('Units','pixels', ...
   'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
   'Position',[227.5 12 75 25], ...
   'String','Cancel', ...
   'FontSize',curr_ini.gui_fontsize, ...
   'FontName',curr_ini.gui_fontname, ...
   'FontWeight',curr_ini.gui_fontweight, ...
   'FontAngle',curr_ini.gui_fontangle, ...
   'Enable','on', ...
   'Callback','closegui', ...
   'Tag','NoButton');

set(histdlg,'UserData',histcontrol, ...
   'HandleVisibility','callback');

if (iswinnt==1)&(iscompiled==1)&(curr_ini.noicon==1);
   set(histdlg,'WindowStyle','normal');
   prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(histdlg,'Name') ' NULL'];
   dos(prog);
   wait_fcn(curr_ini.delay);
   pos = get(histdlg,'Position');
   set(histdlg,'Position',[0 0 1 1]);
end;
%
set(histdlg,'Visible','on');
drawnow;
if (iswinnt==1)&(iscompiled==1);
   if curr_ini.noicon==1;
      drawnow;
      set(histdlg,'WindowStyle','modal');
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(histdlg,'Name') ' NULL'];
      dos(prog);
      wait_fcn(curr_ini.delay);
      set(histdlg,'Position',pos);
   end;
   %
   drawnow;
   if matlabnumber<7;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(histdlg,'Name') id_str];
      dos(prog);
      drawnow;
   else;
      if strcmp(strtrim(id_str),'2')==1;
         uicontrol(histcontrol.ok);
      else;
         uicontrol(histcontrol.cancel);
      end;
   end;
   if curr_ini.noicon==1;
      set(histdlg,'Position',pos);
   end;
end;

waitfor(histdlg,'waitstatus','inactive');
%uiwait(histdlg);

co_str = lower(get(get(histdlg,'CurrentObject'),'String'));
if strcmp(co_str,'cancel')==1;
   fh = 0;
else;
   fh = get(histcontrol.fh,'Value');
end;

delete(histdlg);
drawnow;

%
% end of forecast_horizon.m
%
