%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI for determining the restrictions on a      %
%          vector of beta                                        %
%                                                                %
%          Function: hmatgui                                     %
%                                                                %
%          Input:                                                %
%                        selector: case switch function          %
%                        cm: structure with model info           %
%                                                                %
%          Output:                                               %
%                        h: handle                               %
%                        and Hmat to H_i-modelfile.rest          %
%                                                                %
%          This version:        May 18, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function outh = hmatgui(selector,cm)

if strcmp(selector,'init')==1;
   tm = cm;
   curr_ini = find_inisettings;
   if tm.index==tm.r;
      contstr = 'Done';
   else;
      contstr = 'Continue';
   end;
   tm.variables = strtrim([tm.evars ' ' strtrim([tm.xvars1 ' ' tm.restvec])]);
   tm.variables = vecstr(tm.variables);
   myvariables = strrep(tm.variables(1,:),' ','');
   j = 2;
   while j<=tm.n+tm.q1+tm.dcoint;
      myvariables = [myvariables '|' strrep(tm.variables(j,:),' ','')];
      j = j+1;
   end;
   myreststr = '0';
   for i=tm.r+1:tm.n+tm.q1+tm.dcoint;
      myreststr = [myreststr '|' num2str(i-tm.r)];
   end;
   mycvstr = '1';
   for i=2:tm.r;
      mycvstr = [mycvstr '|' num2str(i)];
   end;
   thisexit = 0;
   tm.cancel = 0;
   tm.oldHmat = tm.Hmat;
   clearenable = 'on';
   if zeros(size(tm.Hmat,1),size(tm.Hmat,2))==tm.Hmat;
      clearenable = 'off';
   end;
   tm.hdim = size(tm.Hmat,2)-size(tm.Hmat,1)+1;
   tm.col = tm.n+tm.q1+tm.dcoint;
   tm.row = tm.n+tm.q1+tm.dcoint-tm.hdim+1;
   heightf = tm.row*20+(tm.row-1)*2+44;
   widthf = tm.col*40+(tm.col-1)*2+24;
   if widthf<350;
      addleft = (350-widthf)/2;
      widthf = 350;
   else;
      addleft = 0;
   end;
   realscrsz = get(0,'ScreenSize');
   if heightf>realscrsz(4)-200;
      heightf = realscrsz(4)-200;
   end;
   height = heightf+150;
   width = widthf+28;
else;
   thisexit = 1;
   hmatfig = get(0,'CurrentFigure');
   hmats = get(hmatfig,'UserData');
   curr_ini = get(hmats.undo,'UserData');
   tm = curr_ini.models;
end;

%
% Display GUI
%

switch selector;
   case 'init';
      hmatfig = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','hmatgui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-width)/2 (realscrsz(4)-height-100)/2 width height+100], ...
         'Tag','FigParent', ...
         'Resize','off', ...
         'Name',['Restrictions on Beta(' int2str(tm.index) ')'], ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','hmatgui exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(hmatfig,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 width-2 height+90],['H(' int2str(tm.index) ') Matrix Specification - Beta(' int2str(tm.index) ') = H(' int2str(tm.index) ')*psi(' int2str(tm.index) ')'],225,[0.5 1]);

      %
      % GUI Header
      %

      mycons = getstoredimages;
      header_txt = 'Beta Restrictions';
      desc_txt = 'Select the number of overidentifying restrictions you''d like to impose, enter the restrictions and the variable by which to normalize the vector. Press "Continue" ("Done") to move to the next restriction (finish).';
      dialog_header_fcn([14 height-5 widthf 82],mirrorimage(maketrans(mycons.betarestbig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);

      %
      % Outer axis for Hmat matrix
      %

      axesbox([14 128 widthf heightf],['Transpose of H(' int2str(tm.index) ')'],90,[0.5 1]);

      %
      % Outer axis for Normalizing control
      %

      axesbox([14 14 width-133 100],'Restriction Parameters',110,[0.5 1]);

      %
      % Edit control for element (i,j) in Hmat
      %

      i = 1;
      while i<=tm.row;
         j = 1;
         while j<=tm.col;

            hmats.editbox = uicontrol('Units','pixels', ...
               'BackgroundColor',[1 1 1], ...
               'Position',[26+addleft+(j-1)*42 140+(i-1)*22 40 20], ...
               'FontSize',curr_ini.gui_fontsize, ...
               'FontName',curr_ini.gui_fontname, ...
               'FontWeight',curr_ini.gui_fontweight, ...
               'FontAngle',curr_ini.gui_fontangle, ...
               'Style','edit', ...
               'HorizontalAlignment','left', ...
               'String',num2str(tm.Hmat(tm.row-i+1,j)), ...
               'Enable','on', ...
               'Callback','hmatgui checkelement', ...
               'UserData',[tm.row-i+1 j tm.Hmat(tm.row-i+1,j)], ...
               'Tag',['EditBox_' int2str(tm.row-i+1) '_' int2str(j)]);

            j = j+1;
         end;
         i = i+1;
      end;

      j = 1;
      while j<=tm.col;

         uicontrol('Units','pixels', ...
            'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
            'Position',[26+addleft+(j-1)*42 138+(i-1)*22 40 20], ...
            'FontSize',curr_ini.gui_fontsize, ...
            'FontName',curr_ini.gui_fontname, ...
            'FontWeight',curr_ini.gui_fontweight, ...
            'FontAngle',curr_ini.gui_fontangle, ...
            'Style','text', ...
            'String',tm.variables(j,:), ...
            'Tag',['VariableText_' int2str(j)]);

         j = j+1;
      end;

      %
      % Controls for selecting cointegration vector
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 76 140 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'HorizontalAlignment','left', ...
         'String','Select cointegration vector:', ...
         'Tag','CointText');

      hmats.cv = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[max([width-193 160]) 81 60 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Value',tm.index, ...
         'String',mycvstr, ...
         'CallBack','hmatgui cointrel', ...
         'UserData','done', ...
         'Tag','CVPopup');

      %
      % Controls for number of restrictions
      %

      uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 51 140 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'HorizontalAlignment','left', ...
         'String','Number of restrictions:', ...
         'Tag','RestText');

      hmats.rest = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[max([width-193 160]) 56 60 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Value',tm.hdim+1-tm.r, ...
         'String',myreststr, ...
         'CallBack','hmatgui rest', ...
         'UserData','done', ...
         'Tag','RestPopup');

      %
      % Text control for normalizing beta
      %

      hmats.text = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 26 140 20], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'HorizontalAlignment','left', ...
         'String','Normalizing Beta by variable:', ...
         'UserData',tm.Hmat, ...
         'Tag','NormalizeText');

      %
      % Popup control for selecting variable to normalize
      %

      hmats.popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[max([width-193 160]) 31 60 20], ...
         'Style','popupmenu', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Value',tm.normvar, ...
         'String',myvariables, ...
         'CallBack','hmatgui change', ...
         'UserData','working', ...
         'Tag','NormalizePopup');

      %
      % Undo button
      %

      hmats.undo = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-105 95 90 20], ...
         'String','Undo', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','hmatgui undo', ...
         'Enable','off', ...
         'Tag','UndoButton');

      %
      % Clear button
      %

      hmats.clear = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-105 70 90 20], ...
         'String','Clear', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','hmatgui clear', ...
         'Enable',clearenable, ...
         'Tag','ClearButton');

      %
      % Continue button
      %

      hmats.continue = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-105 45 90 20], ...
         'String',contstr, ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','hmatgui exit', ...
         'UserData','waiting', ...
         'Enable','on', ...
         'Tag','ContinueButton');

      %
      %  Cancel button
      %

      hmats.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-105 20 90 20], ...
         'String','Cancel', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','hmatgui cancel', ...
         'Enable','on', ...
         'UserData',thisexit, ...
         'Tag','CancelButton');

      set(hmatfig,'UserData',hmats,...
         'HandleVisibility','callback');
   case 'undo'
      set(hmats.text,'UserData',tm.oldHmat);
      set(hmats.rest,'UserData','redraw');
      set(hmats.cancel,'UserData',thisexit);
      set(hmatfig,'UserData',hmats);
      hmatgui('exit');
   case 'clear'
      if curr_ini.sillyq==0;
         txt = 'Are you sure you want to empty the set of restrictions?';
         answer = query(txt,'question',120,'Clear Restrictions',350,'no');
      else;
         answer = 'Yes';
      end;
      if strcmp(answer,'Yes')==1;
         tm.Hmat = zeros(tm.n+tm.q1+tm.dcoint-tm.hdim+1,tm.n+tm.q1+tm.dcoint);
         set(hmats.text,'UserData',tm.Hmat);
         set(hmats.rest,'UserData','redraw');
         set(hmats.cancel,'UserData',thisexit);
         set(hmatfig,'UserData',hmats);
         hmatgui('exit');
      end;
   case 'rest'
      numrest = get(hmats.rest,'Value')+tm.r-1;
      if isequal(tm.hdim,numrest)==0;
         if numrest>tm.hdim;
            tm.Hmat = tm.Hmat(1:tm.n+tm.q1+tm.dcoint-numrest+1,:);
         else;
            tm.Hmat = [tm.Hmat;zeros(tm.hdim-numrest,size(tm.Hmat,2))];
         end;
         set(hmats.text,'UserData',tm.Hmat);
         set(hmats.rest,'UserData','redraw');
         set(hmats.cancel,'UserData',thisexit);
         set(hmatfig,'UserData',hmats);
         hmatgui('exit');
      else;
         set(hmatfig,'UserData',hmats);
      end;
   case 'cointrel'
      newindex = get(hmats.cv,'Value');
      if isequal(newindex,tm.index)==0;
         hmatgui('exit');
      end;
   case 'change'
      tm.normvar = get(hmats.popup,'Value');
      set(hmats.cancel,'UserData',thisexit);
      set(hmatfig,'UserData',hmats);
   case 'checkelement'
      whichelem = gcbo;
      currelem = get(whichelem,'Userdata');
      if isempty(currelem)==0;
         currvalue = get(whichelem,'String');
         currvalue = strrep(currvalue,',','.');
         currvalue = mystr2num(currvalue);
         if isempty(currvalue)==0;
            tm.Hmat(currelem(1,1),currelem(1,2)) = currvalue(1,1);
            set(whichelem,'UserData',[currelem(1,1:2) currvalue(1,1)]);
            set(whichelem,'String',num2str(currvalue(1,1)));
            set(whichelem,'BackgroundColor',[1 0.75 0.75]);
            set(hmats.clear,'Enable','on');
            set(hmats.undo,'Enable','on');
         else;
            set(whichelem,'String',num2str(currelem(1,3)));
         end;
      end;
      set(hmats.text,'UserData',tm.Hmat);
      set(hmats.cancel,'UserData',thisexit);
      set(hmatfig,'UserData',hmats);
   case 'cancel'
      thisexit = 2;
      tm.cancel = 1;
      set(hmats.cancel,'UserData',thisexit);
      set(hmatfig,'UserData',hmats);
      curr_ini.models = tm;
      set(hmats.undo,'UserData',curr_ini);
      hmatgui('exit');
   case 'exit'
      closeexit = 1;
      if strcmp(get(hmats.popup,'UserData'),'done')==0;
         if abs(tm.cancel-1)>0;
            if strcmp(get(hmats.rest,'UserData'),'redraw')==0;
               if isequal(get(hmats.cv,'Value'),tm.index)==1;
                  %
                  % Get info about Hmat and check if it has full rank
                  %
                  checkdet = det(tm.Hmat*tm.Hmat');
                  if abs(checkdet)<0.00000000000001;
                     closeexit = 0;
                     txt = ['The matrix H(' int2str(tm.index) ') does not have full rank. Please insert additional non-zero elements or cancel the operation.'];
                     about(txt,'warning',['Restrictions on Beta(' int2str(tm.index) ') - Information'],130,500);
                  end;
                  %
                  % Check the normalizing variable
                  %
                  if closeexit==1;
                     checkvec = tm.Hmat(:,tm.normvar);
                     checkvec = checkvec'*checkvec;
                     if abs(checkvec)<0.00000000000001;
                        closeexit = 0;
                        txt = ['You have chosen to normalize H(' int2str(tm.index) ') by ' strrep(tm.variables(tm.normvar,:),' ','') '. This implies that the program should divide the coefficients of Beta(' int2str(tm.index) ') by 0. Please change your selection or cancel the operation.'];
                        about(txt,'warning',['Restrictions on Beta(' int2str(tm.index) ') - Information'],130,500);
                     end;
                  end;
                  if closeexit==1;
                     savefile = strcat(tm.modelsdir,'H_',int2str(tm.index),'-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
                     myprintstr = '%2.4f';
                     j = 2;
                     while j<=tm.row;
                        myprintstr = [myprintstr ' %2.4f'];
                        j = j+1;
                     end;
                     myprintstr = [myprintstr '\n'];
                     fid = fopen(savefile,'wt');
                     fprintf(fid,myprintstr,tm.Hmat);
                     fclose(fid);
                  end;
               end;
            end;
         end;
      end;
      if closeexit==1;
         set(hmats.continue,'UserData','done');
         set(hmatfig,'UserData',hmats);
%         if strcmp(get(hmats.popup,'UserData'),'done')==1;
%            delete(get(0,'CurrentFigure'));
%         end;
      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(hmatfig,'Name') ' ' fix_filepath([pwd '\icons\betarest.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(hmats.clear,'UserData',pid);
      end;
   end;
   %
   set(hmatfig,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(hmatfig,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(hmats.cancel);
   end;
end;

if nargout==1;
   outh = get(0,'CurrentFigure');
end;

curr_ini.models = tm;
set(hmats.undo,'UserData',curr_ini);

if thisexit==0;
   waitfor(hmats.continue,'UserData','done');
end;

%
% end of hmatgui.m
%
