function move_preview_plots_fcn(fig,this_gui)

%
% fig is the figure handle and this_gui is the handle for the figure model in print preview
%

old_gui_units = get(this_gui,'Units');
set(this_gui,'Units','pixels');
gui_scale_pos = get(this_gui,'Position');
set(this_gui,'Units','normalized');
gui_pos = get(this_gui,'Position');

toolbar = get(fig,'UserData');
myaxes = get(toolbar.print,'UserData');
inv_hardcopy = get(fig,'InvertHardcopy');

%
% learn about scaling of model figure area relative to print figure area
%

if strcmp(get(fig,'PaperPositionMode'),'auto')==1;
   old_fig_units = get(fig,'Units');
   set(fig,'Units','pixels');
   fig_scale_pos = get(fig,'Position');
   fig_pos = fig_scale_pos;
   set(fig,'Units',old_fig_units);
else;
   old_paper_units = get(fig,'PaperUnits');
   set(fig,'PaperUnits','inches');
   fig_scale_pos = get(fig,'PaperPosition');
   fig_pos = get(fig,'Position');
   %
   % recompute from inches to pixels
   %
   fig_scale_pos = fig_scale_pos*get(0,'ScreenPixelsPerInch');
   set(fig,'PaperUnits',old_paper_units);
end;
x_scaling = gui_scale_pos(3)/fig_scale_pos(3);
y_scaling = gui_scale_pos(4)/fig_scale_pos(4);
%font_scaling = gui_scale_pos(3)/fig_pos(3);
font_scaling = x_scaling;

old_fig_units = get(fig,'Units');
old_paper_units = get(fig,'PaperUnits');
set(fig,'Units','points','PaperUnits','points');
fig_position_points = get(fig,'Position');
fig_paper_position = get(fig,'PaperPosition');
set(fig,'Units',old_fig_units,'PaperUnits',old_paper_units);
x_fig_scaling = fig_position_points(3)/fig_paper_position(3);
y_fig_scaling = fig_position_points(4)/fig_paper_position(4);

%
% axes objects to fix
%

these_axes = get(this_gui,'UserData');

axes_num = 0;
for i=1:size(myaxes,2);
   axes_num = axes_num+1;
   %
   % reposition axes-objects
   %
   old_axes_units = get(myaxes(1,i),'Units');
   set(myaxes(1,i),'Units','normalized');
   axes_pos = get(myaxes(1,i),'Position');
   %
   % deal with axes width and height
   %
   axes_pos(3) = gui_pos(3)*axes_pos(3);
   axes_pos(4) = gui_pos(4)*axes_pos(4);
   %
   % deal with left and bottom
   %
   axes_pos(1) = gui_pos(1)+(gui_pos(3)*axes_pos(1));
   axes_pos(2) = gui_pos(2)+(gui_pos(4)*axes_pos(2));
   set(these_axes(1,axes_num),'Position',axes_pos);
   %
   % deal with colors and sizes
   %
   ax_color = get(myaxes(1,i),'Color');
   ax_xcolor = get(myaxes(1,i),'XColor');
   ax_ycolor = get(myaxes(1,i),'YColor');
   ax_zcolor = get(myaxes(1,i),'ZColor');
   ax_fontsize = round(get(myaxes(1,i),'FontSize')*font_scaling);
   ax_linewidth = get(myaxes(1,i),'LineWidth')*x_scaling;
   my_title = get(myaxes(1,i),'Title');
   title_color = get(my_title,'Color');
   title_fontsize = round(get(my_title,'FontSize')*font_scaling);
   my_xlabel = get(myaxes(1,i),'XLabel');
   xlabel_color = get(my_xlabel,'Color');
   xlabel_fontsize = round(get(my_xlabel,'FontSize')*font_scaling);
   my_ylabel = get(myaxes(1,i),'YLabel');
   ylabel_color = get(my_ylabel,'Color');
   ylabel_fontsize = round(get(my_ylabel,'FontSize')*font_scaling);
   my_zlabel = get(myaxes(1,i),'ZLabel');
   zlabel_color = get(my_zlabel,'Color');
   zlabel_fontsize = round(get(my_zlabel,'FontSize')*font_scaling);
   if strcmp(inv_hardcopy,'on')==1;
      ax_color = [1 1 1];
      ax_xcolor = [0 0 0];
      ax_ycolor = [0 0 0];
      ax_zcolor = [0 0 0];
      title_color = [0 0 0];
      xlabel_color = [0 0 0];
      ylabel_color = [0 0 0];
      zlabel_color = [0 0 0];
   end;
   set(these_axes(1,axes_num),'FontSize',ax_fontsize,'Color',ax_color,'XColor',ax_xcolor,'YColor',ax_ycolor,'ZColor',ax_zcolor,'LineWidth',ax_linewidth);
   set(get(these_axes(1,axes_num),'Title'),'Color',title_color,'FontSize',title_fontsize);
   set(get(these_axes(1,axes_num),'XLabel'),'Color',xlabel_color,'FontSize',xlabel_fontsize);
   set(get(these_axes(1,axes_num),'YLabel'),'Color',ylabel_color,'FontSize',ylabel_fontsize);
   set(get(these_axes(1,axes_num),'ZLabel'),'Color',zlabel_color,'FontSize',zlabel_fontsize);
   %
   % dealing with children
   %
   kids = get(these_axes(1,axes_num),'Children')';
   for j=1:size(kids,2);
      ud_handle = get(kids(1,j),'UserData');
      if strcmp(get(kids(1,j),'Type'),'line')==1;
         line_color = get(ud_handle,'Color');
         marker_edge_color = get(ud_handle,'MarkerEdgeColor');
         marker_face_color = get(ud_handle,'MarkerFaceColor');
         if strcmp(inv_hardcopy,'on')==1;
            if isnumeric(line_color(1))==1;
               line_color = mean(line_color)*ones(1,3);
               if max(line_color)>1;
                  line_color = (1/255)*abs(line_color);
               end;
            end;
            if isnumeric(marker_edge_color(1))==1;
               marker_edge_color = mean(marker_edge_color)*ones(1,3);
               if max(marker_edge_color)>1;
                  marker_edge_color = (1/255)*abs(marker_edge_color);
               end;
            end;
            if isnumeric(marker_face_color)==1;
               marker_face_color = mean(marker_face_color)*ones(1,3);
               if max(marker_face_color)>1;
                  marker_face_color = (1/255)*abs(marker_face_color);
               end;
            end;
         end;
         set(kids(1,j),'Color',line_color,'MarkerEdgeColor',marker_edge_color,'MarkerFaceColor',marker_face_color,'LineWidth',get(ud_handle,'LineWidth')*x_scaling,'MarkerSize',max(round(get(ud_handle,'MarkerSize')*x_scaling),1));
      elseif strcmp(get(kids(1,j),'Type'),'patch')==1;
         patch_face_color = get(ud_handle,'FaceColor');
         patch_edge_color = get(ud_handle,'EdgeColor');
         patch_marker_face_color = get(ud_handle,'MarkerFaceColor');
         patch_marker_edge_color = get(ud_handle,'MarkerEdgeColor');
         if strcmp(inv_hardcopy,'on')==1;
            if isnumeric(patch_face_color(1))==1;
               patch_face_color = mean(patch_face_color)*ones(1,3);
            end;
            if isnumeric(patch_edge_color(1))==1;
               patch_edge_color = mean(patch_edge_color)*ones(1,3);
            end;
            if isnumeric(patch_marker_face_color(1))==1;
               patch_marker_face_color = mean(patch_marker_face_color)*ones(1,3);
            end;
            if isnumeric(patch_marker_edge_color(1))==1;
               patch_marker_edge_color = mean(patch_marker_edge_color)*ones(1,3);
            end;
         end;
         if isnumeric(patch_marker_face_color(1))==1;
            if patch_marker_face_color(1)>1;
               patch_marker_face_color = (1/255)*patch_marker_face_color;
            end;
         end;
         if isnumeric(patch_marker_edge_color(1))==1;
            if patch_marker_edge_color(1)>1;
               patch_marker_edge_color = (1/255)*patch_marker_edge_color;
            end;
         end;
         set(kids(1,j),'FaceColor',patch_face_color,'EdgeColor',patch_edge_color,'LineWidth',get(ud_handle,'LineWidth')*x_scaling,'MarkerSize',max(round(get(ud_handle,'MarkerSize')*x_scaling),1));
      elseif strcmp(get(kids(1,j),'Type'),'surface')==1;
         set(kids(1,j),'LineWidth',get(ud_handle,'LineWidth')*x_scaling,'MarkerSize',max(round(get(ud_handle,'MarkerSize')*x_scaling),1));
      elseif strcmp(get(kids(1,j),'Type'),'light')==1;
         light_color = get(ud_handle,'Color');
         if strcmp(inv_hardcopy,'on')==1;
            if isnumeric(light_color(1))==1;
               light_color = mean(light_color)*ones(1,3);
               if max(light_color)>1;
                  light_color = (1/255)*abs(light_color);
               end;
            end;
         end;
         set(kids(1,j),'Color',light_color);
      elseif strcmp(get(kids(1,j),'Type'),'text')==1;
         text_color = get(ud_handle,'Color');
         if strcmp(inv_hardcopy,'on')==1;
            text_color = mean(text_color)*ones(1,3);
         end;
         set(kids(1,j),'Color',text_color,'FontSize',round(get(ud_handle,'FontSize')*font_scaling));
      end;
   end;
   set(myaxes(1,i),'Units',old_axes_units);
end;

%
% deal with legends
%

for i=1:size(myaxes,2);
   if isempty(get(myaxes(1,i),'UserData'))==0;
      axes_num = axes_num+1;
      if ishandle(get(myaxes(1,i),'UserData'))==1;
         leg_handle = get(myaxes(1,i),'UserData');
      else;
         leg_handle = find_legend(myaxes(1,i));
      end;
      if (isempty(leg_handle)==0)&(strcmp(get(leg_handle,'Visible'),'on')==1);
         old_leg_units = get(leg_handle,'Units');
         set(leg_handle,'Units','normalized');
         leg_pos = get(leg_handle,'Position');
         real_leg_pos = leg_pos;
         %
         % deal with axes width and height
         %
         leg_pos(3) = gui_pos(3)*leg_pos(3);
         leg_pos(4) = gui_pos(4)*leg_pos(4);
         %
         % deal with left and bottom
         %
         leg_pos(1) = gui_pos(1)+(gui_pos(3)*leg_pos(1));
         leg_pos(2) = gui_pos(2)+(gui_pos(4)*leg_pos(2));
         %
         % resize legend based on x_scaling and y_scaling
         % need to determine relative position
         %
         leg_ud = get(leg_handle,'UserData');
         real_axes_handle = leg_ud.PlotHandle;
         old_real_axes_units = get(real_axes_handle,'Units');
         set(real_axes_handle,'Units','normalized');
         real_axes_pos = get(real_axes_handle,'Position');
         set(real_axes_handle,'Units',old_real_axes_units);
         %
         % now we can compare positions (treat outside as lower-left
         %
         if real_leg_pos(1)-real_axes_pos(1)<(real_axes_pos(1)+real_axes_pos(3))-(real_leg_pos(1)+real_leg_pos(3));
            left = 1;
         else;
            left = 0;
         end;
         if real_leg_pos(2)-real_axes_pos(2)<(real_axes_pos(2)+real_axes_pos(4))-(real_leg_pos(2)+real_leg_pos(4));
            lower_c = 1;
         else;
            lower_c = 0;
         end;
         %
         % now we adjust the size of leg_pos
         %
         if (left==1)&(lower_c==0);
            leg_pos(2) = leg_pos(2)+((1-y_fig_scaling)*leg_pos(4));
         elseif (left==0)&(lower_c==0);
            leg_pos(1) = leg_pos(1)+((1-x_fig_scaling)*leg_pos(3));
            leg_pos(2) = leg_pos(2)+((1-y_fig_scaling)*leg_pos(4));
         elseif (left==0)&(lower_c==1);
            leg_pos(1) = leg_pos(1)+((1-x_fig_scaling)*leg_pos(3));
         end;
         leg_pos(3) = x_fig_scaling*leg_pos(3);
         leg_pos(4) = y_fig_scaling*leg_pos(4);
         %
         % deal with colors in axes
         %
         leg_color = get(leg_handle,'Color');
         leg_xcolor = get(leg_handle,'XColor');
         leg_ycolor = get(leg_handle,'YColor');
         if strcmp(inv_hardcopy,'on')==1;
            leg_color = [1 1 1];
            leg_xcolor = [0 0 0];
            leg_ycolor = [0 0 0];
         end;
         set(these_axes(1,axes_num),'Position',leg_pos,'Color',leg_color,'XColor',leg_xcolor,'YColor',leg_ycolor);
         %
         % dealing with children
         %
         kids = get(these_axes(1,axes_num),'Children')';
         for j=1:size(kids,2);
            ud = get(kids(1,j),'UserData');
            if strcmp(get(kids(1,j),'Type'),'line')==1;
               line_color = get(ud.handle,'Color');
               if strcmp(inv_hardcopy,'on')==1;
                  line_color = mean(line_color)*ones(1,3);
               end;
               set(kids(1,j),'Color',line_color,'LineWidth',get(ud.handle,'LineWidth')*x_scaling,'MarkerSize',max(round(get(ud.handle,'MarkerSize')*x_scaling),1));
            elseif strcmp(get(kids(1,j),'Type'),'patch')==1;
               patch_face_color = get(ud.handle,'FaceColor');
               patch_edge_color = get(ud.handle,'EdgeColor');
               patch_marker_face_color = get(ud.handle,'MarkerFaceColor');
               patch_marker_edge_color = get(ud.handle,'MarkerEdgeColor');
               if strcmp(inv_hardcopy,'on')==1;
                  if isnumeric(patch_face_color(1))==1;
                     patch_face_color = mean(patch_face_color)*ones(1,3);
                  end;
                  if isnumeric(patch_edge_color(1))==1;
                     patch_edge_color = mean(patch_edge_color)*ones(1,3);
                  end;
                  if isnumeric(patch_marker_face_color(1))==1;
                     patch_marker_face_color = mean(patch_marker_face_color)*ones(1,3);
                  end;
                  if isnumeric(patch_marker_edge_color(1))==1;
                     patch_marker_edge_color = mean(patch_marker_edge_color)*ones(1,3);
                  end;
               end;
               if isnumeric(patch_marker_face_color(1))==1;
                  if patch_marker_face_color(1)>1;
                     patch_marker_face_color = (1/255)*patch_marker_face_color;
                  end;
               end;
               if isnumeric(patch_marker_edge_color(1))==1;
                  if patch_marker_edge_color(1)>1;
                     patch_marker_edge_color = (1/255)*patch_marker_edge_color;
                  end;
               end;
               set(kids(1,j),'FaceColor',patch_face_color,'EdgeColor',patch_edge_color,'MarkerFaceColor',patch_marker_face_color,'MarkerEdgeColor',patch_marker_edge_color);
               set(kids(1,j),'LineWidth',get(ud.handle,'LineWidth')*x_scaling,'MarkerSize',max(round(get(ud.handle,'MarkerSize')*x_scaling),1));
            elseif strcmp(get(kids(1,j),'Type'),'text')==1;
               text_color = get(ud.handle,'Color');
               if strcmp(inv_hardcopy,'on')==1;
                  text_color = mean(text_color)*ones(1,3);
               end;
               set(kids(1,j),'Color',text_color,'FontSize',round(get(ud.handle,'FontSize')*font_scaling));
            end;
         end;
         set(leg_handle,'Units',old_leg_units);
      end;
   end;
end;

%
% locate a potential colorbar
%

this_colorbar = findobj(fig,'Type','axes','Tag','colorbar')';

if isempty(this_colorbar)==0;
   if strcmp(get(this_colorbar,'Visible'),'on')==1;
      for i=1:size(this_colorbar,2);
         axes_num = axes_num+1;
         old_axes_units = get(this_colorbar(1,i),'Units');
         set(this_colorbar(1,i),'Units','normalized');
         axes_pos = get(this_colorbar(1,i),'Position');
         %
         % deal with axes width and height
         %
         axes_pos(3) = gui_pos(3)*axes_pos(3);
         axes_pos(4) = gui_pos(4)*axes_pos(4);
         %
         % deal with left and bottom
         %
         axes_pos(1) = gui_pos(1)+(gui_pos(3)*axes_pos(1));
         axes_pos(2) = gui_pos(2)+(gui_pos(4)*axes_pos(2));
         set(these_axes(1,axes_num),'Position',axes_pos);
         %
         % deal with colors and sizes
         %
         ax_color = get(this_colorbar(1,i),'Color');
         ax_xcolor = get(this_colorbar(1,i),'XColor');
         ax_ycolor = get(this_colorbar(1,i),'YColor');
         ax_zcolor = get(this_colorbar(1,i),'ZColor');
         ax_fontsize = round(get(this_colorbar(1,i),'FontSize')*font_scaling);
         my_title = get(this_colorbar(1,i),'Title');
         title_color = get(my_title,'Color');
         title_fontsize = round(get(my_title,'FontSize')*font_scaling);
         my_xlabel = get(this_colorbar(1,i),'XLabel');
         xlabel_color = get(my_xlabel,'Color');
         xlabel_fontsize = round(get(my_xlabel,'FontSize')*font_scaling);
         my_ylabel = get(this_colorbar(1,i),'YLabel');
         ylabel_color = get(my_ylabel,'Color');
         ylabel_fontsize = round(get(my_ylabel,'FontSize')*font_scaling);
         my_zlabel = get(this_colorbar(1,i),'ZLabel');
         zlabel_color = get(my_zlabel,'Color');
         zlabel_fontsize = round(get(my_zlabel,'FontSize')*font_scaling);
         if strcmp(inv_hardcopy,'on')==1;
            ax_color = [1 1 1];
            ax_xcolor = [0 0 0];
            ax_ycolor = [0 0 0];
            ax_zcolor = [0 0 0];
            title_color = [0 0 0];
            xlabel_color = [0 0 0];
            ylabel_color = [0 0 0];
            zlabel_color = [0 0 0];
         end;
         set(these_axes(1,axes_num),'FontSize',ax_fontsize,'Color',ax_color,'XColor',ax_xcolor,'YColor',ax_ycolor,'ZColor',ax_zcolor);
         set(get(these_axes(1,axes_num),'Title'),'Color',title_color,'FontSize',title_fontsize);
         set(get(these_axes(1,axes_num),'XLabel'),'Color',xlabel_color,'FontSize',xlabel_fontsize);
         set(get(these_axes(1,axes_num),'YLabel'),'Color',ylabel_color,'FontSize',ylabel_fontsize);
         set(get(these_axes(1,axes_num),'ZLabel'),'Color',zlabel_color,'FontSize',zlabel_fontsize);
         set(this_colorbar(1,i),'Units',old_axes_units);
      end;
   end;
end;

set(this_gui,'Units',old_gui_units);

%
% end of move_preview_plots_fcn.m
%