function [out,abort,res,h_init] = ms_test(Y,X,theta,Omega,alt_model,parm_vec,h_val,rho_low,rho_upp,rho_size,h_start,waitbar,show_clock,show_waitdlg)

%#function carr_hu_plob_max carr_hu_plob_con

%
% computes the Carrasco, Hu, and Ploberger test
%
% Procedure: draw h uniformly from the unit sphere h_val times and select rho from an equispaced grid
% when h_val = 1, the function determines h through numerical optimization (best approach, but can be slower)
%

h_first = [];
out = [];
abort = '';
if nargout>2;
   res = [];
end;
[n T] = size(Y);

%
% compute the first and second order partials of the likelihood function
%

if isempty(X)==0;
   d = size(X,1);
   eta = Y-(theta*X);
else;
   eta = Y;
   d = 0;
end;
%
% parameter selection matrices
%
sel_mat = [];
if alt_model(1,1)==1;
   sel_mat = [eye(parm_vec(1,1)) zeros(parm_vec(1,1),parm_vec(1,2)) zeros(parm_vec(1,1),parm_vec(1,3))];
end;
if alt_model(1,2)==1;
   sel_mat = [sel_mat;[zeros(parm_vec(1,2),parm_vec(1,1)) eye(parm_vec(1,2)) zeros(parm_vec(1,2),parm_vec(1,3))]];
end;
if alt_model(1,3)==1;
   sel_mat = [sel_mat;[zeros(parm_vec(1,3),parm_vec(1,1)) zeros(parm_vec(1,3),parm_vec(1,2)) eye(parm_vec(1,3))]];
end;

Omega_inv = inverse(Omega);
Dn = duplication(n);
%
% compute the dimension of h
%
nump = size(sel_mat,1);
mu_1 = zeros((n*d)+size(Dn,2),T);
for t=1:T;
   fo(t).omega = (1/2)*Dn'*kron(Omega_inv,Omega_inv)*vec((eta(:,t)*eta(:,t)')-Omega);
   so(t).omega = -Dn'*kron(Omega_inv,((Omega_inv*eta(:,t)*eta(:,t)')-((1/2)*eye(n)))*Omega_inv)*Dn;
   if d>0;
      fo(t).theta = kron(eye(d),Omega_inv)*vec(eta(:,t)*X(:,t)');
      so(t).theta = -kron(X(:,t)*X(:,t)',Omega_inv);
      so(t).omega_theta = -2*Dn'*kron(Omega_inv*eta(:,t)*X(:,t)',Omega_inv);
      %
      fo(t).theta = [fo(t).theta;fo(t).omega];
      so(t).theta = [[so(t).theta so(t).omega_theta'];[so(t).omega_theta so(t).omega]];
   else;
      fo(t).theta = fo(t).omega;
      so(t).theta = so(t).omega;
   end;
   mu_1(:,t) = fo(t).theta;
end;
%
% setup for numerical optimization
%
curr_ini = find_inisettings;
if h_val==1;
   if curr_ini.convcrit<10;
      ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
   else;
      ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
   end;
   ccrit = mystr2num(ch_convcrit);
   options = optimset('LargeScale','off','TolCon',ccrit,'TolX',ccrit,'TolFun',ccrit,'MaxIter',curr_ini.maxit,'MaxFunEvals',100*curr_ini.maxit,'GradObj','off','GradConstr','on');
end;
if (waitbar==1)|(show_waitdlg==1);
   setlogtext(strvcat(' ','Computing the Markov switching test...',['   Started at:   'datestr(now,14)]));
end;
if waitbar==1;
   if nump>1;
      input_var.title = ['Progress for ' int2str(rho_size*h_val) ' trials'];
   else;
      input_var.title = ['Progress for ' int2str(rho_size) ' trials'];
   end;
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 0;
   if show_clock==1;
      input_var.clock = 1;
   else;
      input_var.clock = 0;
   end;
   input_var.label = 'Trial number:';
   input_var.name = 'Markov Switching Test';
   mywait = progressdlg(0,input_var);
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
elseif (waitbar==0)&(show_waitdlg==1);
   txt = ['Computing the Markov Switching Test of Carrasco, Hu and Ploberger. Computation started at: ' datestr(now,14)];
   mywait = waitdlg(strvcat(' ',txt),'information','Markov Switching Test',450,100);
   waitf = get(mywait,'UserData');
end;
%
% loop over all rho in the grid
%
rho = rho_low;
if nump>1;
   supTS_mat = zeros(rho_size,1);
   n_times = 1;
   rho_times = 1;
   while rho<=rho_upp;
      if (n_times==1)&(waitbar==1);
         progressdlg([(1/(rho_size*h_val)) 1]);
      end;
      if strcmp(abort,'cancel')==1;
         break;
      end;
      if (waitbar==1)|(show_waitdlg==1);
         setlogtext(['     * rho = ' num2str(rho,'%0.4f')]);
      end;
      %
      % compute terms structure based on first and second order derivatives
      %
      for t=1:T;
         if n_times==1;
            terms(t).term_1 = sel_mat*so(t).theta*sel_mat';
            terms(t).term_2 = sel_mat*fo(t).theta*fo(t).theta'*sel_mat';
         end;
         if (t>1)&(isalmostequal(rho,0)==0);
            rho_vec = (1:1:(t-1))*log(rho);
            rho_vec = exp(reverse(rho_vec));
            terms(t).term_3 = sel_mat*kron( rho_vec*eye(t-1), mu_1(:,t) )*mu_1(:,1:t-1)'*sel_mat';
         else;
            terms(t).term_3 = 0;
         end;
      end;
      %
      %
      %
      if h_val>1;
         supTS_temp_mat = zeros(h_val,1);
         for i=1:h_val;
            if waitbar==1;
               abort = get(mywait,'UserData');
               if (strcmp(abort,'cancel')==1);
                  break;
               else;
                  progressdlg([(n_times/(rho_size*h_val)) n_times]);
               end;
            end;
            %
            % generate h from the unit sphere
            %
            h = rand(nump,1);
            h = (1/sqrt(sum(h.*h)))*h;
            %
            % create the mu_2 matrix
            %
            mu_2 = zeros(1,T);
            for t=1:T;
               mu_2(1,t) = h'*( terms(t).term_1 + terms(t).term_2 + (2*terms(t).term_3) )*h;
            end;
            %
            % regress mu_2 on mu_1
            %
            gamma = (1/2)*(mu_2*mu_1')*inverse(mu_1*mu_1');
            epsilon = ((1/2)*mu_2)-(gamma*mu_1);
            GammaT = (1/(2*sqrt(T)))*sum(mu_2,2);
            denomT = sqrt((1/T)*epsilon*epsilon');
            supTS = GammaT/denomT;
            supTS_temp_mat(i,1) = (1/2)*(max(0,real(supTS))^2);
%            supTS_mat(n_times,1) = (1/2)*(max(0,real(supTS))^2);
            n_times = n_times+1;
         end;
         supTS_mat(rho_times,1) = max(supTS_temp_mat);
      else;
         if waitbar==1;
            abort = get(mywait,'UserData');
            if (strcmp(abort,'cancel')==1);
               break;
            else;
               progressdlg([(n_times/(rho_size*h_val)) n_times]);
            end;
         end;
         %
         % numerical optimization to obtain h vector
         %
         if n_times==1;
            if isempty(h_start)==1;
               rand('state',1);
               h = rand(nump,1);
            else;
               h = h_start;
            end;
         end;
         h = (1/sqrt(sum(h.*h)))*h;
         %
         % create the mu_2 matrix
         %
         mu_2_inner = zeros(nump,T*nump);
   %      mu_2_inner_prime = zeros(nump,T*nump);
         for t=1:T;
            mu_2_inner(:,1+(nump*(t-1)):nump*t) = terms(t).term_1 + terms(t).term_2 + (2*terms(t).term_3);
   %         mu_2_inner_prime(:,1+(nump*(t-1)):nump*t) = terms(t).term_1' + terms(t).term_2' + (2*terms(t).term_3');
         end;
         [h,chp_val,exitflag,output,lambda,grad] = fmincon_fcn('carr_hu_plob_max',h,[],[],[],[],[],[],'carr_hu_plob_con',options,0,'',mu_1,mu_2_inner);
         if ((waitbar==1)|(show_waitdlg==1))&(exitflag==0);
            %
            % try the optimization routine one more time
            %
            setlogtext('       Optmization didn''t converge. Making another try...');
            [h,chp_val,exitflag,output,lambda,grad] = fmincon_fcn('carr_hu_plob_max',h,[],[],[],[],[],[],'carr_hu_plob_con',options,0,'',mu_1,mu_2_inner);
            if exitflag>0;
               setlogtext('       Re-optimization successful!');
            else;
               setlogtext('       Re-optmization didn''t converge.');
            end;
         end;
         if n_times==1;
            h_first = h;
         end;
         supTS_mat(n_times,1) = (1/2)*(max(0,-chp_val)^2);
         n_times = n_times+1;
      end;
      rho = rho+0.025;
      rho_times = rho_times+1;
   end;
else;
   supTS_mat = zeros(rho_size,1);
   n_times = 1;
   while rho<=rho_upp;
      if waitbar==1;
         progressdlg([(n_times/rho_size) n_times]);
      end;
      if strcmp(abort,'cancel')==1;
         break;
      end;
      %
      % compute terms structure based on first and second order derivatives
      %
      for t=1:T;
         if n_times==1;
            terms(t).term_1 = sel_mat*so(t).theta*sel_mat';
            terms(t).term_2 = sel_mat*fo(t).theta*fo(t).theta'*sel_mat';
         end;
         if (t>1)&(isalmostequal(rho,0)==0);
            rho_vec = (1:1:(t-1))*log(rho);
            rho_vec = exp(reverse(rho_vec));
            terms(t).term_3 = sel_mat*kron( rho_vec*eye(t-1), mu_1(:,t) )*mu_1(:,1:t-1)'*sel_mat';
         else;
            terms(t).term_3 = 0;
         end;
      end;
      mu_2 = zeros(1,T);
      for t=1:T;
         mu_2(1,t) = terms(t).term_1 + terms(t).term_2 + (2*terms(t).term_3);
      end;
      %
      % regress mu_2 on mu_1
      %
      gamma = (1/2)*(mu_2*mu_1')*inverse(mu_1*mu_1');
      epsilon = ((1/2)*mu_2)-(gamma*mu_1);
      GammaT = (1/(2*sqrt(T)))*sum(mu_2,2);
      denomT = sqrt((1/T)*epsilon*epsilon');
      supTS = GammaT/denomT;
      supTS_mat(n_times,1) = (1/2)*(max(0,real(supTS))^2);
      n_times = n_times+1;
      rho = rho+0.025;
   end;
end;
if (waitbar==1)|(show_waitdlg==1);
   if strcmp(abort,'cancel')==0;
      setlogtext(['   Completed at: 'datestr(now,14)]);
   else;
      setlogtext('   Aborted');
   end;
end;

if waitbar==1;
   if ishandle(mywait)==1;
      set(mywait,'UserData','done');
      close(mywait);
      drawnow;
   end;
elseif (waitbar==0)&(show_waitdlg==1);
   set(waitf.text,'UserData','done');
   delete(mywait);
   drawnow;
end;

out = max(supTS_mat);

if nargout>2;
   if rho_size==1;
      res = [rho_low supTS_mat];
   else;
      res = [(rho_low:0.025:rho_upp)' supTS_mat];
   end;
   if nargout>3;
      h_init = h_first;
   end;
end;

%
% end of ms_test.m
%
