function [d_mu,d_sigma,mu,sigma] = prior_short_coint(tm)

%
% constructs the mean and the covariance matrix for vec(Psi)
% for the error correction specification based on the tm structure containing the
% model descriptors
%
d_mu = [];
d_sigma = [];

if (tm.d>0)&(tm.prior_inf_deter==1);
   d_mu = zeros(tm.d*tm.n,1);
   d_sigma = (tm.lambda_d^2)*eye(tm.d);
end;

mu = [];
sigma = [];

if tm.prior_inf_shortrun==2;
   return;
end;

mu = zeros((size(tm.Z2,1)-tm.d)*tm.n,1);

if tm.p>1;
   %
   % lag length shrinkage
   %
   K = zeros(tm.p-1,tm.p-1);
   for i=1:(tm.p-1);
      for j=1:(tm.p-1);
         if i==j;
            K(i,i) = i^(2*tm.lambda_l);
         else;
            K(i,j) = (i^(tm.lambda_l))*(j^(tm.lambda_l))*(abs(i-j)^2);
         end;
      end;
   end;
   %
   % correlation shrinkage
   %
   K_2 = tm.lambda_theta*ones(tm.p-1,tm.p-1);
   for i=1:tm.p-1;
      K_2(i,i) = 1;
   end;
   %
   % baseline shrinkage
   %
   K = (tm.lambda_b^2)*(K_2 ./ K);
   sigma = kron(K,eye(tm.n));
end;

if (tm.q1>0)&(tm.pe>0);
   %
   % lag length shrinkage
   %
   K = zeros(tm.pe,tm.pe);
   for i=1:tm.pe;
      for j=1:tm.pe;
         if i==j;
            K(i,i) = i^(2*tm.lambda_l);
         else;
            K(i,j) = (i^(tm.lambda_l))*(j^(tm.lambda_l))*(abs(i-j)^2);
         end;
      end;
   end;
   %
   % correlation shrinkage
   %
   K_2 = tm.lambda_theta*ones(tm.pe,tm.pe);
   for i=1:tm.pe;
      K_2(i,i) = 1;
   end;
   %
   % baseline and exogeneity shrinkage
   %
   K = (tm.lambda_b^2)*(tm.lambda_e^2)*(K_2 ./ K);
   sigma_q1 = kron(K,eye(tm.q1));
   if tm.p>1;
      sigma = [[sigma zeros(size(sigma,1),size(sigma_q1,1))];[zeros(size(sigma_q1,1),size(sigma,1)) sigma_q1]];
   else;
      sigma = sigma_q1;
   end;
end;

if tm.q0>0;
   %
   % lag length shrinkage
   %
   K = zeros(tm.pe+1,tm.pe+1);
   for i=1:tm.pe+1;
      for j=1:tm.pe+1;
         if i==j;
            K(i,i) = i^(2*tm.lambda_l);
         else;
            K(i,j) = (i^(tm.lambda_l))*(j^(tm.lambda_l))*(abs(i-j)^2);
         end;
      end;
   end;
   %
   % correlation shrinkage
   %
   K_2 = tm.lambda_theta*ones(tm.pe+1,tm.pe+1);
   for i=1:tm.pe+1;
      K_2(i,i) = 1;
   end;
   %
   % baseline and exogeneity shrinkage
   %
   K = (tm.lambda_b^2)*(tm.lambda_e^2)*(K_2 ./ K);
   sigma_q0 = kron(K,eye(tm.q0));
   if (tm.p>1)|((tm.q1>0)&(tm.pe>0));
      sigma = [[sigma zeros(size(sigma,1),size(sigma_q0,1))];[zeros(size(sigma_q0,1),size(sigma,1)) sigma_q0]];
   else;
      sigma = sigma_q0;
   end;
end;

%
% end of prior_short_coint.m
%
