function rotate3d_fcn(arg,arg2)
%ROTATE3D Interactively rotate the view of a 3-D plot.
%   ROTATE3D ON turns on mouse-based 3-D rotation.
%   ROTATE3D OFF turns if off.
%   ROTATE3D by itself toggles the state.
%
%   ROTATE3D(FIG,...) works on the figure FIG.
%
%   Double click to restore the original view.
%
%   See also ZOOM.

%   rotate3d on enables  text feedback
%   rotate3d ON disables text feedback.

%   Revised by Rick Paxson 10-25-96
%   Clay M. Thompson 5-3-94
%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 1.35 $  $Date: 1998/08/21 20:02:05 $

if(nargin == 0)
   setState(gcf,'toggle');
elseif nargin==1
   if ishandle(arg)
      setState(arg,'toggle');
   else
      switch(lower(arg)) % how much performance hit here
      case 'motion'
         rotaMotionFcn;
      case 'down'
         c_pointer = get(gcf,'Pointer');
         c_pointer_shape_cdata = get(gcf,'PointerShapeCData');
         this_colorbar = findobj(gcf,'Type','axes','Tag','colorbar')';
         if isempty(this_colorbar)==0;
            if strcmp(get(this_colorbar,'Visible'),'on')==1;
               for i=1:size(this_colorbar,2);
                  if gca==this_colorbar(1,i);
                     set(gcf,'Pointer',c_pointer,'PointerShapeCData',c_pointer_shape_cdata);
                     return;
                  end;
               end;
            end;
         end;
         if (isempty(get(gcf,'CurrentObject'))==1)|(strcmp(get(get(gcf,'CurrentObject'),'Type'),'figure')==1);
            set(gcf,'Pointer',c_pointer,'PointerShapeCData',c_pointer_shape_cdata);
            return;
         end;
         toolbar = get(gcf,'UserData');
         cp = get(gcf,'CurrentPoint');
%         pos = get(toolbar.addtext,'Position');
%         if cp(2)>=pos(2)-6;
%            set(gcf,'Pointer',c_pointer,'PointerShapeCData',c_pointer_shape_cdata);
%            return;
%         end;
         set(gcf,'Pointer','custom','PointerShapeCData',handdown_pointer);
         rotaButtonDownFcn;
      case 'up'
         rotaButtonUpFcn;
         set(gcf,'Pointer','custom','PointerShapeCData',hand_pointer,'WindowButtonMotionFcn','pointer_rotate');
      case 'on'
         setState(gcf,arg);
      case 'off'
         setState(gcf,arg);
      otherwise
         error('Unknown action string.');
      end
   end
elseif nargin==2
   if ~ishandle(arg), error('Unknown figure.'); end
   switch(lower(arg2)) % how much performance hit here
   case 'on'
      setState(arg,arg2);
   case 'off'
      setState(arg,arg2);
   otherwise
      error('Unknown action string.');
   end
end

%
% end of rotate3d_fcn.m
%
