%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for setting sample dates              %
%                                                                %
%          Function: sampledates                                 %
%                                                                %
%          Input:                                                %
%                       samplebeg: first period (real)           %
%                       sampleend: last period (real)            %
%                       p: lag length                            %
%                       freq: data frequence string (y,q,m)      %
%                                                                %
%          Output:                                               %
%                       initfirst: first initial period          %
%                       initlast: last initial period            %
%                       samplefirst: first sample period         %
%                       samplelast: last sample period           %
%                                                                %
%          This version:        August 9, 2002                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2002          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [initfirst,initlast,samplefirst,samplelast] = sampledates(samplebeg,sampleend,p,freq)

if strcmp(freq,'m')==1;
   initfirst = samplebeg-(1/24);
   myfirst = 24*(initfirst-floor(initfirst));
   myfirst = (myfirst+1)/2;
   initfirst = [num2str(floor(initfirst)) ':' num2str(myfirst)];
   %
   initlast = samplebeg-1/24+((p-1)/12);
   mylast = 24*(initlast-floor(initlast));
   mylast = (mylast+1)/2;
   initlast = [num2str(floor(initlast)) ':' num2str(mylast)];
   %
   samplefirst = samplebeg-(1/24)+(p/12);
   myfirst = 24*(samplefirst-floor(samplefirst));
   myfirst = (myfirst+1)/2;
   samplefirst = [num2str(floor(samplefirst)) ':' num2str(myfirst)];
   %
   samplelast = sampleend-(1/24);
   mylast = 24*(samplelast-floor(samplelast));
   mylast = (mylast+1)/2;
   samplelast = [num2str(floor(samplelast)) ':' num2str(mylast)];

elseif strcmp(freq,'q')==1;
   initfirst = samplebeg-(1/8);
   myfirst = 8*(initfirst-floor(initfirst));
   myfirst = (myfirst+1)/2;
   initfirst = [num2str(floor(initfirst)) ':' num2str(myfirst)];
   %
   initlast = samplebeg-1/8+((p-1)/4);
   mylast = 8*(initlast-floor(initlast));
   mylast = (mylast+1)/2;
   initlast = [num2str(floor(initlast)) ':' num2str(mylast)];
   %
   samplefirst = samplebeg-(1/8)+(p/4);
   myfirst = 8*(samplefirst-floor(samplefirst));
   myfirst = (myfirst+1)/2;
   samplefirst = [num2str(floor(samplefirst)) ':' num2str(myfirst)];
   %
   samplelast = sampleend-(1/8);
   mylast = 8*(samplelast-floor(samplelast));
   mylast = (mylast+1)/2;
   samplelast = [num2str(floor(samplelast)) ':' num2str(mylast)];
else;
   initfirst = [num2str(samplebeg) ':1'];
   initlast = [num2str(samplebeg+p-1) ':1'];
   samplefirst = [num2str(samplebeg+p) ':1'];
   samplelast = [num2str(sampleend) ':1'];
end;

%
% end of sampledates.m
%
