%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for saving identifying restrictions %
%          to file                                               %
%                                                                %
%          Function: save_struc_restgui                          %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       cm: structure with model info            %
%                       type_var: string, either short_run,      %
%                             long_run, or c_matrix              %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        March 18, 2006                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = save_struc_restgui(selector,cm,type_var)

if strcmp(selector,'init')==1;
   tm = cm;
   tm.type = type_var;
   curr_ini = find_inisettings;
   %
   thisexit = 0;
   mycons = getstoredimages;
   bgcolor = get(0,'defaultuicontrolbackgroundcolor');
   mybrowse = maketrans(mycons.browse_image,[160 160 164],bgcolor);
   tm.restfile = cm.modelsdir;
   if strcmp(tm.type,'short_run')==1;
      tm.k = tm.n;
      tm.parameter = 'B';
   elseif strcmp(tm.type,'long_run')==1;
      if tm.r<tm.n;
         tm.k = tm.n-tm.r;
      else;
         tm.k = tm.n;
      end;
      tm.parameter = 'A';
   else;
      if tm.r<tm.n;
         tm.k = tm.n-tm.r;
      else;
         tm.k = tm.n;
      end;
      tm.parameter = 'C';
   end;
else;
   thisexit = 1;
%   save_struc_restdlg = get(0,'CurrentFigure');
   save_struc_restdlg = findobj('Type','figure','Tag','SaveIdentRest');
   save_struc_rest = get(save_struc_restdlg,'UserData');
   tm = get(save_struc_rest.restfile,'UserData');
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      save_struc_restdlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','save_struc_restgui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-512)/2 (realscrsz(4)-395)/2 512 395], ...
         'Tag','SaveIdentRest', ...
         'Resize','off', ...
         'Name','Save Alpha Restrictions', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','save_struc_restgui exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(save_struc_restdlg,'DockControl','off');
      end;

      %
      % Outer axis
      %
      if strcmp(tm.type,'short_run')==1;
         axesbox([2 2 510 385],'Restrictions on B',80,[0.5 1]);
         set(save_struc_restdlg,'Name','Save Contemporaneous Restrictions');
      elseif strcmp(tm.type,'long_run')==1;
         axesbox([2 2 510 385],'Restrictions on A = C*B',110,[0.5 1]);
         set(save_struc_restdlg,'Name','Save Long-Run Restrictions');
      else;
         axesbox([2 2 510 385],'Restrictions on C',80,[0.5 1]);
         set(save_struc_restdlg,'Name','Save C Restrictions');
      end;
      %
      % GUI Header
      %

      mycons = getstoredimages;


      if strcmp(tm.type,'short_run')==1;
         header_txt = 'Save Contemporaneous Restrictions File';
         desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. When you''re satisfied it''s the one you wish to save, press "Save" to save the restriction with another filename. Press "Done" to finish.';
         dialog_header_fcn([14 300 484 72],mirrorimage(maketrans(mycons.save_srbig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);
      elseif strcmp(tm.type,'long_run')==1;
         header_txt = 'Save Long-Run Restrictions File';
         desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. When you''re satisfied it''s the one you wish to save, press "Save" to save the restriction with another filename. Press "Done" to finish.';
         dialog_header_fcn([14 300 484 72],mirrorimage(maketrans(mycons.save_lrbig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);
      else;
         header_txt = 'Save C Restrictions File';
         desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. When you''re satisfied it''s the one you wish to save, press "Save" to save the restriction with another filename. Press "Done" to finish.';
         dialog_header_fcn([14 300 484 72],mirrorimage(maketrans(mycons.save_cbig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);
      end;
      desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. When you''re satisfied it''s the one you wish to save, press "Save" to save the restriction with another filename. Press "Done" to finish.';

      %
      % Outer axis for showing restrictions file
      %

      axesbox([14 100 484 182],'The Restrictions',80,[0.5 1]);

      %
      % Outer axis for restrictions file
      %

      axesbox([14 14 380 73],'Restrictions Selection',105,[0.5 1]);

      %
      % Listbox to show restrictions on alpha
      %

      save_struc_rest.gmat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 116 452 150], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Max',2, ...
         'Value',[], ...
         'String','', ...
         'Enable','inactive', ...
         'UserData','working', ...
         'HorizontalAlignment','center', ...
         'Tag','BetaListbox');

      %
      % Show restrictions file
      %

      save_struc_rest.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[30 50 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','text', ...
         'HorizontalAlignment','left', ...
      	 'String','Restrictions file:', ...
      	 'Tag','ShowRest');

      save_struc_rest.restfile = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[30 30 295 22], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','edit', ...
         'HorizontalAlignment','left', ...
         'String',tm.modelsdir, ...
         'Callback','save_struc_restgui checkrest', ...
         'Enable','on', ...
         'Tag','RestFile');

      %
      % Button for selecting output file
      %

      save_struc_rest.restbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[330 30 50 22], ...
         'TooltipString','Browse', ...
         'CData',mybrowse, ...
         'Callback','save_struc_restgui openrest', ...
         'Enable','on', ...
         'Tag','RestButton');

      %
      % Save button
      %

      save_struc_rest.save = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 48 90 20], ...
         'String','Save', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','save_struc_restgui save', ...
         'Enable','off', ...
         'UserData','waiting', ...
         'Tag','SaveButton');

      %
      %  Done button
      %

      save_struc_rest.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 14 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','save_struc_restgui done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(save_struc_restdlg,'UserData',save_struc_rest,...
         'HandleVisibility','callback');
   case 'openrest'
      curr_ini = find_inisettings;
      if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
         newrestfile = get(save_struc_rest.restfile,'String');
         basedir = getpath(newrestfile);
         file_fltr = 'VAR Restrictions Files (*.rest)|*.rest|';
         best_pos = 1;
         title_txt = 'Select file or enter filename';
         [newrestfile,restpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
      else;
         newrestfile = get(save_struc_rest.restfile,'String');
         newrestfile = getpath(newrestfile);
         newrestfile = strcat(newrestfile,'*.rest');
         [newrestfile,restpath] = uigetfile(newrestfile, 'Select file or enter filename');
      end;
      drawnow;
      if isnumeric(newrestfile)==0;
         newrestfile = strcat(restpath,newrestfile);
         if fileexist(newrestfile)==1;
            gvar = loaddata(newrestfile);
            [gv1 gv2] = size(gvar);
            [Rmat,rvec] = rmatrix(gvar,tm.n,tm.n);
            Rmat = Rmat(1:size(Rmat,1),1:tm.n*tm.n);
            check_rank = (rank(Rmat)==gv1);
            if (strcmp(tm.type,'short_run')==0)&(tm.r<tm.n);
               %
               % need to make sure that any restrictions on beta are satisfied
               % and do not make restrictions on C*B redundant
               %
               if tm.restrict(1,2)==1;
                  %
                  % create a random beta which satisfies the selected restrictions
                  %
                  beta = [];
                  for i=1:tm.r;
                     file = strcat(tm.modelsdir,'H_',num2str(i),'-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
                     Hmat = loaddata(file);
                     beta = [beta Hmat*randn(size(Hmat,2),1)];
                  end;
                  %
                  % now we normalize
                  %
                  file = strcat(tm.modelsdir,'normalize-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.norm');
                  h_vec = loaddata(file);
                  for i=1:tm.r;
                     norm_var = h_vec(i,1);
                     normbetai = beta(norm_var,i);
                     if abs(normbetai)<1.0e-006;
                        normbetai = 1;
                     end;
                     for j=1:size(beta,1);
                        beta(j,i) = beta(j,i)/normbetai;
                     end;
                  end;
                  beta = beta(1:tm.n,:);
                  beta_orth = null(beta','r');
               else;
                  beta_orth = randn(tm.n,tm.n-tm.r);
               end;
               old_Rmat = Rmat;
               Rmat = Rmat*kron(eye(tm.n),beta_orth);
            end;
            if rank(Rmat)==gv1;
               checkfmat = null(Rmat,'r');
               checkfmat = checkfmat*randn(size(checkfmat,2),1);
               checkfmat = checkfmat+(Rmat'*inverse(Rmat*Rmat')*rvec);
               if (strcmp(tm.type,'short_run')==1)|(tm.r==tm.n);
                  checkfmat = vecinv(checkfmat,tm.n);
               else;
                  checkfmat = vecinv(checkfmat,tm.n-tm.r);
                  checkfmat = beta_orth*checkfmat;
                  new_Rmat = Rmat;
                  Rmat = old_Rmat;
               end;
               if rank(checkfmat)==tm.k;
                  %
                  % load restrictions into gui
                  %
                  tm.restfile = newrestfile;
                  set(save_struc_rest.restfile,'String',tm.restfile);
                  set(save_struc_rest.save,'Enable','on');
                  myrest = ' ';
                  for i=1:gv1;
                     thisrest = '';
                     k = 1;
                     l = 1;
                     for j=1:tm.n*tm.n;
                        if abs(Rmat(i,j))>0;
                           if Rmat(i,j)==1;
                              if strcmp(thisrest,'')==1;
                                 thisrest = ['   ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              else;
                                 thisrest = [thisrest ' + ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              end;
                           elseif Rmat(i,j)==-1;
                              if strcmp(thisrest,'')==1;
                                 thisrest = ['   - ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              else;
                                 thisrest = [thisrest ' - ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              end;
                           else;
                              if Rmat(i,j)>0;
                                 if strcmp(thisrest,'')==1;
                                    thisrest = ['   ' num2str(Rmat(i,j),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                                 else;
                                    thisrest = [thisrest ' + ' num2str(Rmat(i,j),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                                 end;
                              else;
                                 if strcmp(thisrest,'')==1;
                                    thisrest = ['   - ' num2str(abs(Rmat(i,j)),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                                 else;
                                    thisrest = [thisrest ' - ' num2str(abs(Rmat(i,j)),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                                 end;
                              end;
                           end
                        end;
                        if k==floor(j/tm.n);
                           k = k+1;
                           l = 1;
                        else;
                           l=l+1;
                        end;
                     end;
                     if abs(rvec(i,1))==0;
                        thisrest = [thisrest ' = 0'];
                     else;
                        thisrest = [thisrest ' = ' num2str(rvec(i,1),'%1.2f')];
                     end;
                     myrest = strvcat(myrest,thisrest);
                     set(save_struc_rest.gmat,'String',myrest);
                  end;
                  set(save_struc_rest.gmat,'String',strvcat(myrest,' '));
               else;
                  if strcmp(tm.type,'short_run')==1;
                     txt = ['The file you selected implies that B has rank ' num2str(rank(checkfmat)) ', i.e. less than the covariance matrix of the residuals. Please select another file!'];
                     about(txt,'warning','Restrictions on B - Information',120,350);
                  elseif strcmp(tm.type,'long_run')==1;
                     txt = ['The file you selected implies that A=C*B has rank ' num2str(rank(checkfmat)) ', i.e. less than ' num2str(tm.n-tm.r,'%0.0f') ', the rank of the C matrix. Since B must be of full rank, this is not allowed. Please select another file!'];
                     about(txt,'warning','Restrictions on A=C*B - Information',120,350);
                  else;
                     txt = ['The file you selected implies that C has rank ' num2str(rank(checkfmat)) ', i.e. less than ' num2str(tm.n-tm.r,'%0.0f') '. Please select another file!'];
                     about(txt,'warning','Restrictions on C - Information',120,350);
                  end;
               end;
            else;
               if rank(Rmat)>=1;
                  if strcmp(tm.type,'short_run')==1;
                     txt = ['The ' num2str(gv1) ' restrictions you have selected for B are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(Rmat)) '. Please select another file'];
                  elseif strcmp(tm.type,'long_run')==1;
                     if (check_rank==1)&(tm.r<tm.n);
                        add_text = ' This is probably due to an inconsistency between your selected restrictions and the beta matrix.';
                     else;
                        add_text = '';
                     end;
                     txt = ['The ' num2str(gv1) ' restrictions you have selected for A=C*B are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(new_Rmat)) '.' add_text ' Please select another file'];
                  else;
                     txt = ['The ' num2str(gv1) ' restrictions you have selected for C are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(Rmat)) '. Please select another file'];
                  end;
               else;
                  txt = ['The file you selected, ' getname(newrestfile) ', doesn''t contain any restrictions. Please select another file!'];
               end;
               if strcmp(tm.type,'short_run')==1;
                  about(txt,'warning','Restrictions on B - Information',120,350);
               elseif strcmp(tm.type,'long_run')==1;
                  about(txt,'warning','Restrictions on A=C*B - Information',150,350);
               else;
                  about(txt,'warning','Restrictions on C - Information',120,350);
               end;
            end;
         else;
            set(save_struc_rest.restfile,'String',tm.restfile);
            if fileexist(tm.restfile)==1;
               set(save_struc_rest.save,'Enable','on');
            else;
               set(save_struc_rest.save,'Enable','off');
            end;
         end;
      else;
         set(save_struc_rest.restfile,'String',tm.restfile);
         if fileexist(tm.restfile)==1;
            set(save_struc_rest.save,'Enable','on');
         else;
            set(save_struc_rest.save,'Enable','off');
         end;
      end;
      set(save_struc_restdlg,'UserData',save_struc_rest);
   case 'checkrest'
      newrestfile = get(save_struc_rest.restfile,'String');
      if fileexist(newrestfile)==0;
         set(save_struc_rest.restfile,'String',tm.restfile);
         if fileexist(tm.restfile)==1;
            set(save_struc_rest.save,'Enable','on');
         else;
            set(save_struc_rest.save,'Enable','off');
         end;
      else;
         gvar = loaddata(newrestfile);
         [gv1 gv2] = size(gvar);
         [Rmat,rvec] = rmatrix(gvar,tm.n,tm.n);
         Rmat = Rmat(1:size(Rmat,1),1:tm.n*tm.n);
         check_rank = (rank(Rmat)==gv1);
         if (strcmp(tm.type,'short_run')==0)&(tm.r<tm.n);
            %
            % need to make sure that any restrictions on beta are satisfied
            % and do not make restrictions on C*B redundant
            %
            if tm.restrict(1,2)==1;
               %
               % create a random beta which satisfies the selected restrictions
               %
               beta = [];
               for i=1:tm.r;
                  file = strcat(tm.modelsdir,'H_',num2str(i),'-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
                  Hmat = loaddata(file);
                  beta = [beta Hmat*randn(size(Hmat,2),1)];
               end;
               %
               % now we normalize
               %
               file = strcat(tm.modelsdir,'normalize-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.norm');
               h_vec = loaddata(file);
               for i=1:tm.r;
                  norm_var = h_vec(i,1);
                  normbetai = beta(norm_var,i);
                  if abs(normbetai)<1.0e-006;
                     normbetai = 1;
                  end;
                  for j=1:size(beta,1);
                     beta(j,i) = beta(j,i)/normbetai;
                  end;
               end;
               beta = beta(1:tm.n,:);
               beta_orth = null(beta','r');
            else;
               beta_orth = randn(tm.n,tm.n-tm.r);
            end;
            Rmat = Rmat*kron(eye(tm.n),beta_orth);
         end;
         if rank(Rmat)==gv1;
            checkfmat = null(Rmat,'r');
            checkfmat = checkfmat*randn(size(checkfmat,2),1);
            checkfmat = checkfmat+(Rmat'*inverse(Rmat*Rmat')*rvec);
            if (strcmp(tm.type,'short_run')==1)|(tm.r==tm.n);
               checkfmat = vecinv(checkfmat,tm.n);
            else;
               checkfmat = vecinv(checkfmat,tm.n-tm.r);
               checkfmat = beta_orth*checkfmat;
            end;
            if rank(checkfmat)==tm.k;
               %
               % load restrictions into gui
               %
               tm.restfile = newrestfile;
               set(save_struc_rest.restfile,'String',tm.restfile);
               set(save_struc_rest.save,'Enable','on');
               myrest = ' ';
               for i=1:gv1;
                  thisrest = '';
                  k = 1;
                  l = 1;
                  for j=1:tm.n*tm.n;
                     if abs(Rmat(i,j))>0;
                        if Rmat(i,j)==1;
                           if strcmp(thisrest,'')==1;
                              thisrest = ['   ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                           else;
                              thisrest = [thisrest ' + ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                           end;
                        elseif Rmat(i,j)==-1;
                           if strcmp(thisrest,'')==1;
                              thisrest = ['   - ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                           else;
                              thisrest = [thisrest ' - ' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                           end;
                        else;
                           if Rmat(i,j)>0;
                              if strcmp(thisrest,'')==1;
                                 thisrest = ['   ' num2str(Rmat(i,j),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              else;
                                 thisrest = [thisrest ' + ' num2str(Rmat(i,j),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              end;
                           else;
                              if strcmp(thisrest,'')==1;
                                 thisrest = ['   - ' num2str(abs(Rmat(i,j)),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              else;
                                 thisrest = [thisrest ' - ' num2str(abs(Rmat(i,j)),'%1.2f') '*' tm.parameter '(' num2str(l) ',' num2str(k) ')'];
                              end;
                           end;
                        end
                     end;
                     if k==floor(j/tm.n);
                        k = k+1;
                        l = 1;
                     else;
                        l=l+1;
                     end;
                  end;
                  if abs(rvec(i,1))==0;
                     thisrest = [thisrest ' = 0'];
                  else;
                     thisrest = [thisrest ' = ' num2str(rvec(i,1),'%1.2f')];
                  end;
                  myrest = strvcat(myrest,thisrest);
                  set(save_struc_rest.gmat,'String',myrest);
               end;
               set(save_struc_rest.gmat,'String',strvcat(myrest,' '));
            else;
               if strcmp(tm.type,'short_run')==1;
                  txt = ['The file you selected implies that B has rank ' num2str(rank(checkfmat)) ', i.e. less than the covariance matrix of the residuals. Please select another file!'];
                  about(txt,'warning','Restrictions on B - Information',120,350);
               elseif strcmp(tm.type,'long_run')==1;
                  txt = ['The file you selected implies that A=C*B has rank ' num2str(rank(checkfmat)) ', i.e. less than ' num2str(tm.n-tm.r,'%0.0f') ', the rank of the C matrix. Since B must be of full rank, this is not allowed. Please select another file!'];
                  about(txt,'warning','Restrictions on A=C*B - Information',120,350);
               else;
                  txt = ['The file you selected implies that C has rank ' num2str(rank(checkfmat)) ', i.e. less than ' num2str(tm.n-tm.r,'%0.0f') '. Please select another file!'];
                  about(txt,'warning','Restrictions on C - Information',120,350);
               end;
               set(save_struc_rest.restfile,'String',tm.restfile);
            end;
         else;
            if rank(Rmat)>=1;
               if strcmp(tm.type,'short_run')==1;
                  txt = ['The ' num2str(gv1) ' restrictions you have selected for B are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(Rmat)) '. Please select another file'];
               elseif strcmp(tm.type,'long_run')==1;
                  if (check_rank==1)&(tm.r<tm.n);
                     add_text = ' This is probably due to an inconsistency between your selected restrictions and the beta matrix.';
                  else;
                     add_text = '';
                  end;
                  txt = ['The ' num2str(gv1) ' restrictions you have selected for A=C*B are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(new_Rmat)) '.' add_text ' Please select another file'];
               else;
                  txt = ['The ' num2str(gv1) ' restrictions you have selected for C are linearly dependent, i.e. the rank of the restrictions matrix is ' num2str(rank(Rmat)) '. Please select another file'];
               end;
            else;
               txt = ['The file you selected, ' getname(newrestfile) ', doesn''t contain any restrictions. Please select another file!'];
            end;
            if strcmp(tm.type,'short_run')==1;
               about(txt,'warning','Restrictions on B - Information',120,350);
            elseif strcmp(tm.type,'long_run')==1;
               about(txt,'warning','Restrictions on A=C*B - Information',150,350);
            else;
               about(txt,'warning','Restrictions on C - Information',120,350);
            end;
            set(save_struc_rest.restfile,'String',tm.restfile);
         end;
      end;
      set(save_struc_restdlg,'UserData',save_struc_rest);
   case 'save'
      oldrestfile = get(save_struc_rest.restfile,'String');
      oldrestfile = getpath(oldrestfile);
      oldrestfile = strcat(oldrestfile,'*.rest');
      [newrestfile,restpath] = uiputfile(oldrestfile, 'Save As');
      drawnow;
      if strcmp(tm.type,'short_run')==1;
         add_str = 'B';
         file_type = 'F-';
      elseif strcmp(tm.type,'long_run')==1;
         add_str = 'A=C*B';
         file_type = 'LF-';
      else;
         add_str = 'C';
         file_type = 'C-';
      end;
      if isnumeric(newrestfile)==0;
         newrestfile = strcat(restpath,newrestfile);
         if strcmp(getext(newrestfile),'.rest')==0;
            newrestfile = strcat(newrestfile,'.rest');
         end;
         oldrestfile = get(save_struc_rest.restfile,'String');
         %
         % Check if name is allowed
         %
         check = 0;
         for i=1:tm.n;
            checkname = strcat(restpath,file_type,num2str(tm.n),'x',num2str(i),'-',tm.modelfile,'.rest');
            if strcmp(checkname,newrestfile)==1;
               check = 1;
               break;
            end;
         end;
         if check==0;
            gvar = loaddata(oldrestfile);
            [gv1 gv2] = size(gvar);
            myprintstr = '%2.4f';
            for i=1:gv2-1;
               myprintstr = [myprintstr ' %2.4f'];
            end;
            myprintstr = [myprintstr '\n'];
            fid = fopen(newrestfile,'wt');
            fprintf(fid,myprintstr,gvar');
            fclose(fid);
            set(save_struc_rest.restfile,'String',newrestfile);
            tm.restfile = newrestfile;
            setlogtext(strvcat(' ',['You saved "' getfilename(oldrestfile) '"'],['   as "' getfilename(newrestfile) '"...']));
         else;
            txt = ['The file you selected, ' getname(checkname) ', is a restricted file name for internal use only.'];
            about(txt,'information','Bad file name...',120,350);
         end;
      end;
      set(save_struc_restdlg,'UserData',save_struc_rest);
   case 'done'
      set(save_struc_rest.done,'UserData','done');
   case 'exit'
      set(save_struc_rest.done,'UserData','done');
%      if strcmp(get(save_struc_rest.gmat,'UserData'),'done')==1;
%         delete(save_struc_restdlg);
%      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(save_struc_restdlg,'Name') ' ' fix_filepath([pwd '\icons\var_restfile.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(save_struc_rest.icon,'UserData',pid);
      end;
   end;
   set(save_struc_restdlg,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(save_struc_restdlg,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(save_struc_rest.done);
   end;
end;

if nargout==1;
   h = save_struc_restdlg;
end;

if strcmp(selector,'exit')==0;
   set(save_struc_rest.restfile,'UserData',tm);
end;

if thisexit==0;
   waitfor(save_struc_rest.done,'UserData','done');
end;

%
% end of save_struc_restgui.m
%
