%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI dialog for ssaving restrictions on beta    %
%          from a selected file to another selected file         %
%                                                                %
%          Function: saverestgui                                 %
%                                                                %
%          Input:                                                %
%                       selector: case switch function           %
%                       cm: structure with model info            %
%                                                                %
%          Output:                                               %
%                       h = handle                               %
%                                                                %
%          This version:        March 16, 2006                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function h = saverestgui(selector,cm)

if strcmp(selector,'init')==1;
   tm = cm;
   curr_ini = find_inisettings;
   %
   thisexit = 0;
   mycons = getstoredimages;
   bgcolor = get(0,'defaultuicontrolbackgroundcolor');
   mybrowse = maketrans(mycons.browse_image,[160 160 164],bgcolor);
   tm.restfile = tm.modelsdir;
else;
   thisexit = 1;
%   saverestdlg = get(0,'CurrentFigure');
   saverestdlg = findobj('Type','figure','Tag','SaveRestBeta');
   saverest = get(saverestdlg,'UserData');
   tm = get(saverest.restfile,'UserData');
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      saverestdlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','saverestgui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(realscrsz(3)-512)/2 (realscrsz(4)-395)/2 512 395], ...
         'Tag','SaveRestBeta', ...
         'Resize','off', ...
         'Name','Save Beta Restrictions', ...
         'NumberTitle','off', ...
         'Visible','off', ...
         'CloseRequestFcn','saverestgui exit', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(saverestdlg,'DockControl','off');
      end;

      %
      % Outer axis
      %

      axesbox([2 2 510 385],'Restrictions on Beta',100,[0.5 1]);

      %
      % GUI Header
      %

      mycons = getstoredimages;
      header_txt = 'Save Beta Restrictions';
      desc_txt = 'Open a restrictions file (*.rest) by pressing on the "Browse" button and view its content in the display. When you''re satisfied it''s the one you wish to save, press "Save" to save the restriction with another filename. Press "Done" to finish.';
      dialog_header_fcn([14 300 484 72],mirrorimage(maketrans(mycons.savebetabig_image,[160 160 164],[1 1 1])),header_txt,desc_txt);

      %
      % Outer axis for showing restrictions file
      %

      axesbox([14 100 484 182],'Transpose of Restrictions Matrix',155,[0.5 1]);

      %
      % Outer axis for restrictions file
      %

      axesbox([14 14 380 73],'Restrictions Selection',105,[0.5 1]);

      %
      % Listbox to show restrictions on beta
      %

      saverest.hmat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 116 452 150], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Max',2, ...
         'Value',[], ...
         'String','', ...
         'Enable','inactive', ...
         'UserData','working', ...
         'HorizontalAlignment','center', ...
         'Tag','BetaListbox');

      %
      % Show restrictions file
      %

      saverest.icon = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
      	 'Position',[30 50 150 20], ...
      	 'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
      	 'Style','text', ...
         'HorizontalAlignment','left', ...
      	 'String','Restrictions file:', ...
      	 'Tag','ShowRest');

      saverest.restfile = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[30 30 295 22], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','edit', ...
         'HorizontalAlignment','left', ...
         'String',tm.modelsdir, ...
         'Callback','saverestgui checkrest', ...
         'Enable','on', ...
         'Tag','RestFile');

      %
      % Button for selecting output file
      %

      saverest.restbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[330 30 50 22], ...
         'TooltipString','Browse', ...
         'CData',mybrowse, ...
         'Callback','saverestgui openrest', ...
         'Enable','on', ...
         'Tag','RestButton');

      %
      % Save button
      %

      saverest.save = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 48 90 20], ...
         'String','Save', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','saverestgui save', ...
         'Enable','off', ...
         'UserData','waiting', ...
         'Tag','SaveButton');

      %
      %  Done button
      %

      saverest.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[408 14 90 20], ...
         'String','Done', ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'CallBack','saverestgui done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','DoneButton');

      set(saverestdlg,'UserData',saverest,...
         'HandleVisibility','callback');
   case 'openrest'
      curr_ini = find_inisettings;
      if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
         newrestfile = get(saverest.restfile,'String');
         basedir = getpath(newrestfile);
         file_fltr = 'VAR Restrictions Files (*.rest)|*.rest|';
         best_pos = 1;
         title_txt = 'Select file or enter filename';
         [newrestfile,restpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
      else;
         newrestfile = get(saverest.restfile,'String');
         newrestfile = getpath(newrestfile);
         newrestfile = strcat(newrestfile,'*.rest');
         [newrestfile,restpath] = uigetfile(newrestfile, 'Select file or enter filename');
      end;
      drawnow;
      if isnumeric(newrestfile)==0;
         newrestfile = strcat(restpath,newrestfile);
         if fileexist(newrestfile)==1;
            hvar = loaddata(newrestfile);
            [hv1 hv2] = size(hvar);
            if hv1==tm.n+tm.q1+tm.dcoint;
               if hv2<=tm.n+tm.q1+tm.dcoint-tm.r+1;
                  tm.restfile = newrestfile;
                  set(saverest.restfile,'String',tm.restfile);
                  set(saverest.save,'Enable','on');
                  hvar = showbeta(hvar,tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
                  hv = size(hvar,1);
                  hvec = ' ';
                  for i=1:hv-1;
                     hvec = strvcat(hvec,' ');
                  end;
                  hvar = [hvec hvar];
                  set(saverest.hmat,'String',hvar);
               else;
                  txt = ['The file you selected, ' getname(newrestfile) ', contains too few restrictions.'];
                  about(txt,'information','Too few restrictions...',120,350);
               end;
            else;
               if hv1>tm.n+tm.q1+tm.dcoint;
                  txt = ['The file you selected, ' getname(newrestfile) ', assumes too many variables.'];
                  about(txt,'information','Too many variables...',120,350);
               else;
                  txt = ['The file you selected, ' getname(newrestfile) ', assumes too few variables.'];
                  about(txt,'information','Too few variables...',120,350);
               end;
            end;
         else;
            set(saverest.restfile,'String',tm.restfile);
            if fileexist(tm.restfile)==1;
               set(saverest.save,'Enable','on');
            else;
               set(saverest.save,'Enable','off');
            end;
         end;
      else;
         set(saverest.restfile,'String',tm.restfile);
         if fileexist(tm.restfile)==1;
            set(saverest.save,'Enable','on');
         else;
            set(saverest.save,'Enable','off');
         end;
      end;
      set(saverestdlg,'UserData',saverest);
   case 'checkrest'
      newrestfile = get(saverest.restfile,'String');
      if fileexist(newrestfile)==0;
         set(saverest.restfile,'String',tm.restfile);
         if fileexist(tm.restfile)==1;
            set(saverest.save,'Enable','on');
         else;
            set(saverest.save,'Enable','off');
         end;
      else;
         hvar = loaddata(newrestfile);
         [hv1 hv2] = size(hvar);
         if hv1==tm.n+tm.q1+tm.dcoint;
            if hv2<=tm.n+tm.q1+tm.dcoint-tm.r+1;
               tm.restfile = newrestfile;
               set(saverest.restfile,'String',tm.restfile);
               set(saverest.save,'Enable','on');
               hvar = showbeta(hvar,tm.evars,strtrim([tm.xvars1 ' ' tm.restvec]));
               hv = size(hvar,1);
               hvec = ' ';
               for i=1:hv-1;
                  hvec = strvcat(hvec,' ');
               end;
               hvar = [hvec hvar];
               set(saverest.hmat,'String',hvar);
            else;
               txt = ['The file you selected, ' getname(newrestfile) ', contains too few restrictions.'];
               about(txt,'information','Too few restrictions...',120,350);
               set(saverest.restfile,'String',tm.restfile);
               if fileexist(tm.restfile)==0;
                  set(saverest.save,'Enable','off');
               else;
                  set(saverest.save,'Enable','on');
               end;
            end;
         %
         % used to be a missing else...
         %
         else;
            if hv1>tm.n+tm.q1+tm.dcoint;
               txt = ['The file you selected, ' getname(newrestfile) ', assumes too many variables.'];
               about(txt,'information','Too many variables...',120,350);
            else;
               txt = ['The file you selected, ' getname(newrestfile) ', assumes too few variables.'];
               about(txt,'information','Too few variables...',120,350);
            end;
            set(saverest.restfile,'String',tm.restfile);
            if fileexist(tm.restfile)==0;
               set(saverest.save,'Enable','off');
            else;
               set(saverest.save,'Enable','on');
            end;
         end;
      end;
      set(saverestdlg,'UserData',saverest);
   case 'save'
      oldrestfile = get(saverest.restfile,'String');
      oldrestfile = getpath(oldrestfile);
      oldrestfile = strcat(oldrestfile,'*.rest');
      [newrestfile,restpath] = uiputfile(oldrestfile, 'Save As');
      drawnow;
      if isnumeric(newrestfile)==0;
         newrestfile = strcat(restpath,newrestfile);
         if strcmp(getext(newrestfile),'.rest')==0;
            newrestfile = strcat(newrestfile,'.rest');
         end;
         oldrestfile = get(saverest.restfile,'String');
         %
         % Check if name is allowed
         %
         check = 0;
         for i=1:tm.r;
            checkname = strcat(restpath,'H_',int2str(i),'-',num2str(tm.n),'x',num2str(tm.r),'x',num2str(tm.dcoint+tm.q1),'-',tm.modelfile,'.rest');
            if strcmp(checkname,newrestfile)==1;
               check = 1;
               i = tm.r+1;
            end;
         end;
         if check==0;
            hvar = loaddata(oldrestfile);
            [hv1 hv2] = size(hvar);
            %
            % check if dimensions are valid
            %
            if hv1==tm.n+tm.q1+tm.dcoint;
               if hv2<=tm.n+tm.q1+tm.dcoint-tm.r+1;
                  myprintstr = '%2.4f';
                  for i=1:hv2-1;
                     myprintstr = [myprintstr ' %2.4f'];
                  end;
                  myprintstr = [myprintstr '\n'];
                  fid = fopen(newrestfile,'wt');
                  fprintf(fid,myprintstr,hvar');
                  fclose(fid);
                  set(saverest.restfile,'String',newrestfile);
                  tm.restfile = newrestfile;
                  setlogtext(strvcat(' ',['You saved "' getfilename(oldrestfile) '"'],['   as "' getfilename(newrestfile) '"...']));
               else;
                  txt = ['The file you selected, ' getname(oldrestfile) ', contains too few restrictions.'];
                  about(txt,'information','Too few restrictions...',120,350);
               end;
            else;
               if hv1>tm.n+tm.q1+tm.dcoint;
                  txt = ['The file you selected, ' getname(oldrestfile) ', assumes too many variables.'];
                  about(txt,'information','Too many variables...',120,350);
               else;
                  txt = ['The file you selected, ' getname(oldrestfile) ', assumes too few variables.'];
                  about(txt,'information','Too few variables...',120,350);
               end;
            end;
         else;
            txt = ['The file you selected, ' getname(checkname) ', is a restricted file name for internal use only.'];
            about(txt,'information','Bad file name...',120,350);
         end;
      end;
      set(saverestdlg,'UserData',saverest);
   case 'done'
      set(saverest.done,'UserData','done');
   case 'exit'
      set(saverest.done,'UserData','done');
%      if strcmp(get(saverest.hmat,'UserData'),'done')==1;
%         delete(saverestdlg);
%      end;
end;

if strcmp(selector,'init')==1;
   if (iswinnt==1)&(iscompiled==1)&(curr_ini.loadguiicon==1);
      prog = ['bin\runwindow.exe ' fix_path([pwd '\bin\seticon.exe']) ' ' get(saverestdlg,'Name') ' ' fix_filepath([pwd '\icons\var_restfile.ico'])];
      dos(prog);
      if (strcmp(curr_ini.pstoolsdir,'')==0)&(appexist([curr_ini.pstoolsdir 'pslist.exe'])==1);
         %
         % now we need to determine the pid
         %
         pid = getpid('seticon');
         set(saverest.icon,'UserData',pid);
      end;
   end;
   set(saverestdlg,'Visible','on');
   drawnow;
   if (iswinnt==1)&(iscompiled==1)&(matlabnumber<7);
      drawnow;
      prog = ['bin\run_prog.exe ' fix_path([pwd '\bin\setfocus.exe']) ' ' get(saverestdlg,'Name') ' 1'];
      dos(prog);
      drawnow;
   elseif matlabnumber>=7;
      uicontrol(saverest.done);
   end;
end;

if nargout==1;
   h = saverestdlg;
end;

if strcmp(selector,'exit')==0;
   set(saverest.restfile,'UserData',tm);
end;

if thisexit==0;
   waitfor(saverest.done,'UserData','done');
end;

%
% end of saverestgui.m
%
