%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab GUI for estimating structural VAR model        %
%                                                                %
%          Function: svargui                                     %
%                                                                %
%          Input:                                                %
%                    selector: case switch function              %
%                    curr_model: structure with model info       %
%                                                                %
%          Output:                                               %
%                    myhandle: handle                            %
%                                                                %
%          This version:        July 4, 2006                     %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function myhandle = svargui(selector,curr_model)

if strcmp(selector,'init')==1;
   cm = curr_model;
   cm.restrict = [cm.restrict 0];
   if strcmp(strtrim(cm.shocks),'')==1;
      for i=1:cm.n;
         if i==1;
            cm.shocks = 's_1';
         else;
            cm.shocks = [cm.shocks ' s_' num2str(i)];
         end;
      end;
   end;
   cm.myshocks = vecstr(cm.shocks);
   if size(cm.myshocks,1)~=cm.n;
      cm.shocks = 's_1';
      for i=2:cm.n;
         cm.shocks = [cm.shocks ' s_' num2str(i,'%0.0f')];
      end;
      cm.myshocks = vecstr(cm.shocks);
   end;
   cm.disable = zeros(1,3);
   %
   cm.running = 0;
   cm.cbs = cm.c;
   if cm.restrict(1,2)==1;
      cm.bZ1t = cm.beta'*cm.Z1t;
   else;
      cm.bZ1t = normbeta(cm.beta)'*cm.Z1t;
   end;
   if cm.cointvalue==0;
      if isempty(cm.Z2)==0;
         cm.R1t = cm.Z1t-(cm.M_12*inverse(cm.M_22)*cm.Z2);
      else;
         cm.R1t = cm.Z1t;
      end;
   end;
   if cm.restrict(1,2)==1;
      cm.bR1t = cm.beta'*cm.R1t;
   else;
      cm.bR1t = normbeta(cm.beta)'*cm.R1t;
   end;
   %
   if cm.r==0;
      cm.beta = [];
      cm.alpha = [];
   end;
   %
   cm.lastTheta = cm.Theta;
   cm.B_0 = [];
   cm.Sigma_B0 = [];
   cm.Sigma_B0_adj = [];
   cm.A = [];
   cm.Sigma_A = [];
   cm.S = [];
   cm.gamma = [];
   cm.R_l = [];
   cm.R_mat = [];
   cm.C = [];
   cm.dC_theta = [];
   cm.dB_theta = [];
   %
   thisexit = 0;
   cm.estimated = 0;
   %
   % get string matrices for model information
   %
   cm.model_info = getmodelinfo(cm,1);
   cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,loglike(cm.epsilon),cm.evalue);
   %
   % Test if F-modelfile.rest files exist in modelsdir
   %
   estimateenable = 'off';
   sr_enable = 'off';
   file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
   if fileexist(file)==1;
      estimateenable = 'on';
      sr_enable = 'on';
   end;
   %
   % test if C-modelfile.rest file exists
   %
   estimate_c_enable = 'off';
   file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
   if fileexist(file)==1;
      estimate_c_enable = 'on';
   end;
   %
   lr_enable = 'off';
   if (cm.r>0)|(cm.p>1);
      file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
      if fileexist(file)==1;
         estimateenable = 'on';
         lr_enable = 'on';
      end;
   end;
   %
   if (strcmp(sr_enable,'on')==1)&(strcmp(lr_enable,'on')==1);
      joint_enable = 'on';
   else;
      joint_enable = 'off';
   end;
   %
   curr_ini = find_inisettings;
   if curr_ini.lagtest>0;
      lagenable = 'on';
   else;
      lagenable = 'off';
   end;
   if isnumeric(curr_ini.external)==0;
      curr_ini.external = 0;
   end;
   %
   showpdfdoc = [];
   if (strcmp(curr_ini.pdfviewer,'')==0)|(strcmp(curr_ini.epsviewer,'')==0)|(strcmp(curr_ini.psviewer,'')==0);
      pdf_file = [pwd '\doc\estimate-c.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.estimate_c = pdf_file;
      else;
         showpdfdoc.estimate_c = '';
      end;
      %
      pdf_file = [pwd '\doc\var-decomp.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.var_decomp = pdf_file;
      else;
         showpdfdoc.var_decomp = '';
      end;
      %
      pdf_file = [pwd '\doc\generalized-impulse-response.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.gir = pdf_file;
      else;
         showpdfdoc.gir = '';
      end;
      %
      pdf_file = [pwd '\doc\common-cycles.pdf'];
      if isunix==1;
         pdf_file = strrep(pdf_file,'\','/');
      end;
      if fileexist(pdf_file)==1;
         showpdfdoc.cc = pdf_file;
      else;
         showpdfdoc.cc = '';
      end;
   end;
   %
   % import images
   %
   button_images = read_images;
   %
   if cm.r<cm.n+cm.q1;
      if cm.evalue>=1;
         mypermcompenable = 'off';
      else;
         mypermcompenable = 'on';
      end;
   else;
      mypermcompenable = 'off';
   end;
   if cm.r<cm.n;
      cmatenable = mypermcompenable;
   else;
      cmatenable = 'on';
   end;
   weakexoenable = 'on';
   if cm.r==cm.n;
      weakexoenable = 'off';
   end;
   if (cm.n==1)&(cm.q1+cm.q0==0);
      gcenable = 'off';
   else;
      gcenable = 'on';
   end;
   if (cm.r==0)&(cm.p==1);
      cmatenable = 'off';
      mypermcompenable = 'off';
      gcenable = 'off';
   end;
   cm.trendplace = 0;
   if cm.spectest(1,4)==1;
      if cm.deterselect(1,3)==1;
         cm.trendplace = 2;
      end;
   end;
   cm.old_values = cm;
else;
   thisexit = 1;
   svarguidlg = findobj('Type','figure','Tag','SVARGUI')';
   svar = get(svarguidlg,'UserData');
   curr_ini = get(svar.viewdata,'UserData');
   button_images = curr_ini.images;
   cm = curr_ini.model;
   cm.svar = svar;
   if isfield(svar,'c_undo')==1;
      if strcmp(get(svar.c_undo,'Enable'),'off')==1;
         cm.restrict(1,3) = 0;
      else;
         cm.restrict(1,3) = 1;
      end;
   else;
      cm.restrict(1,3) = 0;
   end;
end;

%
% Set screen size directly instead of using
% get(0,'ScreenSize')!
%

scrsz = [1 1 1024 768];
realscrsz = get(0,'ScreenSize');

switch selector;
   case 'init';
      pos = [(realscrsz(3)-(scrsz(3)/2)-180)/2 (realscrsz(4)-(scrsz(4)/2)-140)/2 scrsz(3)/2+180 scrsz(4)/2+140];
      if min(pos)<0;
         pos = [0 0 scrsz(3)/2+180 scrsz(4)/2+1140];
      end;
      svarguidlg = figure('Color',get(0,'defaultuicontrolbackgroundcolor'), ...
         'FileName','svargui.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',pos, ...
         'Tag','SVARGUI', ...
         'Resize','off', ...
         'Name','Structural VAR - Identification & Estimation', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','svargui wayout', ...
         'Visible','off', ...
         'ToolBar','none');

      if matlabnumber>=7.0;
         set(svarguidlg,'DockControl','off');
      end;

      if strcmp(curr_ini.helpexe,'')==1;
         helpenable = 'off';
      else;
         helpenable = 'on';
      end;

      %
      % File menu
      %

      filemenu = uimenu('Parent',svarguidlg, ...
         'Label','&File', ...
         'Tag','FileMenu');

      %
      % Objects on File menu
      %

      svar.viewdata = uimenu('Parent',filemenu, ...
         'Label','View &Data...', ...
         'Callback','svargui viewdata', ...
         'Enable','on', ...
         'Tag','ViewData');

      add_sep = 'on';

      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);

         svar.savecoint = uimenu('Parent',filemenu, ...
            'Label','Save &Cointegration Relations...', ...
            'Callback','svargui savecoint', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','SaveResid');

         add_sep = 'off';

      end;

      svar.savetc = uimenu('Parent',filemenu, ...
         'Label','Save &Transitory Components...', ...
         'Callback','svargui savetc', ...
         'Separator',add_sep, ...
         'Accelerator','T', ...
         'Enable',mypermcompenable, ...
         'Tag','SaveTC');

      svar.saveresid = uimenu('Parent',filemenu, ...
         'Label','Save &Residuals...', ...
         'Callback','svargui saveresid', ...
         'Accelerator','D', ...
         'Enable','on', ...
         'Tag','SaveCoint');

      uimenu('Parent',filemenu, ...
         'Label','Center &Window', ...
         'Separator','on', ...
         'Accelerator','W', ...
         'Enable','on', ...
         'Callback','center_figure_fcn', ...
         'Tag','CenterMenu');

      new_sep = 'on';

      if cm.cointvalue==1;

         uimenu('Parent',filemenu, ...
            'Label','R&eturn', ...
            'Callback','svargui quit', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','MenuReturn');

         new_sep = 'off';

      end;

      svar.estalpha = uimenu('Parent',filemenu, ...
         'Label','D&one', ...
         'Callback','svargui continue', ...
         'UserData',cm.Theta, ...
         'Accelerator','C', ...
         'Separator',new_sep, ...
         'Enable','on', ...
         'Tag','MenuContinue');

      cm.cointbeta = svar;

      %
      % Test menu
      %

      testmenu = uimenu('Parent',svarguidlg, ...
         'Label','&Tools', ...
         'Tag','TestMenu');

      %
      % Objects on Tools menu
      %

      svar.corrtest = uimenu('Parent',testmenu, ...
         'Label','&Serial Correlation...', ...
         'Callback','svargui corrtest', ...
         'Enable','on', ...
         'Accelerator','S', ...
         'Tag','CorrTest');

      svar.archtest = uimenu('Parent',testmenu, ...
         'Label','&ARCH...', ...
         'Callback','svargui archtest', ...
         'Accelerator','A', ...
         'Enable','on', ...
         'Tag','ArchTest');

      svar.normalitytest = uimenu('Parent',testmenu, ...
         'Label','&Normality...', ...
         'Callback','svargui normalitytest', ...
         'Accelerator','N', ...
         'Enable','on', ...
         'Tag','NormalityTest');

      svar.constancytest = uimenu('Parent',testmenu, ...
         'Label','&Parameter Constancy...', ...
         'Callback','svargui constancytest', ...
         'Accelerator','P', ...
         'Enable','on', ...
         'Tag','ConstancyTest');

      svar.lagorder = uimenu('Parent',testmenu, ...
         'Label','&Lag Order...', ...
         'Callback','svargui lagorder', ...
         'Accelerator','L', ...
         'Separator','on', ...
         'Enable',lagenable, ...
         'Tag','LagOrder');

      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);

         svar.weakexo = uimenu('Parent',testmenu, ...
            'Label','&Weak Exogeneity...', ...
            'Callback','svargui weakexo', ...
            'Enable',weakexoenable, ...
            'Tag','WeakExogeneity');

      end;

      svar.gctest = uimenu('Parent',testmenu, ...
         'Label','&Granger Causality...', ...
         'Callback','svargui gctest', ...
         'Accelerator','G', ...
         'Enable',gcenable, ...
         'Tag','GrangerCausality');

      if (cm.n>2)&((cm.p>1)|(cm.r>0));
         svar.multigctest = uimenu('Parent',testmenu, ...
            'Label','M&ulti-Step Granger Causality...', ...
            'Callback','svargui multigctest', ...
            'Enable','on', ...
            'Tag','MultiGrangerCausality');
      end;

      if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
         svar.exclude = uimenu('Parent',testmenu, ...
            'Label','&Exclusion', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ExclusionMenu');
         
         svar.firstdiffs = uimenu('Parent',svar.exclude, ...
            'Label','&First Differences', ...
            'Callback','svargui firstdiffs', ...
            'Enable','on', ...
            'Tag','FirstDiffMenu');
      end;

      if (cm.r>0)&(cm.r<cm.n+cm.q1)&(cm.cointvalue==1);
         svar.showbeta = uimenu('Parent',testmenu, ...
            'Label','Cointegration &Vectors...', ...
            'Callback','svargui showbeta', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ShowBeta');
      end;

      if (cm.p>1)&(cm.r<cm.n);
         svar.common_cycles = uimenu('Parent',testmenu, ...
            'Label','Co&mmon Cycles...', ...
            'Callback','svargui common_cycles', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','ShowCommonCycles');
      end;

      if cm.datad>0;
         svar.dummy = uimenu('Parent',testmenu, ...
            'Label','&Deterministic Variables', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','DummyVarMenu');

         svar.dummyexclusion = uimenu('Parent',svar.dummy, ...
            'Label','&Exclusion...', ...
            'Callback','svargui dummyexclusion', ...
            'Enable','on', ...
            'Tag','DummyExclusionMenu');

         svar.dummylongrun = uimenu('Parent',svar.dummy, ...
            'Label','&Long Run...', ...
            'Callback','svargui dummylongrun', ...
            'Enable','off', ...
            'Tag','DummyLongRunMenu');

         if (cm.r<cm.n)&(cm.r>0);
            set(svar.dummylongrun,'Enable','on');
         end;

         if cm.datad>1;
            svar.propdummy = uimenu('Parent',svar.dummy, ...
               'Label','&Proportionality...', ...
               'Callback','svargui dummyprop', ...
               'Enable','on', ...
               'Tag','DummyPropMenu');
         end;

      end;

      svar.amat = uimenu('Parent',testmenu, ...
         'Label','A-Ma&trix...', ...
         'Callback','svargui amat', ...
         'Separator','on', ...
         'Enable','off', ...
         'Tag','AMatrix');

      svar.bmat = uimenu('Parent',testmenu, ...
         'Label','B-Matri&x...', ...
         'Callback','svargui bmat', ...
         'Enable','off', ...
         'Tag','BMatrix');

      svar.cmat = uimenu('Parent',testmenu, ...
         'Label','&C-Matrix...', ...
         'Callback','svargui cmat', ...
         'Accelerator','M', ...
         'Enable',cmatenable, ...
         'Tag','CMatrix');

      svar.longrun_gir = uimenu('Parent',testmenu, ...
         'Label','L&ong Run GIRs...', ...
         'Callback','svargui longrun_gir', ...
         'Separator','off', ...
         'Enable',cmatenable, ...
         'Tag','LongRunGIRsMenu');

      svar.residanalysis = uimenu('Parent',testmenu, ...
         'Label','&Residual Analysis...', ...
         'Callback','svargui residanalysis', ...
         'Accelerator','R', ...
         'Enable','on', ...
         'Tag','ResidAnalysis');

      svar.shortdyn = uimenu('Parent',testmenu, ...
         'Label','S&hort Run Dynamics...', ...
         'Callback','svargui shortdyn', ...
         'Accelerator','Y', ...
         'Enable','on', ...
         'Tag','ShortDyn');

      svar.vardecomp = uimenu('Parent',testmenu, ...
         'Label','Var&iance Decompositions...', ...
         'Callback','svargui vardecomp', ...
         'Separator','on', ...
         'Enable','off', ...
         'Tag','VarianceDecompMenu');

      svar.distribution = uimenu('Parent',testmenu, ...
         'Label','Distri&bution...', ...
         'Callback','svargui distribution', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','DistributionMenu');

      %
      % Graphics menu
      %

      graphmenu = uimenu('Parent',svarguidlg, ...
         'Label','&Graphics', ...
         'Tag','GraphicsMenu');

      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);

         svar.menugraph = uimenu('Parent',graphmenu, ...
            'Label','&Cointegration Relations...', ...
            'Callback','svargui graphs', ...
            'Enable','on', ...
            'Tag','MenuGraph');

      end;

      svar.menupermcomp = uimenu('Parent',graphmenu, ...
         'Label','&Permanent and Transitory Components...', ...
         'Callback','svargui permcomp', ...
         'Accelerator','E', ...
         'Enable',mypermcompenable, ...
         'Tag','MenuPermComp');

      svar.menu_permcomp = uimenu('Parent',graphmenu, ...
         'Label','P&ermanent and Transitory Shock Components...', ...
         'Callback','svargui perm_comp', ...
         'Enable','off', ...
         'Tag','Menu_PermComp');

      svar.menuresiduals = uimenu('Parent',graphmenu, ...
         'Label','&Residuals...', ...
         'Callback','svargui residuals', ...
         'Accelerator','I', ...
         'Enable','on', ...
         'Tag','MenuResiduals');

      svar.menufitted = uimenu('Parent',graphmenu, ...
         'Label','Fitted &Terms...', ...
         'Callback','svargui fitted', ...
         'Enable','on', ...
         'Tag','MenuFitted');

      svar.forecast = uimenu('Parent',graphmenu, ...
         'Label','Forec&asting...', ...
         'Callback','svargui forecast', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuFitted');

      svar.varforecast = uimenu('Parent',graphmenu, ...
         'Label','Forecast &Variable Transformations...', ...
         'Callback','svargui varforecast', ...
         'Enable','on', ...
         'Tag','MenuVARForecast');

      if cm.q1+cm.q0>0;
         set(svar.forecast,'Enable','off');
         set(svar.varforecast,'Enable','off');
      end;

      svar.menuchow = uimenu('Parent',graphmenu, ...
         'Label','Recursive Ch&ow Tests', ...
         'Callback','svargui chow', ...
         'Separator','on', ...
         'Accelerator','O', ...
         'Enable','on', ...
         'Tag','MenuChow');

      svar.menuploberger = uimenu('Parent',graphmenu, ...
         'Label','Recursive &Fluctuation Test', ...
         'Callback','svargui ploberger', ...
         'Accelerator','F', ...
         'Enable','on', ...
         'Tag','MenuPloberger');

      svar.menuirfunction = uimenu('Parent',graphmenu, ...
         'Label','&Impulse Responses...', ...
         'Separator','on', ...
         'Accelerator','U', ...
         'Callback','svargui irfunction', ...
         'Enable','off', ...
         'Tag','MenuImpulse');

      svar.histdecomp = uimenu('Parent',graphmenu, ...
         'Label','&Historical Decompositions...', ...
         'Callback','svargui histdecomp', ...
         'Enable','off', ...
         'Tag','MenuHistDecomp');

      svar.genimpulse = uimenu('Parent',graphmenu, ...
         'Label','&Generalized Impulse Responses...', ...
         'Callback','svargui genimpulse', ...
         'Separator','on', ...
         'Enable','on', ...
         'Tag','MenuGeneralizedImpulse');

%      if (isempty(cm.isdummy)==0)&(max(cm.isdummy)==1);
      if cm.datad>0;
         svar.dummyimpulse = uimenu('Parent',graphmenu, ...
            'Label','&Dummy Impulse Responses...', ...
            'Callback','svargui dummyimpulse', ...
            'Enable','on', ...
            'Tag','MenuDummyImpulse');
      end;
         
      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);

         svar.betagraph = uimenu('Parent',graphmenu, ...
            'Label','&Log-likelihood and Beta...', ...
            'Callback','svargui betagraph', ...
            'Separator','on', ...
            'Accelerator','K', ...
            'Enable','off', ...
            'Tag','MenuBetaGraph');

         if cm.restrict(1,2)==1;
            set(svar.betagraph,'Enable','on');
         end;

      end;

      svar.companion = uimenu('Parent',graphmenu, ...
         'Label','Eigenvalues of Companion &Matrix', ...
         'Callback','svargui companion', ...
         'Separator',add_sep, ...
         'Enable','on', ...
         'Tag','MenuCompanion');

      if (cm.r<cm.n)&((cm.r>0)|(cm.p>1));

         %
         % C menu
         %

         cmenu = uimenu('Parent',svarguidlg, ...
            'Label','&C-Matrix', ...
            'Tag','CMenu');

         svar.restrict_cmenu = uimenu('Parent',cmenu, ...
               'Label','&C Restrictions...', ...
               'Callback','svargui restrict_c', ...
               'Enable','on', ...
               'Tag','MenuRestrictC');

         svar.c_openrestmenu = uimenu('Parent',cmenu, ...
               'Label','&Open C Restrictions...', ...
               'Callback','svargui c_openrest', ...
               'Enable','on', ...
               'Tag','MenuOpenCRest');

         svar.c_saverestmenu = uimenu('Parent',cmenu, ...
               'Label','&Save C Restrictions...', ...
               'Callback','svargui c_saverest', ...
               'Enable','on', ...
               'Tag','MenuSaveCRest');

         svar.c_estimatemenu = uimenu('Parent',cmenu, ...
               'Separator','on', ...
               'Label','&Estimate C...', ...
               'Callback','svargui c_estimate', ...
               'Enable',estimate_c_enable, ...
               'Tag','MenuEstimateC');

         svar.c_undomenu = uimenu('Parent',cmenu, ...
               'Label','&Undo C', ...
               'Callback','svargui c_undo', ...
               'Enable','off', ...
               'Tag','MenuUndoC');

      end;

      %
      % Structural menu
      %

      structuralmenu = uimenu('Parent',svarguidlg, ...
         'Label','&Structural', ...
         'Tag','StructuralMenu');

      svar.restrict_srmenu = uimenu('Parent',structuralmenu, ...
            'Label','&Contemporaneous Restrictions...', ...
            'Callback','svargui restrict_sr', ...
            'Enable','on', ...
            'Tag','MenuRestrictSR');

      svar.sr_openrestmenu = uimenu('Parent',structuralmenu, ...
            'Label','&Open Contemporaneous Restrictions...', ...
            'Callback','svargui sr_openrest', ...
            'Enable','on', ...
            'Tag','MenuOpenSR');

      svar.sr_saverestmenu = uimenu('Parent',structuralmenu, ...
            'Label','&Save Contemporaneous Restrictions...', ...
            'Callback','svargui sr_saverest', ...
            'Enable','on', ...
            'Tag','MenuSaveSR');

      svar.restrict_lrmenu = uimenu('Parent',structuralmenu, ...
            'Label','&Long-Run Restrictions...', ...
            'Callback','svargui restrict_lr', ...
            'Separator','on', ...
            'Enable','on', ...
            'Tag','MenuRestrictLR');

      svar.lr_openrestmenu = uimenu('Parent',structuralmenu, ...
            'Label','O&pen Long-Run Restrictions...', ...
            'Callback','svargui lr_openrest', ...
            'Enable','on', ...
            'Tag','MenuOpenLR');

      svar.lr_saverestmenu = uimenu('Parent',structuralmenu, ...
            'Label','S&ave Long-Run Restrictions...', ...
            'Callback','svargui lr_saverest', ...
            'Enable','on', ...
            'Tag','MenuSaveLR');

      svar.signshockmenu = uimenu('Parent',structuralmenu, ...
            'Separator','on', ...
            'Label','S&ign Shocks...', ...
            'Callback','svargui signshock', ...
            'Enable',estimateenable, ...
            'Tag','MenuSignShock');

      svar.struc_estimatemenu = uimenu('Parent',structuralmenu, ...
            'Separator','on', ...
            'Label','&Estimate B...', ...
            'Accelerator','B', ...
            'Callback','svargui struc_estimate', ...
            'Enable',estimateenable, ...
            'Tag','MenuEstimateB');

      svar.struc_undomenu = uimenu('Parent',structuralmenu, ...
            'Label','&Undo B', ...
            'Callback','svargui struc_undo', ...
            'Enable','off', ...
            'Tag','MenuUndoB');

      if (cm.r==0)&(cm.p==1);
         set(svar.restrict_lrmenu,'Enable','off');
         set(svar.lr_openrestmenu,'Enable','off');
         set(svar.lr_saverestmenu,'Enable','off');
      end;

      %
      % Simulate menu
      %

      simulatemenu = uimenu('Parent',svarguidlg, ...
         'Label','&Bootstrap', ...
         'Tag','SimulateMenu');

      svar.bootalpha = uimenu('Parent',simulatemenu, ...
         'Label','&Alpha, Delta, Gamma...', ...
         'Callback','svargui bootalpha', ...
         'Enable','on', ...
         'Tag','MenuBootAlpha');

      if (isempty(cm.Z2)==1)&(cm.dfalpha==cm.n*cm.r);
         set(svar.bootalpha,'Enable','off');
      end;

      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);

         svar.bootbeta = uimenu('Parent',simulatemenu, ...
            'Label','&Beta...', ...
            'Callback','svargui bootbeta', ...
            'Enable','off', ...
            'Tag','MenuBootBeta');

         svar.bootnyblommenu = uimenu('Parent',simulatemenu, ...
            'Label','&Nyblom Distribution...', ...
            'Callback','svargui bootnyblomtest', ...
            'Enable','off', ...
            'Tag','MenuBootNyblom');

         if cm.restrict(1,2)==1;
            set(svar.bootbeta,'Enable','on');
            set(svar.bootnyblommenu,'Enable','on');
         end;
         
         if (curr_ini.nyblom==0)|(curr_ini.nyblombeta==0);
            set(svar.bootnyblommenu,'Enable','off');
         end;

      end;

      svar.bootirmenu = uimenu('Parent',simulatemenu, ...
            'Label','Structural &Parameters...', ...
            'Callback','svargui bootir', ...
            'Enable','off', ...
            'Tag','MenuBootIR');

      svar.bootploberger = uimenu('Parent',simulatemenu, ...
         'Label','&Recursive Fluctuation Test...', ...
         'Callback','svargui bootploberger', ...
         'Enable','on', ...
         'Tag','MenuBootPloberger');

      svar.boot_specification = uimenu('Parent',simulatemenu, ...
         'Label','&Specification Tests...', ...
         'Callback','svargui boot_specification', ...
         'Enable','on', ...
         'Tag','MenuBootSpecification');

      %
      % Help menu
      %

      helpmenu = uimenu('Parent',svarguidlg, ...
         'Label','&Help', ...
         'Tag','HelpMenu');

      svar.helpmenu = uimenu('Parent',helpmenu, ...
         'Label','&Help Topics', ...
         'Callback','help_fcn(''identification'')', ...
         'Enable',helpenable, ...
         'Tag','MenuHelpFcn');

      svar.svar_help = uimenu('Parent',helpmenu, ...
            'Label','&Identification and Estimation', ...
            'Callback','help_fcn(''IdentificationEstimation'');', ...
            'Accelerator','H', ...
            'Separator','off', ...
            'Enable','on', ...
            'Tag','MenuSvarHelp');

      if isempty(showpdfdoc)==0;
         pdf_separator = 'on';
         if strcmp(showpdfdoc.cc,'')==0;
            svar.pdf_cc = uimenu('Parent',helpmenu, ...
               'Label','&Common Cycles - PDF file', ...
               'Callback','show_pdf', ...
               'Separator',pdf_separator, ...
               'UserData',showpdfdoc.cc, ...
               'Enable','on', ...
               'Tag','MenuDocCCEstimate');
            pdf_separator = 'off';
         end;
         %
         if strcmp(showpdfdoc.estimate_c,'')==0;
            svar.pdf_estimate_c = uimenu('Parent',helpmenu, ...
               'Label','&Estimate C - PDF file', ...
               'Callback','show_pdf', ...
               'Separator',pdf_separator, ...
               'UserData',showpdfdoc.estimate_c, ...
               'Enable','on', ...
               'Tag','MenuDocCEstimate');
            pdf_separator = 'off';
         end;
         %
         if strcmp(showpdfdoc.gir,'')==0;
            svar.pdf_gir = uimenu('Parent',helpmenu, ...
               'Label','&Generalized Impulse Responses - PDF file', ...
               'Callback','show_pdf', ...
               'Separator',pdf_separator, ...
               'UserData',showpdfdoc.gir, ...
               'Enable','on', ...
               'Tag','MenuDocGIREstimate');
            pdf_separator = 'off';
         end;
         %
         if strcmp(showpdfdoc.var_decomp,'')==0;
            svar.pdf_var_decomp = uimenu('Parent',helpmenu, ...
               'Label','&Variance Decompositions - PDF file', ...
               'Callback','show_pdf', ...
               'Separator',pdf_separator, ...
               'UserData',showpdfdoc.var_decomp, ...
               'Enable','on', ...
               'Tag','MenuDocVDEstimate');
            pdf_separator = 'off';
         end;
         %
         % add further pdf-file openers here when they are available:
         %
      end;

      xline([1 scrsz(4)/2+139 (scrsz(3)/2)+180],[0.5 1]);

      %
      % Outer axis
      %

      axesbox([2 2 (scrsz(3)/2)+178 scrsz(4)/2+125],'Maximum Likelihood Results',135,[0.5 1]);

      %
      % Outer axis for showing B matrix
      %

      axesbox([14 312 scrsz(3)/2+152 183],'Current B Matrix',75,[0.5 1]);

      %
      % Outer axis for selection type of identifying assumptions
      %

      axesbox([14 203 scrsz(3)/2+152 95],'Restrictions Selection',105,[0.5 1]);

      svar.sr_rest = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 261 270 25], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','radiobutton', ...
         'Value',1, ...
         'Enable',sr_enable, ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','svargui check_rest', ...
         'String','Contemporaneous restrictions:', ...
         'Tag','JointRestRadio');

      %
      % show the type of restrictions in a text control
      %

      show_text([300 261 200 25],curr_ini.gui_foreground_color,'R_{c}\cdotvec(B) = r_c');

      svar.lr_rest = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 236 270 25], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','radiobutton', ...
         'Value',0, ...
         'Enable',lr_enable, ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','svargui check_rest', ...
         'String','Long-run restrictions:', ...
         'Tag','LRRestRadio');

      if cm.r==0;
         c_str = '\Gamma(1)^{-1}';
      elseif cm.r==cm.n;
         if cm.cointvalue==1;
            c_str = '-\alpha^{-1}';
         else;
            c_str = '-\Pi^{-1}';
         end;
      else;
         c_str = '\beta_{\perp}(\alpha_{\perp}\prime\Gamma(1)\beta_{\perp})^{-1}\alpha_{\perp}\prime';
      end;

      show_text([300 236 200 26],curr_ini.gui_foreground_color,['R_{l}\cdotvec(A) = r_l, where A = C\cdotB and C = ' c_str]);

      svar.joint_rest = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 211 270 25], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','radiobutton', ...
         'Value',0, ...
         'Enable',joint_enable, ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'Callback','svargui check_rest', ...
         'String','Contemporaneous & long-run restrictions:', ...
         'Tag','JointRestRadio');

      show_text([300 211 200 25],curr_ini.gui_foreground_color,'R_c\cdotvec(B) = r_c and R_l\cdotvec(A) = r_l',[0 0.3]);

      if (strcmp(sr_enable,'off')==1)&(strcmp(lr_enable,'on')==1);
         set(svar.sr_rest,'Value',0);
         set(svar.lr_rest,'Value',1);
      end;

      if (curr_ini.identify==2)&(strcmp(lr_enable,'on')==1);
         set(svar.sr_rest,'Value',0);
         set(svar.lr_rest,'Value',1);
      elseif (curr_ini.identify==3)&(strcmp(joint_enable,'on')==1);
         set(svar.sr_rest,'Value',0);
         set(svar.lr_rest,'Value',0);
         set(svar.joint_rest,'Value',1);
      end;

      %
      % Outer axis for text model info control
      %

      axesbox([14 14 scrsz(3)/2+152 175],'Model Information',85,[0.5 1]);

      %
      % Listbox for showing estimated beta
      %

      svar.betamat = uicontrol('Units','pixels', ...
         'BackgroundColor','white', ...
         'Position',[30 328 scrsz(3)/2+120 151], ...
         'Style','listbox', ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Max',2, ...
         'Value',[], ...
         'String',strvcat(' ',' ','   The B matrix has not been estimated.'), ...
         'Enable','inactive', ...
         'UserData',0, ...
         'HorizontalAlignment','center', ...
         'Tag','BetaListbox');

      %
      % Text control listing info about model
      %

      svar.model_header = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[30 30 scrsz(3)/2+120 145], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',cm.model_info.header, ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'HorizontalAlignment','left', ...
         'Tag','ModelInfoHeaderText');

      head_ext = get(svar.model_header,'Extent');
      if head_ext(1,4)<=145;
         new_pos = [30 30+(145-head_ext(1,4))/2 head_ext(1,3:4)];
      else;
         new_pos = [30 30 head_ext(1,3) 145];
      end;
      set(svar.model_header,'Position',new_pos);

      svar.model_info = uicontrol('Units','pixels', ...
         'BackgroundColor',get(0,'defaultuicontrolbackgroundcolor'), ...
         'Position',[35+new_pos(1,3) new_pos(1,2) scrsz(3)/2+115-new_pos(1,3) new_pos(1,4)], ...
         'FontSize',curr_ini.gui_fontsize, ...
         'FontName',curr_ini.gui_fontname, ...
         'FontWeight',curr_ini.gui_fontweight, ...
         'FontAngle',curr_ini.gui_fontangle, ...
         'Style','text', ...
         'String',cm.model_info.info, ...
         'ForegroundColor',curr_ini.gui_foreground_color, ...
         'HorizontalAlignment','left', ...
         'Tag','ModelInfoText');

      svar.toolbar = uitoolbar('Parent',svarguidlg, ...
                        'HandleVisibility','off', ...
                        'Interruptible','on', ...
                        'Clipping','on');

      %
      % Quit button
      %

      if cm.cointvalue==1;

         svar.quit = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Return', ...
                            'CData',button_images.return, ...
                            'ClickedCallback','svargui quit', ...
                            'Separator','off', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable','on', ...
                            'Tag','QuitButton');
      end;

      %
      %  continue button
      %

      svar.continue = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Done', ...
                         'CData',button_images.continue, ...
                         'ClickedCallback','svargui continue', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'UserData','waiting', ...
                         'Enable','on', ...
                         'Tag','ContinueButton');

      %
      % Graphics button
      %
      separator_str = 'on';

      if (cm.cointvalue==1)&(cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint);
         svar.graphs = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Cointegration Relations', ...
                            'CData',button_images.graph, ...
                            'ClickedCallback','svargui graphs', ...
                            'Separator','on', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'UserData','working', ...
                            'Enable','on', ...
                            'Tag','Graphics');
         separator_str = 'off';
      end;

      svar.permcomp = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Permanent and Transitory Components', ...
                         'CData',button_images.permcomp, ...
                         'ClickedCallback','svargui permcomp', ...
                         'Separator',separator_str, ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',mypermcompenable, ...
                         'Tag','PermCompGraphics');

      svar.residgraph = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Residuals', ...
                         'CData',button_images.histogram, ...
                         'ClickedCallback','svargui residuals', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'UserData','working', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ResidGraphics');

      %
      % Forecasting button
      %

      svar.forecast_button = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Forecasting', ...
                         'CData',button_images.forecast, ...
                         'ClickedCallback','svargui forecast', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','ForecastGraph');

      if cm.q1+cm.q0>0;
         set(svar.forecast_button,'Enable','off');
      end;

      %
      % Chow test button
      %

      svar.chow = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Recursive Chow Tests', ...
                         'CData',button_images.chow, ...
                         'ClickedCallback','svargui chow', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecChowGraph');

      svar.ploberger = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Recursive Fluctuation Test', ...
                         'CData',button_images.fluctest, ...
                         'ClickedCallback','svargui ploberger', ...
                         'Separator','off', ...
                         'Interruptible','on', ...
                         'UserData',thisexit, ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','RecPlobGraph');

      if (cm.r<cm.n)&((cm.r>0)|(cm.p>1));
   
         %
         % C restrictions
         %

         svar.restrict_c = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','C Restrictions', ...
                            'CData',button_images.c_rest, ...
                            'ClickedCallback','svargui restrict_c', ...
                            'Separator','on', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable','on', ...
                            'Tag','RestrictC');

         svar.c_openrest = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Open C Restrictions', ...
                            'CData',button_images.c_open, ...
                            'ClickedCallback','svargui c_openrest', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable','on', ...
                            'Tag','OpenCRest');

         svar.c_saverest = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Save C Restrictions', ...
                            'CData',button_images.save_c, ...
                            'ClickedCallback','svargui c_saverest', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable','on', ...
                            'Tag','SaveCRest');

         %
         % Estimate the C matrix under linear restrictions
         %

         svar.c_estimate = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Estimate C', ...
                            'CData',button_images.c_est, ...
                            'ClickedCallback','svargui c_estimate', ...
                            'Separator','on', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable',estimate_c_enable, ...
                            'Tag','EstC');

         svar.c_undo = uipushtool('Parent',svar.toolbar, ...
                            'TooltipString','Undo C', ...
                            'CData',button_images.c_undo, ...
                            'ClickedCallback','svargui c_undo', ...
                            'Interruptible','on', ...
                            'Clipping','on', ...
                            'Enable','off', ...
                            'Tag','UndoC');

      end;

      %
      % Identification
      %

      svar.restrict_sr = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Contemporaneous Restrictions', ...
                         'CData',button_images.sr_rest, ...
                         'ClickedCallback','svargui restrict_sr', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','IdentifySR');

      svar.sr_openrest = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Open Contemporaneous Restrictions', ...
                         'CData',button_images.sr_open, ...
                         'ClickedCallback','svargui sr_openrest', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','OpenSR');

      svar.sr_saverest = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Save Contemporaneous Restrictions', ...
                         'CData',button_images.save_sr, ...
                         'ClickedCallback','svargui sr_saverest', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','SaveSR');

      svar.restrict_lr = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Long-Run Restrictions', ...
                         'CData',button_images.lr_rest, ...
                         'ClickedCallback','svargui restrict_lr', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','IdentifyLR');

      svar.lr_openrest = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Open Long-Run Restrictions', ...
                         'CData',button_images.lr_open, ...
                         'ClickedCallback','svargui lr_openrest', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','OpenLR');

      svar.lr_saverest = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Save Long-Run Restrictions', ...
                         'CData',button_images.save_lr, ...
                         'ClickedCallback','svargui lr_saverest', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','on', ...
                         'Tag','SaveLR');


      if (cm.r==0)&(cm.p==1);
         set(svar.restrict_lr,'Enable','off');
         set(svar.lr_openrest,'Enable','off');
         set(svar.lr_saverest,'Enable','off');
      end;

      %
      % Estimate the B matrix under identifying restrictions
      %

      svar.struc_estimate = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Estimate B', ...
                         'CData',button_images.struc_est, ...
                         'ClickedCallback','svargui struc_estimate', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',estimateenable, ...
                         'Tag','EstB');

      svar.struc_undo = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Undo B', ...
                         'CData',button_images.struc_undo, ...
                         'ClickedCallback','svargui struc_undo', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','off', ...
                         'Tag','UndoB');

      %
      % Impulse responses
      %

      svar.irfunction = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Impulse Responses', ...
                         'CData',button_images.irfunction, ...
                         'ClickedCallback','svargui irfunction', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable','off', ...
                         'Tag','ImpulseGraphics');

      svar.signshock = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Select Signs for Shocks', ...
                         'CData',button_images.signshock, ...
                         'ClickedCallback','svargui signshock', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',estimateenable, ...
                         'Tag','SignShockButton');

      svar.help = uipushtool('Parent',svar.toolbar, ...
                         'TooltipString','Help Topics', ...
                         'CData',button_images.help, ...
                         'ClickedCallback','help_fcn(''identification'')', ...
                         'Separator','on', ...
                         'Interruptible','on', ...
                         'Clipping','on', ...
                         'Enable',helpenable, ...
                         'Tag','HelpButton');

      if (cm.p==1)&((cm.r+cm.d)==0);
         set(svar.constancytest,'Enable','off');
         set(svar.menuchow,'Enable','off');
         set(svar.chow,'Enable','off');
         set(svar.menuploberger,'Enable','off');
         set(svar.ploberger,'Enable','off');
      end;

%      if cm.r==cm.n;
%         set(svar.restrict_c,'Visible','off');
%         set(svar.c_openrest,'Visible','off');
%         set(svar.c_saverest,'Visible','off');
%         set(svar.c_estimate,'Visible','off');
%         set(svar.c_undo,'Visible','off');
%      end;

      set(svarguidlg,'UserData',svar,...
         'HandleVisibility','callback');

   case 'c_estimate'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         abort = '';
         %
         % collect restrictions on C
         %
         file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         G = loaddata(file);
         [Rmat_C,rvec_C] = rmatrix(G,cm.n,cm.n);
         %
         % store previous values
         %
         prev.Psi = cm.Psi;
         prev.alpha = cm.alpha;
         prev.beta = cm.beta;
         prev.Omega = cm.Omega;
         prev.epsilon = cm.epsilon;
         prev.Sigma_theta = cm.Sigma_theta;
         prev.restrict = cm.restrict;
         %
         % reset parameter values to original
         %
         cm.Psi = cm.old_values.Psi;
         cm.alpha = cm.old_values.alpha;
         cm.beta = cm.old_values.beta;
         cm.Omega = cm.old_values.Omega;
         cm.Sigma_theta = cm.old_values.Sigma_theta;
         cm.epsilon = cm.old_values.epsilon;
         %
         % impose restrictions on C
         %
         setlogtext(strvcat(' ','Computing restricted C estimates...'));
         [theta,Sigma_theta,beta,stdbeta,C,Sigma_C,num_c_rest,num_a_rest,infomat,abort] = c_restrict(cm.Z0,cm.Z1,cm.Z2,cm.restrict,cm.beta,[cm.Psi cm.alpha],Rmat_C,rvec_C,cm.p,cm.pe,cm.d,cm.dcoint,cm.q0,cm.q1,cm.modelsdir,cm.modelfile,1);
         %
         if strcmp(abort,'cancel')==0;
            if (curr_ini.reestbeta==0)|(cm.restrict(1,2)==0);
               stdbeta = cm.old_values.stdbeta;
            end;
            if curr_ini.noprint==0;
               txt = strvcat(' ','Computing tests...',' ','Please hold!');
               mywait = waitdlg(txt,'information');
               waitf = get(mywait,'UserData');
            end;
            setlogtext(strvcat(['Number of iterations:               ' num2str(infomat(1,1),'%0.0f')],['Value of the convergence criterion: ' num2str(infomat(1,2))],'Computing tests of restrictions...'));
            Z3 = [];
            alpha = [];
            Psi = [];
            if cm.r>0;
               Z3 = beta'*cm.Z1;
               alpha = theta(:,size(theta,2)-cm.r+1:size(theta,2));
            end;
            if isempty(cm.Z2)==0;
               Z3 = [cm.Z2;Z3];
               Psi = theta(:,1:size(theta,2)-cm.r);
            end;
            epsilon = cm.Z0-(theta*Z3);
            Omega = (1/cm.T)*epsilon*epsilon';
            LR = cm.T*log(det(Omega)/det(cm.Omega));
            %
            dfc1 = round((((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.dfbeta)/cm.n);
            dfa = round((num_a_rest+num_c_rest)/cm.n);
            cm.df1 = cm.n*(cm.p-1)+cm.r+cm.q1*cm.p+cm.q0*(cm.p+1)+cm.d+dfc1-dfa;
            %
            if curr_ini.sscorrect==0;
               pval = 1-chis_cdf(LR,num_c_rest);
            else;
               LR = ((cm.T-cm.df1)/(cm.T*num_c_rest))*LR;
               pval = 1-fdis_cdf(LR,num_c_rest,cm.T-cm.df1);
            end;
            %
            % Compute eigenvalues from companion matrix
            %
            eigenvalues = companionrest(alpha,Psi,beta,cm.p,cm.n,cm.d);
            eigenvalues2 = eigenvalues;
            eigenvalues = abs(sort(eigenvalues));
            i = size(eigenvalues,1);
            while i>=1;
               evalue = eigenvalues(i,1);
               if strcmp(num2str(evalue,'%1.10f'),'1.0000000000')==1;
                  i = i-1;
               else;
                  i = 0;
               end;
            end;
            %
            [lnl,ldOmega] = loglike(epsilon);
            if curr_ini.noprint==0;
               sp_str = spacestr(curr_ini.decimals-2);
               C_std = sqrt(abs(diag(Sigma_C)));
               C_std = vecinv(C_std,cm.n);
               for i=1:cm.n;
                  for j=1:cm.n;
                     if abs(C_std(i,j))<1.0e-005;
                        C_std(i,j) = 0;
                        C(i,j) = mystr2num(strtrim(num2str(C(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
                     end;
                  end;
               end;
               C_t = inf2nan(C./C_std);
               %
               % write to output file
               %
               fout = fopen(cm.outputfile,'at');
               if curr_ini.texoutputformat>=2;
                  if cm.dfbeta==0;
                     fprintf(fout,'\\subsection*{Restrictions on $C$}\n\n\\header{For exactly identified $\\beta$ with $r$=%0.0f}\n\n',cm.r);
                  else;
                     fprintf(fout,'\\subsection*{Restrictions on $C$}\n\n\\header{For over-identified $\\beta$ with $r$=%0.0f}\n\n',cm.r);
                  end;
                  fprintf(fout,'\\header{Estimated $C$ coefficients:}\n\n');
                  myprintstr = '%s';
                  myprintstr2 = '';
                  myprintstr3 = '';
                  r_str = 'r';
                  for i=1:cm.n;
                     myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                     myprintstr2 = [myprintstr2 ' & (%0.' num2str(curr_ini.decimals,'%0.0f') 'f)'];
                     myprintstr3 = [myprintstr3 ' & [%0.' num2str(curr_ini.decimals,'%0.0f') 'f]'];
                     r_str = [r_str 'r'];
                  end;
                  fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                  for i=1:cm.n;
                     fprintf(fout,[myprintstr ' \\\\ \n'],cm.myvars(i,:),C(i,:));
                     fprintf(fout,[myprintstr2 ' \\\\ \n'],C_std(i,:));
                     fprintf(fout,[myprintstr3 ' \\\\ \n'],C_t(i,:));
                  end;
                  fprintf(fout,'\\end{tabular}\n\n');
                  %
                  if curr_ini.reestbeta==1;
                     if cm.restrict(1,2)==1;
                        fprintf(fout,'\\header{Re-estimated restricted cointegration relations:}\n\n');
                     else;
                        fprintf(fout,'\\header{Re-estimated cointegration relations:}\n\n');
                     end;
                     myprintstr = '%0.0f';
                     r_str = 'r';
                     for j=1:size(beta,1);
                        myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                        r_str = [r_str 'r'];
                     end;
                     myprintstr = [myprintstr ' \\\\\n'];
                     fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                     fprintf(fout,['relation & ' strrep(strtrim([cm.evars ' ' cm.xvars1 ' ' cm.restvec]),' ',' & ') ' \\\\\n']);
                     if cm.restrict(1,2)==0;
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' normbeta(beta)']');
                     else;
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' beta']');
                        fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{ } \\\\\n']);
                        fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{with asymptotic conditional standard errors:} \\\\\n']);
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' stdbeta']');
                     end;
                     fprintf(fout,'\\end{tabular}\n\n');
                  end;
                  %
                  fprintf(fout,'\\vspace*{2mm}\n\n\\begin{tabular}{ll}\n');
                  fprintf(fout,'Number of iterations for algorithm: & %0.0f \\\\\n',infomat(1,1));
                  fprintf(fout,['Value of convergence criterion ($\\max |R\\vec(C)-r|$): & ' num2str(infomat(1,2)) ' \\\\\n']);
                  fprintf(fout,'\\end{tabular}\n\n');
                  fprintf(fout,'\\header{LR-type statistics}\n\n');
                  fprintf(fout,'\\begin{tabular}{ll}\n');
                  fprintf(fout,'$H_0$: & $C$ restrictions are true \\\\\n');
                  fprintf(fout,'$H_A$: & $C$ restrictions are false \\\\\n');
                  if curr_ini.sscorrect==0;
                     fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],num_c_rest,LR);
                     fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval);
                  else;
                     fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],num_c_rest,cm.T-cm.df1,LR);
                     fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],pval);
                  end;
                  fprintf(fout,'\\end{tabular}\n\n');
                  fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
               else;
                  if cm.dfbeta==0;
                     fprintf(fout,'RESTRICTIONS ON C (for exactly identified beta with rank=%0.0f)\n\n',cm.r);
                  else;
                     fprintf(fout,'RESTRICTIONS ON C (for over-identified beta with rank=%0.0f)\n\n',cm.r);
                  end;
                  fprintf(fout,'Estimated C coefficients:\n\n');
                  myprintstr = ' %s';
                  for i=1:cm.n;
                     myprintstr = [myprintstr '    %' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                  end;
                  for i=1:cm.n;
                     fprintf(fout,[myprintstr '\n'],cm.myvars(i,:),C(i,:));
                  end;
                  fprintf(fout,'\nwith asymptotic standard errors:\n\n');
                  for i=1:cm.n;
                     fprintf(fout,[myprintstr '\n'],cm.myvars(i,:),C_std(i,:));
                  end;
                  fprintf(fout,'\nt-ratios:\n\n');
                  for i=1:cm.n;
                     fprintf(fout,[myprintstr '\n'],cm.myvars(i,:),C_t(i,:));
                  end;
                  fprintf(fout,'\n');
                  %
                  if curr_ini.reestbeta==1;
                     if cm.restrict(1,2)==1;
                        fprintf(fout,'Re-estimated restricted cointegration relations:\n');
                     else;
                        fprintf(fout,'Re-estimated cointegration relations:\n');
                     end;
                     fprintf(fout,'\n');
                     myprintstr = '%2.0f';
                     j = 2;
                     while j<=size(beta,1)+1;
                        myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                        j = j+1;
                     end;
                     myprintstr = [myprintstr '\n'];
                     if cm.restrict(1,2)==1;
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' beta']');
                     else;
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' normbeta(beta)']');
                     end;
                     fprintf(fout,'\n');
                     if cm.restrict(1,2)==1;
                        fprintf(fout,'with asymptotic conditional standard errors:\n');
                        fprintf(fout,'\n');
                        fprintf(fout,myprintstr,[(1:1:size(beta,2))' stdbeta']');
                        fprintf(fout,'\n');
                     end;
                  end;
                  %
                  fprintf(fout,'Number of iterations for algorithm:               %0.0f\n',infomat(1,1));
                  fprintf(fout,['Value of convergence criterion (max|R*vec(C)-r|): ' num2str(infomat(1,2)) '\n\n']);
                  fprintf(fout,'  LR-type statistics\n  ~~~~~~~~~~~~~~~~~~\n\n');
                  fprintf(fout,'Null hypothesis:        C restrictions are true\n');
                  fprintf(fout,'Alternative hypothesis: C restrictions are false\n\n');
                  if curr_ini.sscorrect==0;
                     fprintf(fout,['LR(%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],num_c_rest,LR);
                     fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval);
                  else;
                     fprintf(fout,['F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],num_c_rest,cm.T-cm.df1,LR);
                     fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],pval);
                  end;
                  fprintf(fout,'\n===============================================================================\n\n');
               end;
               fclose(fout);
               if curr_ini.shortdyn==1;
                  txt = strvcat(' ','Computing short run dynamics.','Please hold!');
                  set(waitf.text,'String',txt);
                  shortstr = shortrun(alpha,Psi,Omega,beta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,0,Sigma_theta);
                  fout = fopen(cm.outputfile,'at');
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\n\\ruler\n\n%%==============================================================================\n\n');
                  else;
                     fprintf(fout,'\n===============================================================================\n\n');
                  end;
                  fclose(fout);
               end;
               %
               fout = fopen(cm.outputfile,'at');
               if curr_ini.texoutputformat>=2;
                  fprintf(fout,'\\subsection*{Value of the maximized log-likelihood function:}\n\n');
                  fprintf(fout,'\\begin{tabular}{ll}\n');
                  fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%0.0f,r=%0.0f,\\mbox{\\rm \\#free } \\alpha \\mbox{\\rm \\hspace*{1mm} params}=%0.0f,\\mbox{\\rm \\#free } \\beta \\mbox{\\rm \\hspace*{1mm} params}=%0.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,cm.r,cm.n*cm.r-num_a_rest,(cm.n+cm.q1+cm.dcoint-cm.r)*cm.r-cm.dfbeta,lnl);
                  fprintf(fout,[' & $\\ln(\\det(\\Omega))$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],ldOmega);
                  fprintf(fout,'\\end{tabular}\n\n');
                  fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
               else;
                  fprintf(fout,'Value of the maximized log-likelihood function:\n\n');
                  fprintf(fout,['   lnL(lags=%1.0f,rank=%1.0f,#free alpha params=%1.0f,#free beta params=%1.0f)=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,cm.r,cm.n*cm.r-num_a_rest,(cm.n+cm.q1+cm.dcoint-cm.r)*cm.r-cm.dfbeta,lnl);
                  fprintf(fout,['   ln(det(Omega))=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],ldOmega);
                  fprintf(fout,'===============================================================================\n\n');
               end;
               fclose(fout);
               %
               % compute tests for additional deterministic variables
               %
               if (curr_ini.dummytests==1)&(cm.datad>0);
                  oldrestrict = cm.restrict;
                  oldepsilon = cm.epsilon;
                  cm.restrict(1,3) = 1;
                  cm.epsilon = epsilon;
                  txt = strvcat(' ','Computing exclusion tests for additional deterministics.','Please hold!');
                  set(waitf.text,'String',txt);
                  print_exclude_dummies(cm,curr_ini);
                  if (cm.r>0)&(cm.r<cm.n);
                     txt = strvcat(' ','Computing long-run tests for additional deterministics.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_longrun_dummies(cm,curr_ini);
                  end;
                  cm.restrict = oldrestrict;
                  cm.epsilon = oldepsilon;
               end;
               %
               % Calculate lag-order test
               %
               old_restrict = cm.restrict;
               cm.restrict(1,3) = 1;
               if curr_ini.lagtest>0;
                  setlogtext('Computing lag order tests...');
                  txt = strvcat(' ','Computing lag order tests.','Please hold!');
                  set(waitf.text,'String',txt);
                  print_lagordertest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,cm.restrict(1,1));
               end;
               if curr_ini.gctest==1;
                  setlogtext('Computing Granger causality tests...');
                  txt = strvcat(' ','Computing Granger causality tests.','Please hold!');
                  set(waitf.text,'String',txt);
                  print_gctest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,cm.restrict(1,1));
                  if (cm.n>2)&((cm.r>0)|(cm.p>1));
                     setlogtext('Computing multi-step Granger causality tests...');
                     txt = strvcat(' ','Computing multi-step Granger causality tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_mgctest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.restrict(1,1),1);
                  end;
               end;
               if curr_ini.excludediffs==1;
                  if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
                     setlogtext('Computing exclusion tests for first differences...');
                     txt = strvcat(' ','Computing exclusion tests for first differences.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_firstdiffstests(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.df1,1);
                  end;
               end;
               if cm.r<cm.n;
                  if curr_ini.weakexo==1;
                     setlogtext('Computing weak exogeneity tests...');
                     txt = strvcat(' ','Computing weak exogeneity tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_weakexo(cm,curr_ini,beta,alpha,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df1,cm.restrict(1,1));
                  end;
               end;
               %
               if (cm.r<cm.n)&(cm.p>1);
                  if curr_ini.commoncycles==1;
                     oldepsilon = cm.epsilon;
                     cm.epsilon = epsilon;
                     setlogtext('Computing common cycle tests...');
                     txt = strvcat(' ','Computing common cycle tests.','Please hold!');
                     set(waitf.text,'String',txt);
                     print_commoncycles(cm,curr_ini,alpha,Psi,beta);
                     cm.epsilon = oldepsilon;
                  end;
               end;
               cm.restrict = old_restrict;
               if curr_ini.restat==1;
                  txt = strvcat(' ','Computing residual statistics.','Please hold!');
                  set(waitf.text,'String',txt);
               end;
               %
               % residual statistics
               %
               print_residual_analysis(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
               %
               % specification tests
               %
               if sum(cm.spectest)>0;
                  txt = strvcat(' ','Computing specification tests.','Please hold!');
                  set(waitf.text,'String',txt);
               end;
               %
               if cm.spectest(1,1)==1;
                  setlogtext('Computing serial correlation tests...');
                  print_corrtest(cm,curr_ini,epsilon,beta,cm.dfbeta,cm.dfalpha,cm.df1);
               end;
               %
               if cm.spectest(1,2)==1;
                  setlogtext('Computing ARCH tests...');
                  print_archtest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
               end;
               %
               if cm.spectest(1,3)==1;
                  setlogtext('Computing normality tests...');
                  print_normalitytest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df1);
               end;
               %
               if cm.spectest(1,4)==1;
                  if rank(beta'*beta,1.0e-004)==cm.r;
                     if cm.r>0;
                        Zb = beta'*cm.Z1;
                     else;
                        Zb = 0;
                     end;
                     fout = fopen(cm.outputfile,'at');
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,['\\subsection*{Parameter constancy tests}\n\n\\header{For $r$=' num2str(cm.r) ', lags=' num2str(cm.p) ', \\#free $\\beta$ params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ', \\#free $\\alpha$ params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') '}\n\n']);
                     else;
                        fprintf(fout,['PARAMETER CONSTANCY TESTS\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') ')\n\n']);
                     end;
                     fclose(fout);
                     if curr_ini.lstrtest==1;
                        setlogtext('Computing LSTR1 constancy tests...');
                        print_lstr1test(cm,curr_ini,epsilon,Zb,cm.df1);
                     end;
                     %
%                     if (cm.q1+cm.q0==0)&(curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                     if (curr_ini.nyblom==1)&(curr_ini.nyblombeta==1);
                        setlogtext('Computing Nyblom constancy tests for beta...');
                        cm.Theta = [Psi alpha];
                        set(svar.estalpha,'UserData',cm.Theta);
                        cm.oldrestrict = cm.restrict;
                        cm.restrict(1,3) = 1;
                        if cm.restrict(1,1)==1;
                           print_alpha_nyblom(cm,curr_ini);
                        else;
                           print_nyblom(cm,curr_ini);
                        end;
                        cm.restrict = cm.oldrestrict;
                        cm.Theta = cm.lastTheta;
                        set(svar.estalpha,'UserData',cm.Theta);
                     end;
                     %
                     if cm.p+cm.r+cm.d>1;
                        if curr_ini.chow==1;
                           setlogtext('Computing Chow tests...');
                           print_chow(cm,curr_ini,beta,1);
                        end;
                        %
                        setlogtext('Computing fluctuation tests...');
                        print_ploberger(cm,curr_ini,beta,1);
                     end;
                  else;
                     fout = fopen(cm.outputfile,'at');
                     myrank = rank(beta'*beta,1.0e-004);
                     setlogtext(strvcat(' ','WARNING: The estimated beta matrix is badly identified...',['         Numerical rank is ' num2str(myrank(1,1)) '...']));
                     txt = ['The estimated restricted beta matrix is badly identified. The rank of beta''beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Please consider some other restrictions.'];
                     about(txt,'warning','Badly identified beta matrix',120,350);
                     if curr_ini.texoutputformat>=2;
                        fprintf(fout,'\\vspace*{3mm}\n\\newline\n\\noindent\n\\textbf{WARNING:} Parameter constancy tests have not been calculated. The\n');
                        fprintf(fout,'         estimated restricted $\\beta$ matrix is badly identified.\n');
                        fprintf(fout,'         The rank of $\\beta''\\beta$ is %0.0f, i.e. less than %0.0f, when\n',myrank,cm.r);
                        fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n\n');
                        fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                     else;
                        fprintf(fout,'WARNING: Parameter constancy tests have not been calculated. The\n');
                        fprintf(fout,'         estimated restricted beta matrix is badly identified.\n');
                        fprintf(fout,'         The rank of beta''beta is %1.0f, i.e. less than %1.0f, when\n',myrank,cm.r);
                        fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n');
                        fprintf(fout,'\n===============================================================================\n\n');
                     end;
                     fclose(fout);
                  end;
               end;
            end;
            %
            % close wait dialogue
            %
            if curr_ini.noprint==0;
               set(waitf.text,'UserData','done');
               delete(mywait);
               drawnow;
            end;
            %
            % display results
            %
            thiscdlg = cdlg('init',cm,infomat,pval,LR,epsilon,Psi,alpha,beta,Sigma_theta,C,Sigma_C,num_c_rest,evalue);
            crest = get(thiscdlg,'UserData');
            userestc = get(crest.betamat,'UserData');
            set(crest.alphamat,'UserData','done');
            delete(thiscdlg);
            drawnow;
            if curr_ini.noprint==0;
               fout = fopen(cm.outputfile,'at');
            end;
            if userestc==1;
               %
               % setting parameters and controls
               %
               setlogtext(strvcat(' ','Using the estimated parameters under C restrictions...'));
               cm.restrict(1,3) = 1;
               cm.beta = beta;
               cm.alpha = alpha;
               cm.Psi = Psi;
               cm.Omega = Omega;
               cm.epsilon = epsilon;
               cm.Sigma_theta = Sigma_theta;
               cm.Theta = [Psi alpha];
               cm.df = cm.df1;
               cm.evalue = evalue;
               if cm.restrict(1,2)==1;
                  cm.bZ1t = cm.beta'*cm.Z1t;
                  cm.bR1t = cm.beta'*cm.R1t;
               else;
                  cm.bZ1t = normbeta(cm.beta)'*cm.Z1t;
                  cm.bR1t = normbeta(cm.beta)'*cm.R1t;
               end;
               if isempty(stdbeta)==0;
                  cm.stdbeta = stdbeta;
               end;
               %
               % saving parameters for B estimation initialization
               %
               cm.C_values.Psi = cm.Psi;
               cm.C_values.alpha = cm.alpha;
               cm.C_values.beta = cm.beta;
               cm.C_values.Omega = cm.Omega;
               cm.C_values.Sigma_theta = cm.Sigma_theta;
               cm.C_values.epsilon = cm.epsilon;
               cm.C_values.evalue = cm.evalue;
               cm.C_values.df = cm.df;
               if isempty(stdbeta)==0;
                  cm.C_values.stdbeta = cm.stdbeta;
               end;
               %
               set(svar.c_undo,'Enable','on');
               set(svar.c_undomenu,'Enable','on');
               %
               set(svar.estalpha,'UserData',cm.Theta);
               %
               cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,lnl,cm.evalue);
               set(svar.model_info,'String',cm.model_info.info);
               %
               % printing
               %
               if curr_ini.noprint==0;
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\\header{You selected to use the estimated parameters under $C$ restrictions}\n\n');
                  else;
                     fprintf(fout,'You selected to use the estimated parameters under C restrictions\n\n');
                  end;
                  [eigenvalues1,AscInd] = sort(abs(eigenvalues2));
                  nr = size(AscInd,1);
                  DescInd = zeros(nr,1);
                  i = 1;
                  while i<=nr;
                     DescInd(i)=AscInd(nr+1-i);
                     i =i+1;
                  end;
                  eigenvalues2 = eigenvalues2(DescInd);
                  if curr_ini.texoutputformat>=2;
                     fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
                     fprintf(fout,'\\begin{tabular}{rrrr}\n');
                     fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
                     fprintf(fout,'number & real & imaginary & modulus \\\\\n');
                     fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                     fprintf(fout,'\\end{tabular}\n\n');
                  else;
                     fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
                     fprintf(fout,'Eigenvalues\n');
                     fprintf(fout,['number    real   ' sp_str '   imaginary   ' sp_str '  modulus\n\n']);
                     fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                     fprintf(fout,'\n');
                  end;
               end;
            else;
               cm.Psi = prev.Psi;
               cm.alpha = prev.alpha;
               cm.beta = prev.beta;
               cm.Omega = prev.Omega;
               cm.epsilon = prev.epsilon;
               cm.Sigma_theta = prev.Sigma_theta;
               cm.restrict = prev.restrict;
%               if strcmp(get(svar.c_undo,'Enable'),'off')==1;
%                  cm.restrict(1,3)==0;
%               else;
%                  cm.restrict(1,3)==1;
%               end;
               setlogtext(strvcat(' ','Skipping the estimated parameters under C restrictions...'));
            end;
            if curr_ini.noprint==0;
               if curr_ini.texoutputformat>=2;
                  fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
               else;
                  fprintf(fout,'*******************************************************************************\n\n');
               end;
               fclose(fout);
            end;
         else;
            cm.Psi = prev.Psi;
            cm.alpha = prev.alpha;
            cm.beta = prev.beta;
            cm.Omega = prev.Omega;
            cm.epsilon = prev.epsilon;
            cm.Sigma_theta = prev.Sigma_theta;
            cm.restrict = prev.restrict;
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'c_undo'
      if cm.running==0;
         cm.running = 1;
         %
         % resetting parameters and controls
         %
         setlogtext(strvcat(' ','Skipping the estimated parameters under C restrictions...'));
         cm.restrict(1,3) = 0;
         cm.Psi = cm.old_values.Psi;
         cm.alpha = cm.old_values.alpha;
         cm.Theta = [cm.Psi cm.alpha];
         cm.beta = cm.old_values.beta;
         cm.Omega = cm.old_values.Omega;
         cm.Sigma_theta = cm.old_values.Sigma_theta;
         cm.epsilon = cm.old_values.epsilon;
         cm.evalue = cm.old_values.evalue;
         if isfield(cm,'stdbeta')==1;
            cm.stdbeta = cm.old_values.stdbeta;
         end;
         cm.df = cm.old_values.df;
         if cm.restrict(1,2)==1;
            cm.bZ1t = cm.beta'*cm.Z1t;
            cm.bR1t = cm.beta'*cm.R1t;
         else;
            cm.bZ1t = normbeta(cm.beta)'*cm.Z1t;
            cm.bR1t = normbeta(cm.beta)'*cm.R1t;
         end;
         set(svar.c_undo,'Enable','off');
         set(svar.c_undomenu,'Enable','off');
         %
         set(svar.estalpha,'UserData',cm.Theta);
         %
         cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,loglike(cm.epsilon),cm.evalue);
         set(svar.model_info,'String',cm.model_info.info);
         %
         if curr_ini.noprint==0;
            fout = fopen(cm.outputfile,'at');
            if curr_ini.texoutputformat>=2;
               fprintf(fout,'\\header{But then you changed your mind about the parameters under $C$ restrictions!}\n\n');
               fprintf(fout,'\\ruler\n\n%%******************************************************************************\n\n');
            else;
               fprintf(fout,'But then you changed your mind about the parameters under C restrictions!\n');
               fprintf(fout,'\n*******************************************************************************\n\n');
            end;
            fclose(fout);
         end;
         %
         cm.running = 0;
      end;
   case 'c_saverest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_save_struc_restgui = save_struc_restgui('init',cm,'c_matrix');
         save_struc_rest = get(this_save_struc_restgui,'UserData');
         set(save_struc_rest.gmat,'UserData','done');
         killguipid(this_save_struc_restgui);
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'c_openrest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_open_struc_rest = open_struc_restgui('init',cm,'c_matrix');
         open_struc_rest = get(this_open_struc_rest,'UserData');
         set(open_struc_rest.gmat,'UserData','done');
         killguipid(this_open_struc_rest);
         %
         % check if restricts file exists
         %
         file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            set(svar.c_estimate,'Enable','on');
            set(svar.c_estimatemenu,'Enable','on');
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'restrict_c'
      if cm.running==0;
         cm.running = 1;
         %
         % Setting up a default C-rest matrix
         %
         cm.F = zeros(1,6);
         %
         % Checking if these already exists a file, and if so
         % check it and then load it
         %
         file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            cm.F = loaddata(file);
         end;
         %
         %  Hide current window
         %
         set(svarguidlg,'Visible','off');
         drawnow;
         rd = 'redraw';
         while strcmp(rd,'redraw')==1;
            thisfmat = identmatdlg('init',cm,'c_matrix');
            fmat = get(thisfmat,'UserData');
            cancel = get(fmat.cancel,'UserData');
            rd = get(fmat.numres,'UserData');
            cm.F = get(fmat.text,'UserData');
            set(fmat.numelem,'UserData','done');
            killrestpid(thisfmat);
            drawnow;
         end;
         set(svarguidlg,'Visible','on');
         if cancel==0;
            file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               cm.F = loaddata(file);
               [Rmat,rvec] = rmatrix(cm.F,cm.n,cm.n);
            else;
               Rmat = 0;
               cm.F = [];
            end;
            %
            if (rank(Rmat)==size(cm.F,1))&(rank(Rmat)>=1);
               set(svar.c_estimate,'Enable','on');
               set(svar.c_estimatemenu,'Enable','on');
            else;
               set(svar.c_estimate,'Enable','off');
               set(svar.c_estimatemenu,'Enable','off');
            end;
         else;
            %
            % check if file exists
            %
            file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==0;
               set(svar.c_estimate,'Enable','off');
               set(svar.c_estimatemenu,'Enable','off');
            end;
         end;
         cm.running = 0;
      end;
   case 'vardecomp'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         abort = '';
         do_print = 0;
         if ((isfield(cm,'ir')==0)|(isempty(cm.ir)==1))&(isempty(cm.B_0)==0);
            do_print = 1;
            setlogtext(strvcat(['Computing impulse responses up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
            if curr_ini.method(1,1)==1;
               [ir,stdvec,betair,betair_std,abort,vd] = impulses_vlaar_fcn(cm,cm.type,cm.B_0,cm.Sigma_B0,cm.gamma,cm.C,cm.dC_theta,cm.h,cm.S,cm.R_mat,cm.R_l,cm.Psi,cm.alpha,cm.beta,cm.Sigma_theta,1);
               if strcmp(abort,'cancel')==0;
                  %
                  % get the value from the normal distribution corresponding to the
                  % chosen levels of significance c
                  %
                  c_val = norm_inv((100+cm.c)/200);
                  %
                  % Now we compute the upper and lower c percent confidence band size
                  %
                  cm.ir = ir;
                  cm.orig_ir = ir;
                  cm.stdvec = stdvec;
                  cm.vd = vd;
                  cm.betair = betair;
                  cm.orig_betair = betair;
                  cm.betair_std = betair_std;
               end;
            elseif curr_ini.method(1,2)==1;
               [ir,stdvec,betair,betair_std,abort,vd] = impulses_warne_fcn(cm,cm.type,cm.B_0,cm.Sigma_B0,cm.C,cm.dB_theta,cm.h,cm.S,cm.Psi,cm.alpha,cm.beta,cm.Sigma_theta,1);
               if strcmp(abort,'cancel')==0;
                  %
                  % get the value from the normal distribution corresponding to the
                  % chosen levels of significance c
                  %
                  c_val = norm_inv((100+cm.c)/200);
                  %
                  % Now we compute the upper and lower c percent confidence band size
                  %
                  cm.ir = ir;
                  cm.orig_ir = ir;
                  cm.stdvec = stdvec;
                  cm.vd = vd;
                  cm.betair = betair;
                  cm.orig_betair = betair;
                  cm.betair_std = betair_std;
               end;
            end;
            if strcmp(abort,'cancel')==0;
               for i=1:size(cm.ir,2);
                  for j=1:cm.n*cm.n;
                     if abs(cm.stdvec(j,i))<1.0e-008;
                        cm.stdvec(j,i) = 0;
                     end;
                  end;
               end;
               cm.upper = ir+(c_val*cm.stdvec);
               cm.lower = ir-(c_val*cm.stdvec);
               %
               if isempty(cm.betair)==0;
                  for i=1:size(cm.betair,2);
                     for j=1:cm.r*cm.n;
                        if abs(cm.betair_std(j,i))<1.0e-008;
                           cm.betair_std(j,i) = 0;
                        end;
                     end;
                  end;
                  %
                  cm.betair_upper = cm.betair+(c_val*cm.betair_std);
                  cm.betair_lower = cm.betair-(c_val*cm.betair_std);
               else;
                  cm.betair_upper = [];
                  cm.betair_lower = [];
               end;
            end;
         end;
         if strcmp(abort,'cancel')==0;
            if isfield(cm,'signshock')==1;
               H = change_ir_signs(cm.signshock,cm.orig_B_0,cm.n);
               cm.ir = kron(H',eye(cm.n))*cm.orig_ir;
               c_val = norm_inv((100+cm.c)/200);
               cm.upper = cm.ir+(c_val*cm.stdvec);
               cm.lower = cm.ir-(c_val*cm.stdvec);
               if isempty(cm.betair)==0;
                  cm.betair = kron(H',eye(cm.r))*cm.orig_betair;
                  cm.betair_upper = cm.betair+(c_val*cm.betair_std);
                  cm.betair_lower = cm.betair-(c_val*cm.betair_std);
               end;
            end;
            %
            % perform the variance decompositions
            %
            [vardec,std_vardec] = var_decomp(cm.n,cm.ir,cm.vd.Sigma_theta,cm.vd.Sigma_B0,cm.vd.partial);
            %
            % print to output-file if requested and not already done
            %
            if (curr_ini.noprint==0)&(do_print==1);
               print_vardecomp(vardec,std_vardec,cm.n,cm.myvars,cm.myshocks,cm.outputfile,curr_ini.texoutputformat);
            end;
            show_vardecomp(cm,vardec,std_vardec);
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'irfunction'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         abort = '';
         do_print = 0;
         if ((isfield(cm,'ir')==0)|(isempty(cm.ir)==1))&(isempty(cm.B_0)==0);
            do_print = 1;
            setlogtext(strvcat(['Computing impulse responses up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
            if curr_ini.method(1,1)==1;
               [ir,stdvec,betair,betair_std,abort,vd] = impulses_vlaar_fcn(cm,cm.type,cm.B_0,cm.Sigma_B0,cm.gamma,cm.C,cm.dC_theta,cm.h,cm.S,cm.R_mat,cm.R_l,cm.Psi,cm.alpha,cm.beta,cm.Sigma_theta,1);
               if strcmp(abort,'cancel')==0;
                  %
                  % get the value from the normal distribution corresponding to the
                  % chosen levels of significance c
                  %
                  c_val = norm_inv((100+cm.c)/200);
                  %
                  % Now we compute the upper and lower c percent confidence band size
                  %
                  cm.ir = ir;
                  cm.orig_ir = ir;
                  cm.stdvec = stdvec;
                  cm.vd = vd;
                  cm.betair = betair;
                  cm.orig_betair = betair;
                  cm.betair_std = betair_std;
               end;
            elseif curr_ini.method(1,2)==1;
               [ir,stdvec,betair,betair_std,abort,vd] = impulses_warne_fcn(cm,cm.type,cm.B_0,cm.Sigma_B0,cm.C,cm.dB_theta,cm.h,cm.S,cm.Psi,cm.alpha,cm.beta,cm.Sigma_theta,1);
               if strcmp(abort,'cancel')==0;
                  %
                  % get the value from the normal distribution corresponding to the
                  % chosen levels of significance c
                  %
                  c_val = norm_inv((100+cm.c)/200);
                  %
                  % Now we compute the upper and lower c percent confidence band size
                  %
                  cm.ir = ir;
                  cm.orig_ir = ir;
                  cm.stdvec = stdvec;
                  cm.vd = vd;
                  cm.betair = betair;
                  cm.orig_betair = betair;
                  cm.betair_std = betair_std;
               end;
            end;
            if strcmp(abort,'cancel')==0;
               for i=1:size(ir,2);
                  for j=1:cm.n*cm.n;
                     if abs(cm.stdvec(j,i))<1.0e-008;
                        cm.stdvec(j,i) = 0;
                     end;
                  end;
               end;
               cm.upper = ir+(c_val*cm.stdvec);
               cm.lower = ir-(c_val*cm.stdvec);
               %
               if isempty(cm.betair)==0;
                  for i=1:size(cm.betair,2);
                     for j=1:cm.r*cm.n;
                        if abs(cm.betair_std(j,i))<1.0e-008;
                           cm.betair_std(j,i) = 0;
                        end;
                     end;
                  end;
                  cm.betair_upper = cm.betair+(c_val*cm.betair_std);
                  cm.betair_lower = cm.betair-(c_val*cm.betair_std);
               else;
                  cm.betair_upper = [];
                  cm.betair_lower = [];
               end;
            end;
         end;
         if strcmp(abort,'cancel')==0;
            if isfield(cm,'signshock')==1;
               H = change_ir_signs(cm.signshock,cm.orig_B_0,cm.n);
               cm.ir = kron(H',eye(cm.n))*cm.orig_ir;
               c_val = norm_inv((100+cm.c)/200);
               cm.upper = cm.ir+(c_val*cm.stdvec);
               cm.lower = cm.ir-(c_val*cm.stdvec);
               if isempty(cm.betair)==0;
                  cm.betair = kron(H',eye(cm.r))*cm.orig_betair;
                  cm.betair_upper = cm.betair+(c_val*cm.betair_std);
                  cm.betair_lower = cm.betair-(c_val*cm.betair_std);
               end;
            end;
            %
            % perform the variance decompositions
            %
            [vardec,std_vardec] = var_decomp(cm.n,cm.ir,cm.vd.Sigma_theta,cm.vd.Sigma_B0,cm.vd.partial);
            %
            % print to output-file if requested and not already done
            %
            if (curr_ini.noprint==0)&(do_print==1);
               print_vardecomp(vardec,std_vardec,cm.n,cm.myvars,cm.myshocks,cm.outputfile,curr_ini.texoutputformat);
            end;
            thisgraph = ir_graphdlg('init',cm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'signshock'
      if cm.running==0;
         cm.running = 1;
         if get(svar.sr_rest,'Value')==1;
            type_var = 'short_run';
         elseif get(svar.lr_rest,'Value')==1;
            type_var = 'long_run';
         else;
            type_var = 'joint';
         end;
         if curr_ini.method(1,1)==1;
            [status1,num_res,tot_res,B_random] = test_ident_m1(cm,type_var);
            if (status1==1)&(num_res==floor((cm.n*(cm.n-1))/2));
               %
               % re-check status1
               %
               [status1,num_res,tot_res,B_random] = test_ident_m1(cm,type_var);
            end;
         %
         % 2. Re-estimate reduced form along with B
         %    (curr_ini.reestbeta tells us if beta should be updated or not)
         %
         elseif curr_ini.method(1,2)==1;
            [status1,num_res,tot_res,B_random,keep_rest,del_rest,use_psi_rest,C_rank] = test_ident_m2(cm,type_var);
         end;
         %
         if status1==0;
            oldsignshock = sign_shock_fcn(cm,B_random);
            this_sign = signdlg('init',cm,B_random);
            my_sign = get(this_sign,'UserData');
            set(my_sign.apply,'UserData','done');
            killguipid(this_sign);
            newsignshock = sign_shock_fcn(cm,B_random);
            check_sign_change = max(abs(oldsignshock(:,2)-newsignshock(:,2)));
            check_var_change = max(abs(oldsignshock(:,1)-newsignshock(:,1)));
            if (check_sign_change>0)&(check_var_change==0);
               setlogtext('You changed the pattern of required signs of shocks...');
            elseif check_var_change>0;
               setlogtext('You changed the pattern of required signs of shocks on variables...');
            end;
            %
            % Update cm.ir, etc, using new signs
            %
            if isempty(cm.B_0)==0;
               cm.signshock = sign_shock_fcn(cm,B_random);
               H = change_ir_signs(cm.signshock,cm.orig_B_0,cm.n);
               cm.B_0 = cm.orig_B_0*H;
               %
               % Need to fix the displayed B matrix
               %
               curr_B0 = vecstr(cm.evars,':      ');
               curr_B0 = strvcat(' ',' ',curr_B0);
               for j=1:cm.n;
                  addto_B0 = '';
                  for i=1:cm.n;
                     if cm.B_0(i,j)<0;
                        if abs(cm.B_0(i,j))>1.0e-008;
                           addto_B0 = strvcat(addto_B0,[num2str(cm.B_0(i,j),'%1.4f') '     ']);
                        else;
                           addto_B0 = strvcat(addto_B0,' 0      ');
                        end;
                     else;
                        if abs(cm.B_0(i,j))>1.0e-008;
                           addto_B0 = strvcat(addto_B0,[' ' num2str(cm.B_0(i,j),'%1.4f') '     ']);
                        else;
                           addto_B0 = strvcat(addto_B0,' 0      ');
                        end;
                     end;
                  end;
                  sizeadd = size(addto_B0,2)-16;
                  if j>9;
                     sizeadd = sizeadd-1;
                  end;
                  padd_B0 = '';
                  for i=1:sizeadd;
                     padd_B0 = [padd_B0 ' '];
                  end;
                  curr_B0 = [curr_B0 strvcat(' ',[padd_B0 cm.myshocks(j,:)],addto_B0)];
               end;
               curr_B0 = strvcat(curr_B0,' ');
               set(svar.betamat,'String',curr_B0);
            end;
         elseif status1==1;
            if (strcmp(type_var,'long_run')==1)&(num_res>(cm.n*(cm.n-1))/2)&(cm.r>0)&(cm.r<cm.n);
               txt = ['The model cannot be identified using long-run restrictions only. While the total number of restrictions ' num2str(num_res,'%0.0f') ' is greater than what is required for exact identification, it is still under-identified. Please add some restrictions on the contemporaneous effects.'];
               about(txt,'warning','Identification and Estimation',130,350);
            else;
               if num_res<(cm.n*(cm.n-1))/2;
                  txt = ['The model is under-identified. You have selected ' num2str(num_res,'%0.0f') ' restrictions, while exact identification requires ' num2str((cm.n*(cm.n-1))/2,'%0.0f') ' restrictions. Please add some more restrictions.'];
               elseif (cm.restrict(1,3)==1)&(num_res>=(cm.n*(cm.n-1))/2);
                  txt = ['The model is under-identified. Although the number of identifying restrictions (' num2str(num_res,'%0.0f') ') may seem to be sufficient, the restrictions you have on the C matrix interfere with them. Please reformulate your identifying restrictions.'];
               elseif (cm.restrict(1,1)==1)&(num_res>=(cm.n*(cm.n-1))/2);
                  txt = ['The model is under-identified. Although the number of identifying restrictions ('  num2str(num_res,'%0.0f') ') may seem to be sufficient, the restrictions you have on the alpha matrix interfere with them. Please reformulate your identifying restrictions.'];
               else;
                  txt = 'The model is under-identified for some unknown reason. Please reformulate your identifying restrictions.';
               end;
               about(txt,'warning','Identification and Estimation',120,350);
            end;
         elseif status1==2;
            txt = 'The restrictions imply that the B matrix is singular. Since Omega = B*B'' this is not feasible. Please edit your existing restrictions.';
            about(txt,'warning','Identification and Estimation',120,350);
         elseif status1==3;
            txt = ['Some of your restrictions are redundant. The total number of restrictions is ' num2str(tot_res,'%0.0f') ', while the number of linearly independent restrictions is ' num2str(num_res,'%0.0f') '. Please remove the redundant ones.'];
            about(txt,'warning','Identification and Estimation',120,350);
         elseif status1==4;
            txt = 'Some of the long-run restrictions cannot be satisfied by the model. Please remove these restrictions.';
            about(txt,'warning','Identification and Estimation',120,350);
         elseif status1==5;
            txt = ['The restrictions imply either that B is singular or that the rank of C, ' num2str(C_rank,'%0.0f') ', is incompatible with the number of cointegration relations. Please rewrite your restrictions.'];
            about(txt,'warning','Identification and Estimation',120,350);
         end;
         cm.running = 0;
      end;
   case 'struc_estimate'
      if cm.running==0;
         cm.running = 1;
            if get(svar.sr_rest,'Value')==1;
               type_var = 'short_run';
            elseif get(svar.lr_rest,'Value')==1;
               type_var = 'long_run';
            else;
               type_var = 'joint';
            end;
            %
            % test identification based on which method is used
            %
            % 1. Condition on reduced form
            %
            if curr_ini.method(1,1)==1;
               [status1,num_res,tot_res,B_random] = test_ident_m1(cm,type_var);
               if (status1==1)&(num_res==floor((cm.n*(cm.n-1))/2));
                  %
                  % re-check status1
                  %
                  [status1,num_res,tot_res,B_random] = test_ident_m1(cm,type_var);
               end;
            %
            % 2. Re-estimate reduced form along with B
            %    (curr_ini.reestbeta tells us if beta should be updated or not)
            %
            elseif curr_ini.method(1,2)==1;
               [status1,num_res,tot_res,B_random,keep_rest,del_rest,use_psi_rest,C_rank] = test_ident_m2(cm,type_var);
               if (status1==1)&(num_res==floor((cm.n*(cm.n-1))/2));
                  %
                  % re-check status1
                  %
                  [status1,num_res,tot_res,B_random,keep_rest,del_rest,use_psi_rest,C_rank] = test_ident_m2(cm,type_var);
               end;
            end;
            %
            % if status is OK continue
            %
            if status1==0;
               set(svarguidlg,'Visible','off');
               drawnow;
               %
               % get the signs of the shocks
               %
               cm.signshock = sign_shock_fcn(cm,B_random);
               %
               % Remember values
               %
               prev.Psi = cm.Psi;
               prev.alpha = cm.alpha;
               prev.beta = cm.beta;
               prev.epsilon = cm.epsilon;
               prev.Omega = cm.Omega;
               prev.Sigma_theta = cm.Sigma_theta;
               %
               % we're now ready to begin estimation of B
               %
               setlogtext(strvcat(' ','Estimating the B matrix...'));
               if curr_ini.method(1,1)==1;
                  %
                  % reset all parameters that may change
                  %
                  if cm.restrict(1,3)==0;
                     cm.Psi = cm.old_values.Psi;
                     cm.alpha = cm.old_values.alpha;
                     cm.beta = cm.old_values.beta;
                     cm.Omega = cm.old_values.Omega;
                     cm.Sigma_theta = cm.old_values.Sigma_theta;
                     cm.epsilon = cm.old_values.epsilon;
                  else;
                     cm.Psi = cm.C_values.Psi;
                     cm.alpha = cm.C_values.alpha;
                     cm.beta = cm.C_values.beta;
                     cm.Omega = cm.C_values.Omega;
                     cm.Sigma_theta = cm.C_values.Sigma_theta;
                     cm.epsilon = cm.C_values.epsilon;
                  end;
                  %
                  [status,abort,B_0,S,gamma,Sigma_B0,infomat,R_l,R_mat,rank_test] = vlaar(cm,type_var,cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,[],1,1);
                  if (strcmp(abort,'cancel')==0)&((status==0)|(status==2));
                     Psi = cm.Psi;
                     alpha = cm.alpha;
                     beta = cm.beta;
                     stdbeta = cm.stdbeta;
                     Omega = cm.Omega;
                     Sigma_theta = cm.Sigma_theta;
                     epsilon = (B_0/chol(cm.Omega)')*cm.epsilon;
                     epsilon_v = epsilon;
                     dB_theta = [];
                     %
                     % Omega would be different from cm.Omega if there were over-identifying restrictions,
                     % and we would set Omega = B_0*B_0'. If that were the case, then formally Sigma_theta,
                     % alpha, beta, Psi, etc should be re-estimated. Then we could re-estimate B_0 as well.
                     % Under the Vlaar method we skip this. Asymptotically it won't matter :)
                     %
                     if strcmp(type_var,'short_run')==1;
                        [A,Sigma_A,C,dC_theta,Sigma_B0_adj] = amat_sr_vlaar(B_0,Sigma_B0,Psi,alpha,beta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                     else;
                        [A,Sigma_A,C,dC_theta,Sigma_B0_adj] = amat_lr_vlaar(B_0,Sigma_B0,S,R_mat,R_l,gamma,Psi,alpha,beta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                     end;
                  end;
               elseif curr_ini.method(1,2)==1;
                  %
                  % under this method degrees of freedom may change as some over-identifying restrictions are
                  % imposed on the short-run dynamics; this would affect the use of cm.df
                  %
                  %
                  % reset all parameters that may change
                  %
                  if cm.restrict(1,3)==0;
                     cm.Psi = cm.old_values.Psi;
                     cm.alpha = cm.old_values.alpha;
                     cm.beta = cm.old_values.beta;
                     cm.Omega = cm.old_values.Omega;
                     cm.Sigma_theta = cm.old_values.Sigma_theta;
                     cm.epsilon = cm.old_values.epsilon;
                  else;
                     cm.Psi = cm.C_values.Psi;
                     cm.alpha = cm.C_values.alpha;
                     cm.beta = cm.C_values.beta;
                     cm.Omega = cm.C_values.Omega;
                     cm.Sigma_theta = cm.C_values.Sigma_theta;
                     cm.epsilon = cm.C_values.epsilon;
                  end;
                  %
                  [status,abort,B_0,Psi,alpha,beta,C,A,stdbeta,S,Sigma_theta,Sigma_B0,Sigma_B0_adj,Sigma_A,infomat,R_mat,epsilon_v,num_rest,dC_theta,dB_theta,max_case,rank_test] = warne(cm,type_var,keep_rest,del_rest,use_psi_rest,cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z0,cm.Z1,cm.Z2,[],1,1,0);
                  epsilon = (B_0/chol(cm.Omega)')*epsilon_v;
                  Omega = B_0*B_0';
                  gamma = [];
                  R_l = [];
               end;
               %
               % dealing with the results
               %
               if strcmp(abort,'cancel')==0;
                  if (status==0)|(status==2);
                     cm.estimated = 1;
                     if curr_ini.noprint==0;
                        txt = strvcat(' ','Writing to output file...','Please hold!');
                        mywait = waitdlg(txt,'information');
                        waitf = get(mywait,'UserData');
                     end;
                     if curr_ini.method(1,1)==1;
                        setlogtext(strvcat(['Number of iterations: ' num2str(infomat(1,1),'%0.0f')],['Value of gradient:    ' num2str(infomat(1,2))]));
                        df = round(((cm.n*(cm.n+1))/2)-size(S,2));
                        if df>0;
                           if curr_ini.noprint==0;
                              txt = strvcat(' ','Computing test for over-identifying restrictions...','Please hold!');
                              set(waitf.text,'String',txt);
                           end;
                           if curr_ini.sscorrect==0;
                              setlogtext(['Computing the LR test of the ' num2str(df,'%0.0f') ' restrictions']);
                           else;
                              setlogtext(['Computing the F test of the ' num2str(df,'%0.0f') ' restrictions']);
                           end;
                           Omega_r = B_0*B_0';
                           L_r = -cm.T*(log(det(Omega_r))+sum(diag((Omega_r\Omega))));
                           L_u = -cm.T*(log(det(Omega))+cm.n);
                           LR_B0 = (L_u-L_r);
                           if curr_ini.sscorrect==0;
                              pval = chis_cdf(LR_B0,df);
                           else;
                              LR_B0 = ((cm.T-cm.df)/(cm.T*df))*LR_B0;
                              pval = fdis_cdf(LR_B0,df,cm.T-cm.df);
                           end;
                        else;
                           LR_B0 = [];
                           pval = [];
                        end;
                     elseif curr_ini.method(1,2)==1;
                        setlogtext(strvcat(['Number of iterations:     ' num2str(infomat(1,1),'%0.0f')],['Value of lnL(i)-lnL(i-1): ' num2str(infomat(1,2))]));
                        df = round(num_rest-((cm.n*(cm.n-1))/2));
                        if df>0;
                           if curr_ini.noprint==0;
                              txt = strvcat(' ','Computing test for over-identifying restrictions...','Please hold!');
                              set(waitf.text,'String',txt);
                           end;
                           if curr_ini.sscorrect==0;
                              setlogtext(['Computing the LR test of the ' num2str(df,'%0.0f') ' restrictions']);
                           else;
                              setlogtext(['Computing the F test of the ' num2str(df,'%0.0f') ' restrictions']);
                           end;
                           L_r = loglike_fcn(B_0,epsilon_v);
                           L_u = loglike(cm.epsilon);
                           LR_B0 = 2*(L_u-L_r);
                           if curr_ini.sscorrect==0;
                              pval = chis_cdf(LR_B0,df);
                           else;
                              LR_B0 = ((cm.T-cm.df)/(cm.T*df))*LR_B0;
                              pval = fdis_cdf(LR_B0,df,cm.T-cm.df);
                           end;
                        else;
                           LR_B0 = [];
                           pval = [];
                        end;
                     end;
                     %
                     % compute standard errors for B_0
                     %
                     std_B_0 = vecinv(sqrt(abs(diag(Sigma_B0_adj))),cm.n);
                     std_A = vecinv(sqrt(abs(diag(Sigma_A))),cm.n);
                     %
                     % fix matrices with standard errors in case there are numerical problems
                     %
                     for i=1:cm.n;
                        for j=1:cm.n;
                           if abs(A(i,j))<1.0e-008;
                              std_A(i,j) = 0;
                           end;
                           if abs(B_0(i,j))<1.0e-008;
                              std_B_0(i,j) = 0;
                           end;
                        end;
                     end;
                     %
                     % parameters may have changes under the 2nd method
                     % while residuals can also have changed under the 1st
                     %
                     if curr_ini.method(1,2)==1;
                        %
                        % Compute eigenvalues from companion matrix
                        %
                        eigenvalues = companionrest(alpha,Psi,beta,cm.p,cm.n,cm.d);
                        eigenvalues2 = eigenvalues;
                        eigenvalues = abs(sort(eigenvalues));
                        i = size(eigenvalues,1);
                        while i>=1;
                           evalue = eigenvalues(i,1);
                           if strcmp(num2str(evalue,'%1.10f'),'1.0000000000')==1;
                              i = i-1;
                           else;
                              i = 0;
                           end;
                        end;
                     else;
                        evalue = cm.evalue;
                     end;
                     %
                     % value of the log-likelihood function
                     % and log(det(Omega))
                     %
                     [lnl,ldOmega] = loglike_fcn(B_0,epsilon_v);
                     %
                     % write to the output file
                     %
                     if curr_ini.noprint==0;
                        txt = strvcat(' ','Writing A and B matrices to output...','Please hold!');
                        set(waitf.text,'String',txt);
                        fout = fopen(cm.outputfile,'at');
                        if curr_ini.texoutputformat>=2;
                           fprintf(fout,'\\subsection*{Estimated $B$ matrix for structural VAR model:}\n\n');
                           my_printstr = '%s';
                           my_printstr2 = '';
                           my_printstr3 = '';
                           r_str = 'r';
                           my_shocks = '';
                           for i=1:cm.n;
                              my_printstr = [my_printstr ' & %0.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
                              my_printstr2 = [my_printstr2 ' & (%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f)'];
                              my_printstr3 = [my_printstr3 ' & [%0.' num2str(2+curr_ini.decimals,'%0.0f') 'f]'];
                              r_str = [r_str 'r'];
                              next_shock = strtrim(cm.myshocks(i,:));
                              if istexgreek(next_shock)==1;
                                 next_shock = ['\\' next_shock];
                              end;
                              if isempty(findstr(next_shock,'_'))==0;
                                 next_shock = ['$' strrep(next_shock,'_','_{') '}$'];
                              end;
                              my_shocks = [my_shocks ' & ' next_shock];
                           end;
                           fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                           fprintf(fout,[my_shocks ' \\\\\n']);
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr ' \\\\\n'],cm.myvars(i,:),B_0(i,:));
                              fprintf(fout,[my_printstr2 ' \\\\\n'],std_B_0(i,:));
                              fprintf(fout,[my_printstr3 ' \\\\\n'],inf2nan(B_0(i,:)./std_B_0(i,:)));
                           end;
                           fprintf(fout,'\\end{tabular}\n\n');
                           %
                           % enter information about iterations
                           %
                           fprintf(fout,'\\vspace*{2mm}\n\n\\begin{tabular}{ll}\n');
                           fprintf(fout,'Number of iterations for algorithm: & %0.0f \\\\\n',infomat(1,1));
                           if curr_ini.method(1,1)==1;
                              fprintf(fout,['Value of convergence criterion ($\\max |\\partial \\ln L/\\partial \\gamma|$): & ' num2str(infomat(1,2)) ' \\\\\n']);
                           elseif curr_ini.method(1,2)==1;
                              fprintf(fout,['Value of convergence criterion ($\\ln L(i)-\\ln L(i-1)$): & ' num2str(infomat(1,2)) ' \\\\\n']);
                           end;
                           fprintf(fout,'\\end{tabular}\n\n');
                           %
                           if df>0;
                              fprintf(fout,'\\header{LR-type test statistic}\n\n');
                              fprintf(fout,'\\begin{tabular}{ll}\n');
                              if curr_ini.sscorrect==0;
                                 fprintf(fout,[' & LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],df,LR_B0);
                              else;
                                 fprintf(fout,[' & F(%0.0f,%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],df,cm.T-cm.df,LR_B0);
                              end;
                              if isempty(pval)==0;
                                 fprintf(fout,[' & $p$-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],1-pval);
                              end;
                              fprintf(fout,'\\end{tabular}\n\n');
                           end;
                           fprintf(fout,'\\header{Estimated $A=CB$ matrix:}\n\n');
                           fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                           fprintf(fout,[my_shocks ' \\\\\n']);
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr ' \\\\\n'],cm.myvars(i,:),A(i,:));
                              fprintf(fout,[my_printstr2 ' \\\\\n'],std_A(i,:));
                              fprintf(fout,[my_printstr3 ' \\\\\n'], inf2nan(A(i,:)./std_A(i,:)));
                           end;
                           fprintf(fout,'\\end{tabular}\n\n');
                           fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                        else;
                           fprintf(fout,'Estimated B matrix for structural VAR model:\n\n');
                           my_printstr = ' %s';
                           sp_str = spacestr(curr_ini.decimals-2);
                           num_sh_str_1 = 14-size(cm.myshocks,2);
                           num_sh_str_2 = 16-size(cm.myshocks,2);
                           if num_sh_str_1<1;
                              num_sh_str_1 = 1;
                              num_sh_str_2 = 2;
                           end;
                           add_str_1 = '';
                           for i=1:num_sh_str_2;
                              if i<=num_sh_str_1-2;
                                 add_str_1 = [add_str_1 ' '];
                              end;
                           end;
                           shock_str = [add_str_1 sp_str cm.myshocks(1,:)];
                           for i=1:cm.n;
                              my_printstr = [my_printstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(2+curr_ini.decimals,'%0.0f') 'f'];
                              if i<cm.n;
                                 shock_str = [shock_str '    ' sp_str cm.myshocks(i+1,:)];
                              end;
                           end;
                           fprintf(fout,[shock_str '\n\n']);
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),B_0(i,:));
                           end;
                           fprintf(fout,'\nwith asymptotic standard errors:\n\n');
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),std_B_0(i,:));
                           end;
                           fprintf(fout,'\nt-ratios:\n\n');
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),inf2nan(B_0(i,:)./std_B_0(i,:)));
                           end;
                           fprintf(fout,'\n');
                           %
                           % enter information about iterations
                           %
                           fprintf(fout,'Number of iterations for algorithm:                %0.0f\n',infomat(1,1));
                           if curr_ini.method(1,1)==1;
                              fprintf(fout,['Value of convergence criterion (max|dlnL/dgamma|): ' num2str(infomat(1,2)) '\n\n']);
                           elseif curr_ini.method(1,2)==1;
                              fprintf(fout,['Value of convergence criterion (lnL(i)-lnL(i-1)):  ' num2str(infomat(1,2)) '\n\n']);
                           end;
                           %
                           if df>0;
                              fprintf(fout,'  LR-type test statistic\n  ~~~~~~~~~~~~~~~~~~~~~~\n\n');
                              if curr_ini.sscorrect==0;
                                 fprintf(fout,['LR(%0.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],df,LR_B0);
                              else;
                                 fprintf(fout,['F(%1.0f,%1.0f) = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],df,cm.T-cm.df,LR_B0);
                              end;
                              if isempty(pval)==0;
                                 fprintf(fout,['p-value = %0.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],1-pval);
                              end;
                              fprintf(fout,'\n');
                           end;
                           fprintf(fout,'Estimated A=C*B matrix:\n\n');
                           fprintf(fout,[shock_str '\n\n']);
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),A(i,:));
                           end;
                           fprintf(fout,'\nwith asymptotic standard errors:\n\n');
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),std_A(i,:));
                           end;
                           fprintf(fout,'\nt-ratios:\n\n');
                           for i=1:cm.n;
                              fprintf(fout,[my_printstr '\n'],cm.myvars(i,:),inf2nan(A(i,:)./std_A(i,:)));
                           end;
                           fprintf(fout,'\n');
                           fprintf(fout,'===============================================================================\n\n');
                        end;
                        fclose(fout);
                        %
                        % display short-run dynamics and the C matrix under "FIML" method
                        %
                        if (curr_ini.method(1,2)==1)&(df>0);
                           if (curr_ini.shortdyn==1)&(curr_ini.noprint==0);
                              txt = strvcat(' ','Writing short-run dynamics to output...','Please hold!');
                              set(waitf.text,'String',txt);
                              shortstr = shortrun(alpha,Psi,Omega,beta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,0,Sigma_theta);
                              fout = fopen(cm.outputfile,'at');
                              if curr_ini.texoutputformat>=2;
                                 fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                              else;
                                 fprintf(fout,'\n===============================================================================\n\n');
                              end;
                              if cm.evalue<1;
                                 res_str = 'For restricted';
                                 if cm.dfbeta==0;
                                    res_str = 'For';
                                 end;
                                 if cm.r<cm.n;
                                    C = cmat(Psi,alpha,beta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                                 else;
                                    C = cmat_stat(Psi,alpha,beta,epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,Sigma_theta);
                                 end;
                                 myC = showc(C(:,1:cm.n),cm.myvars);
                                 myCstd = showc(C(:,cm.n+1:2*cm.n),cm.myvars);
                                 %
                                 C_std = C(:,cm.n+1:2*cm.n);
                                 C_t = C(:,1:cm.n);
                                 for i=1:cm.n;
                                    for j=1:cm.n;
                                       if abs(C_std(i,j))<1.0e-005;
                                          C_std(i,j) = 0;
                                          C_t(i,j) = mystr2num(strtrim(num2str(C_t(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
                                       end;
                                    end;
                                 end;
                                 %
                                 myCt = showc(inf2nan(C_t./C_std),cm.myvars);
                                 if curr_ini.texoutputformat>=2;
                                    myC = text_latextable(myC);
                                    myCstd = text_latextable(myCstd);
                                    myCstd = myCstd(2:size(myCstd,1),:);
                                    myCt = text_latextable(myCt);
                                    myCt = myCt(2:size(myCt,1),:);
                                    myC = merge_latextable(myC,myCstd,myCt);
                                    r_str = 'r';
                                    for i=1:cm.n;
                                       r_str = [r_str 'r'];
                                    end;
                                    fprintf(fout,'\\subsection*{The $C$ matrix}\n\n\\header{%s $r$=%0.0f}\n\n',res_str,cm.r);
                                    fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                                    for i=1:(size(myC,1));
                                       fprintf(fout,'%s\n',myC(i,:));
                                    end;
                                    fprintf(fout,'\\end{tabular}\n\n');
                                    fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                                 else;
                                    fprintf(fout,'THE C-MATRIX (%s rank=%1.0f)\n\n',lower(res_str),cm.r);
                                    for i=1:(size(myC,1));
                                       fprintf(fout,'%s\n',myC(i,:));
                                    end;
                                    fprintf(fout,'\nEstimated asymptotic standard errors:\n\n');
                                    for i=1:(size(myC,1));
                                       fprintf(fout,'%s\n',myCstd(i,:));
                                    end;
                                    fprintf(fout,'\nt-ratios:\n\n');
                                    for i=1:(size(myC,1));
                                       fprintf(fout,'%s\n',myCt(i,:));
                                    end;
                                    fprintf(fout,'\n===============================================================================\n\n');
                                 end;
                              end;
                              fclose(fout);
                           end;
                           %
                           % print new values for beta and stdbeta if they have potentially changed
                           %
                           if (strcmp(type_var,'short_run')==0)&(curr_ini.reestbeta==1);
                              txt = strvcat(' ','Writing beta parameters to output...','Please hold!');
                              set(waitf.text,'String',txt);
                              fout = fopen(cm.outputfile,'at');
                              if curr_ini.texoutputformat>=2;
                                 if cm.restrict(1,2)==1;
                                    fprintf(fout,'\\header{Re-estimated restricted cointegration relations:}\n\n');
                                 else;
                                    fprintf(fout,'\\header{Re-estimated cointegration relations:}\n\n');
                                 end;
                                 myprintstr = '%0.0f';
                                 r_str = 'r';
                                 j = 2;
                                 while j<=size(beta,1)+1;
                                    myprintstr = [myprintstr ' & %0.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                                    r_str = [r_str 'r'];
                                    j = j+1;
                                 end;
                                 myprintstr = [myprintstr ' \\\\\n'];
                                 fprintf(fout,['\\begin{tabular}{' r_str '}\n']);
                                 fprintf(fout,['relation & ' strrep(strtrim([cm.evars ' ' cm.xvars1 ' ' cm.restvec]),' ',' & ') ' \\\\\n']);
                                 if cm.restrict(1,2)==0;
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' normbeta(beta)']');
                                 else;
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' beta']');
                                    fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{ } \\\\\n']);
                                    fprintf(fout,['\\multicolumn{' num2str(size(r_str,2),'%0.0f') '}{c}{with asymptotic conditional standard errors:} \\\\\n']);
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' stdbeta']');
                                 end;
                                 fprintf(fout,'\\end{tabular}\n\n');
                                 fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                              else;
                                 if cm.restrict(1,2)==1;
                                    fprintf(fout,'Re-estimated restricted cointegration relations:\n');
                                 else;
                                    fprintf(fout,'Re-estimated cointegration relations:\n');
                                 end;
                                 fprintf(fout,'\n');
                                 myprintstr = '%2.0f';
                                 j = 2;
                                 while j<=size(beta,1)+1;
                                    myprintstr = [myprintstr ' %' num2str(6+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f'];
                                    j = j+1;
                                 end;
                                 myprintstr = [myprintstr '\n'];
                                 if cm.restrict(1,2)==0;
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' normbeta(beta)']');
                                 else;
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' beta']');
                                    fprintf(fout,'\n');
                                    fprintf(fout,'with asymptotic conditional standard errors:\n');
                                    fprintf(fout,'\n');
                                    fprintf(fout,myprintstr,[(1:size(beta,2))' stdbeta']');
                                 end;
                                 fprintf(fout,'\n\n===============================================================================\n\n');
                              end;
                              fclose(fout);
                           end;
                        end;
                        %
                        % value of the log-likelihood function
                        %
                        if df>0;
                           fout = fopen(cm.outputfile,'at');
                           if curr_ini.texoutputformat>=2;
                              fprintf(fout,'\\subsection*{Value of the maximized log-likelihood function:}\n\n');
                              fprintf(fout,'\\begin{tabular}{ll}\n');
                              if cm.r<cm.n;
                                 numbp = ((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.dfbeta;
                                 fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%0.0f,r=%0.0f,\\mbox{\\rm \\#free } \\beta \\mbox{\\rm \\hspace*{1mm} parameters}=%0.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,cm.r,numbp,lnl);
                              else;
                                 fprintf(fout,[' & $\\ln L(\\mbox{\\rm lags}=%0.0f)$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],cm.p,lnl);
                              end;
                              fprintf(fout,[' & $\\ln(\\det(\\Omega))$ = %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],ldOmega);
                              fprintf(fout,'\\end{tabular}\n\n');
                              fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                           else;
                              fprintf(fout,'Value of the maximized log-likelihood function:\n\n');
                              if cm.r<cm.n;
                                 numbp = ((cm.n+cm.q1+cm.dcoint-cm.r)*cm.r)-cm.dfbeta;
                                 fprintf(fout,['   lnL(lags=%1.0f,rank=%1.0f,#free beta parameters=%1.0f)=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,cm.r,numbp,lnl);
                              else;
                                 fprintf(fout,['   lnL(lags=%1.0f)=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],cm.p,lnl);
                              end;
                              fprintf(fout,['   ln(det(Omega))=%0.' num2str(curr_ini.decimals,'%0.0f') 'f\n\n'],ldOmega);
                              fprintf(fout,'===============================================================================\n\n');
                           end;
                           fclose(fout);
                        end;
                        %
                        % calculate lag-order test etc.
                        % only useful if reduced-form or beta parameters have changed
                        %
                        if (curr_ini.method(1,2)==1)&(df>0);
                           if curr_ini.lagtest>0;
                              setlogtext('Computing lag order tests...');
                              txt = strvcat(' ','Computing lag order tests.','Please hold!');
                              set(waitf.text,'String',txt);
                              print_lagordertest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df,cm.restrict(1,1));
                           end;
                           if curr_ini.gctest==1;
                              setlogtext('Computing Granger causality tests...');
                              txt = strvcat(' ','Computing Granger causality tests.','Please hold!');
                              set(waitf.text,'String',txt);
                              print_gctest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df,cm.restrict(1,1));
                              if (cm.n>2)&((cm.r>0)|(cm.p>1));
                                 setlogtext('Computing multi-step Granger causality tests...');
                                 txt = strvcat(' ','Computing multi-step Granger causality tests.','Please hold!');
                                 set(waitf.text,'String',txt);
                                 print_mgctest(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.restrict(1,1),cm.restrict(1,3));
                              end;
                           end;
                           if curr_ini.excludediffs==1;
                              if ((cm.p>1)&(cm.n>2))|(cm.q0>0)|((cm.q1>0)&(cm.pe>0));
                                 setlogtext('Computing exclusion tests for first differences...');
                                 txt = strvcat(' ','Computing exclusion tests for first differences.','Please hold!');
                                 set(waitf.text,'String',txt);
                                 print_firstdiffstests(cm,curr_ini,beta,alpha,Psi,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.df,cm.restrict(1,3));
                              end;
                           end;
                           if (cm.r<cm.n)&(cm.r>0);
                              if curr_ini.weakexo==1;
                                 setlogtext('Computing weak exogeneity tests...');
                                 txt = strvcat(' ','Computing weak exogeneity tests.','Please hold!');
                                 set(waitf.text,'String',txt);
                                 print_weakexo(cm,curr_ini,beta,alpha,epsilon,Sigma_theta,cm.dfbeta,cm.dfalpha,cm.T-cm.df,cm.restrict(1,1));
                              end;
                           end;
                        end;
                        %
                        % print if the reduced form residuals have changed
                        %
                        if df>0;
                           if curr_ini.restat==1;
                              txt = strvcat(' ','Computing residual statistics.','Please hold!');
                              set(waitf.text,'String',txt);
                           end;
                           %
                           % residual statistics
                           %
                           print_residual_analysis(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df);
                        end;
                        %
                        % specification tests
                        %
                        if (sum(cm.spectest)>0)&(df>0);
                           txt = strvcat(' ','Computing specification tests.','Please hold!');
                           set(waitf.text,'String',txt);
                        end;
                        %
                        if (cm.spectest(1,1)==1)&(df>0);
                           setlogtext('Computing serial correlation tests...');
                           print_corrtest(cm,curr_ini,epsilon,beta,cm.dfbeta,cm.dfalpha,cm.df);
                        end;
                        %
                        if (cm.spectest(1,2)==1)&(df>0);
                           setlogtext('Computing ARCH tests...');
                           print_archtest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df);
                        end;
                        %
                        if (cm.spectest(1,3)==1)&(df>0);
                           setlogtext('Computing normality tests...');
                           print_normalitytest(cm,curr_ini,epsilon,cm.dfbeta,cm.dfalpha,cm.df);
                        end;
                        %
                        % only useful if reduced-form or beta parameters have changed
                        %
                        if (curr_ini.method(1,2)==1)&(df>0);
                           if cm.spectest(1,4)==1;
                              if rank(beta'*beta,1.0e-004)==cm.r;
                                 if cm.r>0;
                                    Zb = beta'*cm.Z1;
                                 else;
                                    Zb = 0;
                                 end;
                                 fout = fopen(cm.outputfile,'at');
                                 if curr_ini.texoutputformat>=2;
                                    fprintf(fout,['\\subsection*{Parameter constancy tests}\n\n\\header{For $r$=' num2str(cm.r) ', lags=' num2str(cm.p) ', \\#free $\\beta$ params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ', \\#free $\\alpha$ params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') '}\n\n']);
                                 else;
                                    fprintf(fout,['PARAMETER CONSTANCY TESTS\n\n(for rank=' num2str(cm.r) ',lags=' num2str(cm.p) ',#free beta params=' num2str(cm.r*(cm.n+cm.q1+cm.dcoint-cm.r)-cm.dfbeta,'%0.0f') ',#free alpha params=' num2str(cm.n*cm.r-cm.dfalpha,'%0.0f') ')\n\n']);
                                 end;
                                 fclose(fout);
                                 if curr_ini.lstrtest==1;
                                    setlogtext('Computing LSTR1 constancy tests...');
                                    print_lstr1test(cm,curr_ini,epsilon,Zb,cm.df);
                                 end;
                                 %
%                                 if (cm.q1+cm.q0==0)&(curr_ini.nyblom==1)&(curr_ini.nyblombeta==1)&(cm.r>0)&(cm.r<cm.n);
                                 if (curr_ini.nyblom==1)&(curr_ini.nyblombeta==1)&(cm.r>0)&(cm.r<cm.n);
                                    setlogtext('Computing Nyblom constancy tests for beta...');
                                    cm.Theta = [Psi alpha];
                                    set(svar.estalpha,'UserData',cm.Theta);
                                    if cm.restrict(1,1)==1;
                                       print_alpha_nyblom(cm,curr_ini);
                                    elseif (cm.restrict(1,1)==0)&(cm.restrict(1,2)==1);
                                       print_nyblom(cm,curr_ini);
                                    end;
                                    cm.Theta = cm.lastTheta;
                                    set(svar.estalpha,'UserData',cm.Theta);
                                 end;
                                 %
                                 if cm.p+cm.r+cm.d>1;
                                    ind = 0;
                                    if cm.restrict(1,1)==1;
                                       ind = 1;
                                    end;
                                    if curr_ini.chow==1;
                                       setlogtext('Computing Chow tests...');
                                       print_chow(cm,curr_ini,beta,ind);
                                    end;
                                    %
                                    setlogtext('Computing fluctuation tests...');
                                    print_ploberger(cm,curr_ini,beta,ind);
                                 end;
                              else;
                                 fout = fopen(cm.outputfile,'at');
                                 myrank = rank(beta'*beta,1.0e-004);
                                 setlogtext(strvcat(' ','WARNING: The estimated beta matrix is badly identified...',['         Numerical rank is ' num2str(myrank(1,1)) '...']));
                                 txt = ['The estimated restricted beta matrix is badly identified. The rank of beta''beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Please consider some other restrictions.'];
                                 about(txt,'warning','Badly identified beta matrix',120,350);
                                 if curr_ini.texoutputformat>=2;
                                    fprintf(fout,'\\vspace*{3mm}\n\\newline\n\\noindent\n\\textbf{WARNING:} Parameter constancy tests have not been calculated. The\n');
                                    fprintf(fout,'         estimated restricted $\\beta$ matrix is badly identified.\n');
                                    fprintf(fout,'         The rank of $\\beta''\\beta$ is %0.0f, i.e. less than %0.0f, when\n',myrank,cm.r);
                                    fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n\n');
                                    fprintf(fout,'\\ruler\n\n%%==============================================================================\n\n');
                                 else;
                                    fprintf(fout,'WARNING: Parameter constancy tests have not been calculated. The\n');
                                    fprintf(fout,'         estimated restricted beta matrix is badly identified.\n');
                                    fprintf(fout,'         The rank of beta''beta is %1.0f, i.e. less than %1.0f, when\n',myrank,cm.r);
                                    fprintf(fout,'         a tolerance level of 1.0e-004 is used.\n');
                                    fprintf(fout,'\n===============================================================================\n\n');
                                 end;
                                 fclose(fout);
                              end;
                           end;
                        end;
                        %
                     end;
                     %
                     % close wait dialogue
                     %
                     if curr_ini.noprint==0;
                        set(waitf.text,'UserData','done');
                        delete(mywait);
                        drawnow;
                     end;
                     %
                     %  we are now ready to open the dialog diaplaying the results in a gui
                     %
                     thisbdlg = bdlg('init',cm,infomat,pval,LR_B0,epsilon,Psi,alpha,beta,stdbeta,Sigma_theta,df,B_0,Sigma_B0,Sigma_B0_adj,A,Sigma_A,evalue,S,gamma,R_l,R_mat,type_var,C,dC_theta,dB_theta);
                     brest = get(thisbdlg,'UserData');
                     usebrest = get(brest.betamat,'UserData');
                     c_ini = get(brest.viewdata,'UserData');
                     tm = c_ini.models;
                     if isempty(tm.ir)==0;
                        ir = tm.ir;
                        stdvec = tm.stdvec;
                        upper_c = tm.upper;
                        lower_c = tm.lower;
                        vd = tm.vd;
                        betair = tm.betair;
                        betair_std = tm.betair_std;
                     else;
                        ir = [];
                     end;
                     set(brest.alphamat,'UserData','done');
                     delete(thisbdlg);
                     drawnow;
                     %
                     % change cm structure if estimated B has been accepted.
                     %
                     if usebrest==1;
                        setlogtext('You decided to use the estimated B matrix...');
                        if strcmp(computer,'PCWIN')==1;
                           file = [pwd '\structural_est.txt'];
                        else;
                           file = [pwd '/structural_est.txt'];
                        end;
                        f_est = fopen(file,'wt');
                        fclose(f_est);
                        cm.beta = beta;
                        cm.stdbeta = stdbeta;
                        cm.epsilon = epsilon;
                        cm.Psi = Psi;
                        cm.alpha = alpha;
                        cm.Sigma_theta = Sigma_theta;
                        cm.dfb = df;
                        cm.B_0 = B_0;
                        cm.Omega = B_0*B_0';
                        cm.orig_B_0 = B_0;
                        cm.Sigma_B0 = Sigma_B0;
                        cm.Sigma_B0_adj = Sigma_B0_adj;
                        cm.A = A;
                        cm.Sigma_A = Sigma_A;
                        cm.evalue = evalue;
                        cm.S = S;
                        cm.gamma = gamma;
                        cm.R_l = R_l;
                        cm.R_mat = R_mat;
                        cm.C = C;
                        cm.dC_theta = dC_theta;
                        cm.dB_theta = dB_theta;
                        cm.structural_df = df;
                        cm.std_A = std_A;
                        cm.std_B_0 = std_B_0;
                        cm.LR_B0 = LR_B0;
                        if curr_ini.method(1,2)==1;
                           cm.keep_rest = keep_rest;
                           cm.del_rest = del_rest;
                           cm.use_psi_rest = use_psi_rest;
                           cm.max_case = max_case;
                           %
                           % write eigenvalues of companions matrix to the output file
                           %
                           if (curr_ini.noprint==0)&(df>0);
                              [eigenvalues1,AscInd] = sort(abs(eigenvalues2));
                              nr = size(AscInd,1);
                              DescInd = zeros(nr,1);
                              for i=1:nr;
                                 DescInd(i)=AscInd(nr+1-i);
                              end;
                              eigenvalues2 = eigenvalues2(DescInd);
                              %
                              fout = fopen(cm.outputfile,'at');
                              if curr_ini.texoutputformat>=2;
                                 fprintf(fout,'\\header{You decided to use these estimated $B$ matrix.}\n\n');
                              else;
                                 fprintf(fout,'You decided to use the estimated B matrix.\n\n');
                              end;
                              if curr_ini.texoutputformat>=2;
                                 fprintf(fout,'\\subsection*{Properties of the stacked VAR(1) companion matrix:}\n\n');
                                 fprintf(fout,'\\begin{tabular}{rrrr}\n');
                                 fprintf(fout,'\\multicolumn{4}{c}{Eigenvalues} \\\\\n');
                                 fprintf(fout,'number & real & imaginary & modulus \\\\\n');
                                 fprintf(fout,['%0.0f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f & %0.' num2str(curr_ini.decimals,'%0.0f') 'f \\\\\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                                 fprintf(fout,'\\end{tabular}\n\n\\ruler\n\n%%==============================================================================\n\n');
                              else;
                                 fprintf(fout,'Properties of the stacked VAR(1) companion matrix:\n');
                                 fprintf(fout,'Eigenvalues\n');
                                 fprintf(fout,['number    real   ' sp_str '   imaginary   ' sp_str '  modulus\n\n']);
                                 fprintf(fout,['%3.0f      %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f     %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f         %' num2str(3+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f\n'],[(1:size(eigenvalues2,1))' real(eigenvalues2) imag(eigenvalues2) abs(eigenvalues2)]');
                                 fprintf(fout,'\n===============================================================================\n\n');
                              end;
                              fclose(fout);
                           end;
                        end;
                        if isempty(ir)==0;
                           cm.ir = ir;
                           cm.orig_ir = ir;
                           cm.upper = upper_c;
                           cm.lower = lower_c;
                           cm.stdvec = stdvec;
                           cm.vd = vd;
                           cm.betair = betair;
                           cm.orig_betair = betair;
                           cm.betair_std = betair_std;
                        else;
                           if isfield(cm,'ir')==1;
                              cm.ir = [];
                              cm.orig_ir = [];
                              cm.upper = [];
                              cm.lower = [];
                              cm.stdvec = [];
                              cm.vd = [];
                              cm.betair = [];
                              cm.orig_betair = [];
                              cm.betair_std = [];
                           end;
                        end;
                        cm.type = type_var;
                        %
                        set(svar.irfunction,'Enable','on');
                        set(svar.menuirfunction,'Enable','on');
                        if strcmp(get(svar.menupermcomp,'Enable'),'on')==1;
                           set(svar.menu_permcomp,'Enable','on');
                        end;
                        set(svar.histdecomp,'Enable','on');
                        set(svar.vardecomp,'Enable','on');
                        set(svar.bootirmenu,'Enable','on');
                        set(svar.struc_undo,'Enable','on');
                        set(svar.struc_undomenu,'Enable','on');
                        %
                        cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,loglike_fcn(cm.B_0,cm.epsilon),cm.evalue);
                        set(svar.model_info,'String',cm.model_info.info);
                        %
                        if isfield(svar,'restrict_c')==1;
                           %
                           % fix C-estimation part
                           %
                           set(svar.restrict_c,'Enable','off');
                           set(svar.c_openrest,'Enable','off');
                           set(svar.c_saverest,'Enable','off');
                           set(svar.c_estimate,'Enable','off');
                           set(svar.c_undo,'Enable','off');
                           set(svar.restrict_cmenu,'Enable','off');
                           set(svar.c_openrestmenu,'Enable','off');
                           set(svar.c_saverestmenu,'Enable','off');
                           set(svar.c_estimatemenu,'Enable','off');
                           set(svar.c_undomenu,'Enable','off');
                        end;
                        if (isfield(svar,'dummy')==1)&(df>0);
                           set(svar.dummy,'Enable','off');
                        end;
                        if (isfield(svar,'common_cycles'))&(df>0);
                           set(svar.common_cycles,'Enable','off');
                        end;
                        %
                        % Need to fix the displayed B matrix
                        %
                        curr_B0 = vecstr(cm.evars,':      ');
                        curr_B0 = strvcat(' ',' ',curr_B0);
                        for j=1:cm.n;
                           addto_B0 = '';
                           for i=1:cm.n;
                              if cm.B_0(i,j)<0;
                                 if abs(cm.B_0(i,j))>1.0e-008;
                                    addto_B0 = strvcat(addto_B0,[num2str(cm.B_0(i,j),'%1.4f') '     ']);
                                 else;
                                    addto_B0 = strvcat(addto_B0,' 0      ');
                                 end;
                              else;
                                 if abs(cm.B_0(i,j))>1.0e-008;
                                    addto_B0 = strvcat(addto_B0,[' ' num2str(cm.B_0(i,j),'%1.4f') '     ']);
                                 else;
                                    addto_B0 = strvcat(addto_B0,' 0      ');
                                 end;
                              end;
                           end;
                           sizeadd = size(addto_B0,2)-16;
                           if j>9;
                              sizeadd = sizeadd-1;
                           end;
                           padd_B0 = '';
                           for i=1:sizeadd;
                              padd_B0 = [padd_B0 ' '];
                           end;
                           curr_B0 = [curr_B0 strvcat(' ',[padd_B0 cm.myshocks(j,:)],addto_B0)];
                        end;
                        curr_B0 = strvcat(curr_B0,' ');
                        set(svar.betamat,'String',curr_B0);
                        set(svar.amat,'Enable','on');
                        set(svar.bmat,'Enable','on');
                        %
                        % fix menus and toolbar in case df>0
                        %
                        if df>0;
                           set(svar.constancytest,'Enable','off');
                           set(svar.menuchow,'Enable','off');
                           set(svar.menuploberger,'Enable','off');
                           set(svar.bootploberger,'Enable','off');
                           set(svar.chow,'Enable','off');
                           set(svar.ploberger,'Enable','off');
                           set(svar.bootalpha,'Enable','off');
                           %
                           % these may later on be dropped when functions for estimating
                           % beta subject to all restrictions on [Psi alpha], i.e. on C
                           % has been implemented
                           %
                           if isfield(svar,'betagraph')==1;
                              if strcmp(get(svar.betagraph,'Enable'),'on')==1;
                                 cm.disable(1) = 1;
                                 set(svar.betagraph,'Enable','off');
                              end;
                           end;
                           %
                           if isfield(svar,'bootbeta')==1;
                              if strcmp(get(svar.bootbeta,'Enable'),'on')==1;
                                 set(svar.bootbeta,'Enable','off');
                                 cm.disable(2) = 1;
                              end;
                           end;
                           %
                           if isfield(svar,'bootnyblommenu')==1;
                              if strcmp(get(svar.bootnyblommenu,'Enable'),'on')==1;
                                 set(svar.bootnyblommenu,'Enable','off');
                                 cm.disable(3) = 1;
                              end;
                           end;
                        end;
                     else;
                        cm.Psi = prev.Psi;
                        cm.alpha = prev.alpha;
                        cm.beta = prev.beta;
                        cm.epsilon = prev.epsilon;
                        cm.Omega = prev.Omega;
                        cm.Sigma_theta = prev.Sigma_theta;
                     end;
                  else;
                     txt = ['The test for local identification failed. The tested matrix has rank ' num2str(rank_test,'%0.0f') ', i.e., less than the number of endogenous variables squared (' num2str(cm.n*cm.n,'%0.0f') '). Please edit your existing restrictions or select another method.'];
                     about(txt,'warning','Identification and Estimation',120,350,'local_identification');
                  end;
               else;
                  setlogtext('Estimation of B aborted...');
               end;
               set(svarguidlg,'Visible','on');
               drawnow;
            elseif status1==1;
               if (strcmp(type_var,'long_run')==1)&(num_res>(cm.n*(cm.n-1))/2)&(cm.r>0)&(cm.r<cm.n);
                  txt = ['The model cannot be identified using long-run restrictions only. While the total number of restrictions ' num2str(num_res,'%0.0f') ' is greater than what is required for exact identification, it is still under-identified. Please add some restrictions on the contemporaneous effects.'];
                  about(txt,'warning','Identification and Estimation',130,350);
               else;
                  if num_res<(cm.n*(cm.n-1))/2;
                     txt = ['The model is under-identified. You have selected ' num2str(num_res,'%0.0f') ' restrictions, while exact identification requires ' num2str((cm.n*(cm.n-1))/2,'%0.0f') ' restrictions. Please add some more restrictions.'];
                  elseif (cm.restrict(1,3)==1)&(num_res>=(cm.n*(cm.n-1))/2);
                     txt = ['The model is under-identified. Although the number of identifying restrictions (' num2str(num_res,'%0.0f') ') may seem to be sufficient, the restrictions you have on the C matrix interfere with them. Please reformulate your identifying restrictions.'];
                  elseif (cm.restrict(1,1)==1)&(num_res>=(cm.n*(cm.n-1))/2);
                     txt = ['The model is under-identified. Although the number of identifying restrictions ('  num2str(num_res,'%0.0f') ') may seem to be sufficient, the restrictions you have on the alpha matrix interfere with them. Please reformulate your identifying restrictions.'];
                  else;
                     txt = 'The model is under-identified for some unknown reason. Please reformulate your identifying restrictions.';
                  end;
                  about(txt,'warning','Identification and Estimation',120,350);
               end;
            elseif status1==2;
               txt = 'The restrictions imply that the B matrix is singular. Since Omega = B*B'' this is not feasible. Please edit your existing restrictions.';
               about(txt,'warning','Identification and Estimation',120,350);
            elseif status1==3;
               txt = ['Some of your restrictions are redundant. The total number of restrictions is ' num2str(tot_res,'%0.0f') ', while the number of linearly independent restrictions is ' num2str(num_res,'%0.0f') '. Please remove the redundant ones.'];
               about(txt,'warning','Identification and Estimation',120,350);
            elseif status1==4;
               txt = 'Some of the long-run restrictions cannot be satisfied by the model. Please remove these restrictions.';
               about(txt,'warning','Identification and Estimation',120,350);
            elseif status1==5;
               txt = ['The restrictions imply either that B is singular or that the rank of C, ' num2str(C_rank,'%0.0f') ', is incompatible with the number of cointegration relations. Please rewrite your restrictions.'];
               about(txt,'warning','Identification and Estimation',120,350);
            end;
         cm.running = 0;
      end;
   case 'struc_undo'
      if cm.restrict(1,3)==0;
         cm.beta = cm.old_values.beta;
         cm.epsilon = cm.old_values.epsilon;
         cm.Psi = cm.old_values.Psi;
         cm.alpha = cm.old_values.alpha;
         cm.Theta = [cm.Psi cm.alpha];
         cm.Sigma_theta = cm.old_values.Sigma_theta;
         cm.df = cm.old_values.df;
         if isfield(cm,'stdbeta')==1;
            cm.stdbeta = cm.old_values.stdbeta;
         end;
         cm.evalue = cm.old_values.evalue;
         cm.Omega = cm.old_values.Omega;
      else;
         cm.beta = cm.C_values.beta;
         cm.epsilon = cm.C_values.epsilon;
         cm.Psi = cm.C_values.Psi;
         cm.alpha = cm.C_values.alpha;
         cm.Theta = [cm.Psi cm.alpha];
         cm.Sigma_theta = cm.C_values.Sigma_theta;
         cm.df = cm.C_values.df;
         if isfield(cm,'stdbeta')==1;
            cm.stdbeta = cm.C_values.stdbeta;
         end;
         cm.evalue = cm.C_values.evalue;
         cm.Omega = cm.C_values.Omega;
      end;
      set(svar.estalpha,'UserData',cm.Theta);
      cm.dfb = 0;
      cm.B_0 = cm.old_values.B_0;
      cm.Sigma_B0 = cm.old_values.Sigma_B0;
      cm.Sigma_B0_adj = cm.old_values.Sigma_B0_adj;
      cm.A = cm.old_values.A;
      cm.Sigma_A = cm.old_values.Sigma_A;
      cm.S = cm.old_values.S;
      cm.gamma = cm.old_values.gamma;
      cm.R_l = cm.old_values.R_l;
      cm.R_mat = cm.old_values.R_mat;
      cm.C = cm.old_values.C;
      cm.dC_theta = cm.old_values.dC_theta;
      cm.dB_theta = cm.old_values.dB_theta;
      %
      cm.bZ1t = cm.beta'*cm.Z1t;
      cm.bR1t = cm.beta'*cm.R1t;
      %
      if isfield(cm,'ir')==1;
         cm = rmfield(cm,'ir');
         cm = rmfield(cm,'upper');
         cm = rmfield(cm,'lower');
         cm = rmfield(cm,'type');
         cm = rmfield(cm,'stdvec');
         cm = rmfield(cm,'betair');
         cm = rmfield(cm,'betair_std');
         cm = rmfield(cm,'betair_upper');
         cm = rmfield(cm,'betair_lower');
      end;
      %
      if isfield(svar,'restrict_c')==1;
         set(svar.restrict_c,'Enable','on');
         set(svar.c_openrest,'Enable','on');
         set(svar.c_saverest,'Enable','on');
         set(svar.restrict_cmenu,'Enable','on');
         set(svar.c_openrestmenu,'Enable','on');
         set(svar.c_saverestmenu,'Enable','on');
         %
         file = strcat(cm.modelsdir,'C-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            set(svar.c_estimate,'Enable','on');
            set(svar.c_estimatemenu,'Enable','on');
            if cm.restrict(1,3)==1;
               set(svar.c_undo,'Enable','on');
               set(svar.c_undomenu,'Enable','on');
            end;
         end;
      end;
      %
      set(svar.estalpha,'UserData',cm.Theta);
      set(svar.irfunction,'Enable','off');
      set(svar.menuirfunction,'Enable','off');
      set(svar.menu_permcomp,'Enable','off');
      set(svar.histdecomp,'Enable','off');
      set(svar.vardecomp,'Enable','off');
      set(svar.bootirmenu,'Enable','off');
      set(svar.struc_undo,'Enable','off');
      set(svar.struc_undomenu,'Enable','off');
      %
      cm.model_info.info = setmodelinfo(cm.model_info.info,cm.T,cm.df,loglike(cm.epsilon),cm.evalue);
      set(svar.model_info,'String',cm.model_info.info);
      %
      set(svar.betamat,'String',strvcat(' ',' ','   The estimated B matrix was recycled.'));
      set(svar.amat,'Enable','off');
      set(svar.bmat,'Enable','off');
      if isfield(svar,'dummy')==1;
         set(svar.dummy,'Enable','on');
      end;
      if isfield(svar,'common_cycles')==1;
         set(svar.common_cycles,'Enable','on');
      end;
      setlogtext('You decided to recycle the estimated B matrix...');
      %
      % fix menus and toolbar in case they've been disabled by df>0
      %
      set(svar.constancytest,'Enable','on');
      set(svar.menuchow,'Enable','on');
      set(svar.menuploberger,'Enable','on');
      set(svar.bootploberger,'Enable','on');
      set(svar.chow,'Enable','on');
      set(svar.ploberger,'Enable','on');
      if (isempty(cm.Z2)==0)|(tm.dfalpha~=tm.n*tm.r);
         set(svar.bootalpha,'Enable','on');
      end;
      %
      if isfield(svar,'betagraph')==1;
         if cm.disable(1)==1;
            set(svar.betagraph,'Enable','on');
         end;
      end;
      %
      if isfield(svar,'bootbeta')==1;
         if cm.disable(2)==1;
            set(svar.bootbeta,'Enable','on');
         end;
      end;
      %
      if isfield(svar,'bootnyblommenu')==1;
         if cm.disable(3)==1;
            set(svar.bootnyblommenu,'Enable','on');
         end;
         if (curr_ini.nyblom==0)|(curr_ini.nyblombeta==0);
            set(svar.bootnyblommenu,'Enable','off');
         end;
      end;
      cm.disable = zeros(1,3);
   case 'check_rest'
      whichobj = gcbo;
      set(whichobj,'Value',1);
      if whichobj==svar.sr_rest;
         set(svar.lr_rest,'Value',0);
         set(svar.joint_rest,'Value',0);
      elseif whichobj==svar.lr_rest;
         set(svar.sr_rest,'Value',0);
         set(svar.joint_rest,'Value',0);
      else;
         set(svar.sr_rest,'Value',0);
         set(svar.lr_rest,'Value',0);
      end;
   case 'lr_openrest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_open_struc_rest = open_struc_restgui('init',cm,'long_run');
         open_struc_rest = get(this_open_struc_rest,'UserData');
         set(open_struc_rest.gmat,'UserData','done');
         killguipid(this_open_struc_rest);
         %
         if strcmp(computer,'PCWIN')==1;
            file = [pwd '\structural_est.txt'];
         else;
            file = [pwd '/structural_est.txt'];
         end;
         if fileexist(file)==1;
            check_time = 1;
            est_time = getfiledate(file);
         else;
            check_time = 0;
         end;
         %
         % check if restricts file exists
         %
         estimateenable = get(svar.struc_estimate,'Enable');
         lr_enable = 'off';
         file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            lr_enable = 'on';
            estimateenable = 'on';
            if check_time==1;
               lr_time = getfiledate(file);
               if lr_time>est_time;
                  check_time = 2;
               end;
            end;
         end;
         sr_enable = 'off';
         file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            sr_enable = 'on';
            if check_time==1;
               sr_time = getfiledate(file);
               if sr_time>est_time;
                  check_time = 2;
               end;
            end;
         end;
         if check_time==2;
            set(svar.bootirmenu,'Enable','off');
         end;
         set(svar.struc_estimate,'Enable',estimateenable);
         set(svar.struc_estimatemenu,'Enable',estimateenable);
         set(svar.signshock,'Enable',estimateenable);
         set(svar.signshockmenu,'Enable',estimateenable);
         set(svar.lr_rest,'Enable',lr_enable);
         if ((get(svar.sr_rest,'Value')==1)|(get(svar.joint_rest,'Value')==1))&(strcmp(lr_enable,'on')==1)&(strcmp(sr_enable,'off')==1);
            set(svar.lr_rest,'Value',1);
            set(svar.sr_rest,'Value',0);
            set(svar.joint_rest,'Value',0);
         end;
         if (strcmp(sr_enable,'on')==1)&(strcmp(lr_enable,'on')==1);
            set(svar.joint_rest,'Enable','on');
         else;
            set(svar.joint_rest,'Enable','off');
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'sr_openrest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_open_struc_rest = open_struc_restgui('init',cm,'short_run');
         open_struc_rest = get(this_open_struc_rest,'UserData');
         set(open_struc_rest.gmat,'UserData','done');
         killguipid(this_open_struc_rest);
         %
         if strcmp(computer,'PCWIN')==1;
            file = [pwd '\structural_est.txt'];
         else;
            file = [pwd '/structural_est.txt'];
         end;
         if fileexist(file)==1;
            check_time = 1;
            est_time = getfiledate(file);
         else;
            check_time = 0;
         end;
         %
         % check if restricts file exists
         %
         estimateenable = get(svar.struc_estimate,'Enable');
         lr_enable = 'off';
         file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            lr_enable = 'on';
            if check_time==1;
               lr_time = getfiledate(file);
               if lr_time>est_time;
                  check_time = 2;
               end;
            end;
         end;
         sr_enable = 'off';
         file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            sr_enable = 'on';
            estimateenable = 'on';
            if check_time==1;
               sr_time = getfiledate(file);
               if sr_time>est_time;
                  check_time = 2;
               end;
            end;
         end;
         if check_time==2;
            set(svar.bootirmenu,'Enable','off');
         end;
         set(svar.struc_estimate,'Enable',estimateenable);
         set(svar.struc_estimatemenu,'Enable',estimateenable);
         set(svar.signshock,'Enable',estimateenable);
         set(svar.signshockmenu,'Enable',estimateenable);
         set(svar.sr_rest,'Enable',sr_enable);
         if ((get(svar.lr_rest,'Value')==1)|(get(svar.joint_rest,'Value')==1))&(strcmp(lr_enable,'off')==1)&(strcmp(sr_enable,'on')==1);
            set(svar.sr_rest,'Value',1);
            set(svar.lr_rest,'Value',0);
            set(svar.joint_rest,'Value',0);
         end;
         if (strcmp(sr_enable,'on')==1)&(strcmp(lr_enable,'on')==1);
            set(svar.joint_rest,'Enable','on');
         else;
            set(svar.joint_rest,'Enable','off');
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'lr_saverest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_save_struc_restgui = save_struc_restgui('init',cm,'long_run');
         save_struc_rest = get(this_save_struc_restgui,'UserData');
         set(save_struc_rest.gmat,'UserData','done');
         killguipid(this_save_struc_restgui);
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'sr_saverest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         this_save_struc_restgui = save_struc_restgui('init',cm,'short_run');
         save_struc_rest = get(this_save_struc_restgui,'UserData');
         set(save_struc_rest.gmat,'UserData','done');
         killguipid(this_save_struc_restgui);
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'restrict_sr'
      if cm.running==0;
         cm.running = 1;
         %
         % Setting up a default F matrix
         %
         cm.F = zeros(floor((cm.n*(cm.n-1))/2),6);
         num = 1;
         for i=1:cm.n-1;
            for j=i+1:cm.n;
               cm.F(num,1) = i;
               cm.F(num,2) = j;
               num = num+1;
            end;
         end;
         %
         % Checking if these already exists a file, and if so
         % check it and then load it
         %
         file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            cm.F = loaddata(file);
         end;
         %
         %  Hide current window
         %
         set(svarguidlg,'Visible','off');
         drawnow;
         rd = 'redraw';
         while strcmp(rd,'redraw')==1;
            thisfmat = identmatdlg('init',cm,'short_run');
            fmat = get(thisfmat,'UserData');
            cancel = get(fmat.cancel,'UserData');
            rd = get(fmat.numres,'UserData');
            cm.F = get(fmat.text,'UserData');
            set(fmat.numelem,'UserData','done');
            killrestpid(thisfmat);
            drawnow;
         end;
         set(svarguidlg,'Visible','on');
         if cancel==0;
            if strcmp(computer,'PCWIN')==1;
               file = [pwd '\structural_est.txt'];
            else;
               file = [pwd '/structural_est.txt'];
            end;
            if fileexist(file)==1;
               check_time = 1;
               est_time = getfiledate(file);
            else;
               check_time = 0;
            end;
            file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               cm.F = loaddata(file);
               [Rmat,rvec] = rmatrix(cm.F,cm.n,cm.n);
               if check_time==1;
                  sr_time = getfiledate(file);
                  if sr_time>est_time;
                     set(svar.bootirmenu,'Enable','off');
                  end;
               end;
            else;
               Rmat = 0;
               cm.F = [];
               set(svar.sr_rest,'Enable','off');
               set(svar.joint_rest,'Enable','off');
               if strcmp(get(svar.lr_rest,'Enable'),'off')==1;
                  set(svar.sr_rest,'Value',1);
                  set(svar.lr_rest,'Value',0);
                  set(svar.joint_rest,'Value',0);
               else;
                  set(svar.sr_rest,'Value',0);
                  set(svar.lr_rest,'Value',1);
                  set(svar.joint_rest,'Value',0);
               end;
            end;
            if rank(Rmat)==size(cm.F,1);
               %
               % check if restricts file exists
               %
               estimateenable = get(svar.struc_estimate,'Enable');
               lr_enable = 'off';
               file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
               if fileexist(file)==1;
                  lr_enable = 'on';
               end;
               sr_enable = 'off';
               file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
               if fileexist(file)==1;
                  sr_enable = 'on';
                  estimateenable = 'on';
               end;
               set(svar.struc_estimate,'Enable',estimateenable);
               set(svar.struc_estimatemenu,'Enable',estimateenable);
               set(svar.signshock,'Enable',estimateenable);
               set(svar.signshockmenu,'Enable',estimateenable);
               set(svar.sr_rest,'Enable',sr_enable);
               if ((get(svar.lr_rest,'Value')==1)|(get(svar.joint_rest,'Value')==1))&(strcmp(lr_enable,'off')==1)&(strcmp(sr_enable,'on')==1);
                  set(svar.sr_rest,'Value',1);
                  set(svar.lr_rest,'Value',0);
                  set(svar.joint_rest,'Value',0);
               end;
               if (strcmp(sr_enable,'on')==1)&(strcmp(lr_enable,'on')==1);
                  set(svar.joint_rest,'Enable','on');
               else;
                  set(svar.joint_rest,'Enable','off');
               end;
            end;
         else;
            %
            % check if file exists
            %
            file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==0;
               set(svar.sr_rest,'Enable','off');
               set(svar.joint_rest,'Enable','off');
               if strcmp(get(svar.lr_rest,'Enable'),'off')==1;
                  set(svar.sr_rest,'Value',1);
                  set(svar.lr_rest,'Value',0);
                  set(svar.joint_rest,'Value',0);
               else;
                  set(svar.sr_rest,'Value',0);
                  set(svar.lr_rest,'Value',1);
                  set(svar.joint_rest,'Value',0);
               end;
            end;
         end;
         cm.running = 0;
      end;
   case 'restrict_lr'
      if cm.running==0;
         cm.running = 1;
         %
         % Setting up a default F matrix
         %
         if cm.n-cm.r-1>0;
            cm.F = zeros((((cm.n-cm.r)*(cm.n-cm.r-1))/2)+(cm.n-cm.r-1)*cm.r,6);
         else;
            cm.F = zeros(floor((cm.n*(cm.n-1))/2),6);
         end;
         num = 1;
         if cm.n-cm.r-1>0;
            for i=1:cm.n;
               for j=i+1:cm.n-cm.r;
                  cm.F(num,1) = i;
                  cm.F(num,2) = j;
                  num = num+1;
               end;
               if i<cm.n-cm.r;
                  for j=cm.n-cm.r+1:cm.n;
                     cm.F(num,1) = i;
                     cm.F(num,2) = j;
                     num = num+1;
                  end;
               end;
            end;
         else;
            for i=1:cm.n-1;
               for j=i+1:cm.n;
                  cm.F(num,1) = i;
                  cm.F(num,2) = j;
                  num = num+1;
               end;
            end;
         end;
         %
         % Checking if these already exists a file, and if so
         % check it and then load it
         %
         file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
         if fileexist(file)==1;
            cm.F = loaddata(file);
         end;
         %
         %  Hide current window
         %
         set(svarguidlg,'Visible','off');
         drawnow;
         rd = 'redraw';
         while strcmp(rd,'redraw')==1;
            thisfmat = identmatdlg('init',cm,'long_run');
            fmat = get(thisfmat,'UserData');
            cancel = get(fmat.cancel,'UserData');
            rd = get(fmat.numres,'UserData');
            cm.F = get(fmat.text,'UserData');
            set(fmat.numelem,'UserData','done');
            killrestpid(thisfmat);
            drawnow;
         end;
         set(svarguidlg,'Visible','on');
         if cancel==0;
            if strcmp(computer,'PCWIN')==1;
               file = [pwd '\structural_est.txt'];
            else;
               file = [pwd '/structural_est.txt'];
            end;
            if fileexist(file)==1;
               check_time = 1;
               est_time = getfiledate(file);
            else;
               check_time = 0;
            end;
            file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               if check_time==1;
                  lr_time = getfiledate(file);
                  if lr_time>est_time;
                     set(svar.bootirmenu,'Enable','off');
                  end;
               end;
               cm.F = loaddata(file);
               [Rmat,rvec] = rmatrix(cm.F,cm.n,cm.n);
               if cm.r<cm.n;
                  %
                  % need to make sure that any restrictions on beta are satisfied
                  % and do not make restrictions on C*B redundant
                  %
                  if cm.restrict(1,2)==1;
                     %
                     % create a random beta which satisfies the selected restrictions
                     %
                     beta = [];
                     for i=1:cm.r;
                        file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
                        Hmat = loaddata(file);
                        beta = [beta Hmat*randn(size(Hmat,2),1)];
                     end;
                     %
                     % now we normalize
                     %
                     file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
                     h_vec = loaddata(file);
                     for i=1:cm.r;
                        norm_var = h_vec(i,1);
                        normbetai = beta(norm_var,i);
                        if abs(normbetai)<1.0e-006;
                           normbetai = 1;
                        end;
                        for j=1:size(beta,1);
                           beta(j,i) = beta(j,i)/normbetai;
                        end;
                     end;
                     beta = beta(1:cm.n,:);
                     beta_orth = null(beta','r');
                  else;
                     beta_orth = randn(cm.n,cm.n-cm.r);
                  end;
                  Rmat = Rmat*kron(eye(cm.n),beta_orth);
               end;
            else;
               Rmat = 0;
               cm.F = [];
               set(svar.lr_rest,'Enable','off','Value',0);
               set(svar.joint_rest,'Enable','off','Value',0);
               set(svar.sr_rest,'Value',1);
            end;
            if rank(Rmat)==size(cm.F,1);
               %
               % check if restricts file exists
               %
               estimateenable = get(svar.struc_estimate,'Enable');
               lr_enable = 'off';
               file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
               if fileexist(file)==1;
                  lr_enable = 'on';
                  estimateenable = 'on';
               end;
               sr_enable = 'off';
               file = strcat(cm.modelsdir,'F-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
               if fileexist(file)==1;
                  sr_enable = 'on';
               end;
               set(svar.struc_estimate,'Enable',estimateenable);
               set(svar.struc_estimatemenu,'Enable',estimateenable);
               set(svar.signshock,'Enable',estimateenable);
               set(svar.signshockmenu,'Enable',estimateenable);
               set(svar.lr_rest,'Enable',lr_enable);
               if ((get(svar.sr_rest,'Value')==1)|(get(svar.joint_rest,'Value')==1))&(strcmp(lr_enable,'on')==1)&(strcmp(sr_enable,'off')==1);
                  set(svar.lr_rest,'Value',1);
                  set(svar.sr_rest,'Value',0);
                  set(svar.joint_rest,'Value',0);
               end;
               if (strcmp(sr_enable,'on')==1)&(strcmp(lr_enable,'on')==1);
                  set(svar.joint_rest,'Enable','on');
               else;
                  set(svar.joint_rest,'Enable','off');
               end;
            end;
         else;
            %
            % check if file exists
            %
            file = strcat(cm.modelsdir,'LF-',int2str(cm.n),'x',int2str(cm.r),'-',cm.modelfile,'.rest');
            if fileexist(file)==0;
               set(svar.lr_rest,'Enable','off','Value',0);
               set(svar.joint_rest,'Enable','off','Value',0);
               set(svar.sr_rest,'Value',1);
            end;
         end;
         cm.running = 0;
      end;
   case 'distribution'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         %
         thisgraph = distdlg('init',cm.cbs);
         killguipid(thisgraph);
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootploberger'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         pkk_bootstrap('init',cm,'nocoint');
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'boot_specification'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         specification_bootstrap('init',cm,'nocoint');
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootalpha'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         alpha_bootstrap('init',cm);
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootbeta'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         beta_bootstrap('init',cm);
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootir'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         ir_bootstrap('init',cm);
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'bootnyblomtest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         nyblom_bootstrap('init',cm,get(svarguidlg,'UserData'));
         drawnow;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'betagraph'
      if cm.running==0;
         cm.running = 1;
         %
         % Find the free parameters of beta
         %
         bH = [];
         for i=1:cm.r;
            file = strcat(cm.modelsdir,'H_',num2str(i),'-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.rest');
            if fileexist(file)==1;
               h = loaddata(file);
               if i==1;
                  bH = h;
               else;
                  bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
               end;
            end;
         end;
         thisR = null(bH','r');
         file = strcat(cm.modelsdir,'normalize-',num2str(cm.n),'x',num2str(cm.r),'x',num2str(cm.dcoint+cm.q1),'-',cm.modelfile,'.norm');
         h = loaddata(file);
         for i=1:cm.r;
            thisvec = zeros(size(bH,1),1);
            thisvec(h(i,1)+(i-1)*size(cm.beta,1),1) = 1;
            thisR = [thisR thisvec];
         end;
         R_orth = null(thisR');
         for i=1:size(R_orth,2);
            if max(R_orth(:,i))>0;
               R_orth(:,i) = (1/max(R_orth(:,i)))*R_orth(:,i);
            else;
               R_orth(:,i) = (1/min(R_orth(:,i)))*R_orth(:,i);
            end;
         end;
         cm.R_orth = round(R_orth);
         %
         % The matrix R_orth shows the free parameters of vec(beta)
         %
         set(svarguidlg,'Visible','off');
         drawnow;
         mybetagraphgui = betagraphdlg('init',cm);
         graphbeta = get(mybetagraphgui,'UserData');
         set(graphbeta.graph,'UserData','done');
         killguipid(mybetagraphgui);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'showbeta'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         if cm.restrict(1,2)==1;
            txt = showbeta(cm.beta,cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]),['%' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f']);
         else;
            txt = showbeta(normbeta(cm.beta),cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]),['%' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f']);
         end;
         if isempty(cm.stdbeta)==0;
            txt = strvcat(txt,'with estimated asymptotic conditional standard errors:',showbeta(cm.stdbeta,cm.evars,strtrim([cm.xvars1 ' ' cm.restvec]),['%' num2str(8+curr_ini.decimals,'%0.0f') '.' num2str(curr_ini.decimals,'%0.0f') 'f']));
         end;
         textgui(txt,'Cointegration Vectors',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'residanalysis'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_residual_analysis(svarguidlg,cm,ind);
         cm.running = 0;
      end;
   case 'histdecomp'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         %
         fh = forecast_horizon('Historical Decompositions',cm.freq,cm.T,'ok');
         %
         if fh>0;
            [f_err,f_err_decomp,abort] = hist_decomp(cm,fh,cm.alpha,cm.beta,cm.Psi,cm.B_0,cm.epsilon,1);
            %
            if strcmp(abort,'cancel')==0;
               thisgraph = histdecompdlg('init',fh,f_err,f_err_decomp,cm);
               histdecomp = get(thisgraph,'UserData');
               set(histdecomp.show,'UserData','done');
               killguipid(thisgraph);
            end;
         end;
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'fitted'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         mytheta = [cm.Psi cm.alpha];
         thisgraph = fittedterms('init',cm,mytheta,cm.beta);
         fitted = get(thisgraph,'UserData');
         set(fitted.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'companion'
      if cm.running==0;
         cm.running = 1;
         rind = 0;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
            rind = 1;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_companion(cm.beta,cm.alpha,cm.Psi,cm,curr_ini,ind,rind);
         cm.running = 0;
      end;
   case 'common_cycles'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         show_common_cycles(cm,curr_ini,cm.alpha,cm.Psi,cm.beta);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'dummyprop'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         proportionality_dummies_dlg('init',cm,curr_ini);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'dummylongrun'
      if cm.running==0;
         cm.running = 1;
         show_longrun_dummies(svarguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'dummyexclusion'
      if cm.running==0;
         cm.running = 1;
         show_exclude_dummies(svarguidlg,cm,curr_ini);
         cm.running = 0;
      end;
   case 'dummyimpulse'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         %
         alpha = cm.alpha;
         beta = cm.beta;
         Psi = cm.Psi;
         Omega = cm.Omega;
         Sigma_theta = cm.Sigma_theta;
         %
         % check if we have a structural model
         %
         if strcmp(get(svar.struc_undo,'Enable'),'off')==1;
            %
            txt = 'Would you like to impose restrictions on the dummy variables?';
            if (cm.datad>1)|((cm.r<cm.n)&(cm.r>0));
               answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'no');
            else;
               answer = 'No';
            end;
            if strcmp(answer,'Yes')==1;
               %
               % open a select restrictions dialog
               %
               tm = restrict_dummies_dlg('init',cm,curr_ini);
               if strcmp(tm.action,'cancel')==0;
                  %
                  % check if we should use new estimated
                  %
                  if (isfield(tm,'applied')==1)&(isempty(tm.applied)==0)&(ischar(tm.applied)==1)&(strcmp(tm.applied,'applied')==1);
                     if isfield(tm,'dm')==1;
                        txt = 'Are you sure you want to use the restricted estimates for the dummy impulse responses?';
                        answer = query(txt,'question',120,'Structural VAR - Dummy Restrictions',300,'yes');
                        if strcmp(answer,'Yes')==1;
                           alpha = tm.dm.alpha;
                           beta = tm.dm.beta;
                           Psi = tm.dm.Psi;
                           Omega = tm.dm.Omega;
                           Sigma_theta = tm.dm.Sigma_theta;
                        end;
                     end;
                  end;
               else;
                  %
                  % no dummy impulses
                  %
                  set(svarguidlg,'Visible','on');
                  drawnow;
                  return;
               end;
            end;
         end;
         %
         setlogtext(strvcat(['Computing impulse responses for dummies up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
         abort = '';
         %
         [cm.duir,cm.duir_std,cm.buir,cm.buir_std,abort] = dummy_impulses_fcn(cm,alpha,beta,Psi,Omega,Sigma_theta,1);
         %
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+cm.c)/200);
            for i=1:size(cm.duir,2);
               for j=1:cm.n*cm.datad;
                  if abs(cm.duir_std(j,i))<1.0e-008;
                     cm.duir_std(j,i) = 0;
                  end;
               end;
            end;
            cm.duir_upper = cm.duir+(c_val*cm.duir_std);
            cm.duir_lower = cm.duir-(c_val*cm.duir_std);
            %
            if isempty(cm.buir)==0;
               for i=1:size(cm.buir,2);
                  for j=1:cm.r*cm.datad;
                     if abs(cm.buir_std(j,i))<1.0e-008;
                        cm.buir_std(j,i) = 0;
                     end;
                  end;
               end;
               cm.buir_upper = cm.buir+(c_val*cm.buir_std);
               cm.buir_lower = cm.buir-(c_val*cm.buir_std);
            else;
               cm.buir_upper = [];
               cm.buir_lower = [];
            end;
            %
            thisgraph = duir_graphdlg('init',cm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'genimpulse'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         setlogtext(strvcat(['Computing generalized impulse responses up to period ' num2str(cm.h)],['with asymptotic ' num2str(cm.c) ' % confidence bands...']));
         abort = '';
         %
         [cm.gir,cm.gir_std,cm.bir,cm.bir_std,abort] = generalized_impulses_fcn(cm,cm.alpha,cm.beta,cm.Psi,cm.Omega,cm.Sigma_theta,1,cm.B_0,cm.Sigma_B0);
         if strcmp(abort,'cancel')==0;
            %
            % get the value from the normal distribution corresponding to the
            % chosen levels of significance c
            %
            c_val = norm_inv((100+cm.c)/200);
            for i=1:size(cm.gir,2);
               for j=1:cm.n*cm.n;
                  if abs(cm.gir_std(j,i))<1.0e-008;
                     cm.gir_std(j,i) = 0;
                  end;
               end;
            end;
            cm.gir_upper = cm.gir+(c_val*cm.gir_std);
            cm.gir_lower = cm.gir-(c_val*cm.gir_std);
            if isempty(cm.bir)==0;
               for i=1:size(cm.bir,2);
                  for j=1:cm.r*cm.n;
                     if abs(cm.bir_std(j,i))<1.0e-008;
                        cm.bir_std(j,i) = 0;
                     end;
                  end;
               end;
               cm.bir_upper = cm.bir+(c_val*cm.bir_std);
               cm.bir_lower = cm.bir-(c_val*cm.bir_std);
            else;
               cm.bir_upper = [];
               cm.bir_lower = [];
            end;
            thisgraph = gir_graphdlg('init',cm);
            ir_graph = get(thisgraph,'UserData');
            set(ir_graph.show,'UserData','done');
            killguipid(thisgraph);
         end;
         %
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'quit'
      if cm.running==0;
         if curr_ini.sillyq==0;
            if (cm.cointvalue==1)&((cm.r==0)|(cm.r==cm.n));
               txt = 'Are you sure you want to return to the Cointegration Rank Tests dialog?';
            elseif (cm.cointvalue==1)&((cm.r>0)&(cm.r~=cm.n+cm.q1+cm.dcoint));
               txt = 'Are you sure you want to return to the Restrictions on (alpha,beta) dialog?';
            end;
            answer = query(txt,'question',120,'Identification and Estimation',350);
         else;
            answer = 'Yes';
         end;
         if strcmp(answer,'Yes');
            thisexit = 2;
            cm.restrict = [0 0];
            set(svar.continue,'UserData','done');
         end;
         set(svar.ploberger,'UserData',thisexit);
         set(svarguidlg,'UserData',svar);
      end;
   case 'graphs'
      if cm.running==0;
         set(svar.ploberger,'UserData',thisexit);
         set(svarguidlg,'Visible','off');
         drawnow;
         cm.running = 1;
         thisgraph = cointgraphdlg('init',cm,cm.bZ1t,cm.bR1t,cm.restrict(1,2));
         graph = get(thisgraph,'UserData');
         set(graph.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         drawnow;
         set(svarguidlg,'UserData',svar);
         cm.running = 0;
      end;
   case 'varforecast'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         thisgraph = var_forecastdlg('init',cm,cm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'forecast'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         thisgraph = forecastdlg('init',cm,cm.restrict);
         f_cast = get(thisgraph,'UserData');
         set(f_cast.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'residuals'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         set(svarguidlg,'Visible','off');
         drawnow;
         thisgraph = epsilondlg('init',cm,ind);
         epsi = get(thisgraph,'UserData');
         set(epsi.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'ploberger'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_ploberger(svarguidlg,cm.beta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'chow'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_chow(svarguidlg,cm.beta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'perm_comp'
      if cm.running==0;
         cm.running = 1;
         [y_t,perm_y,trans_y] = permcomp(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.x0,cm.x1,cm.x1_r,cm.x2,cm.d,cm.dcoint,cm.p,cm.pe,cm.q1,cm.q0);
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.restrict(1,2)==0;
            ind = 0;
         end;
         set(svarguidlg,'Visible','off');
         drawnow;
         %
         % now we look into the shock components
         %
         [perm_shock,trans_shock,abort] = shockcomp(cm,cm.Psi,cm.alpha,cm.beta,cm.B_0,cm.epsilon,perm_y(:,1),trans_y,1);

         if strcmp(abort,'cancel')==0;
            thisgraph = scdlg('init',cm,y_t,perm_y,trans_y,perm_shock,trans_shock,ind);
            sc = get(thisgraph,'UserData');
            set(sc.show,'UserData','done');
            killguipid(thisgraph);
         end;
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'permcomp'
      if cm.running==0;
         cm.running = 1;
         [y_t,perm_y,trans_y] = permcomp(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.x0,cm.x1,cm.x1_r,cm.x2,cm.d,cm.dcoint,cm.p,cm.pe,cm.q1,cm.q0);
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.restrict(1,2)==0;
            ind = 0;
         end;
         set(svarguidlg,'Visible','off');
         drawnow;
         thisgraph = pctcdlg('init',cm,y_t,perm_y,trans_y,ind);
         pctc = get(thisgraph,'UserData');
         set(pctc.show,'UserData','done');
         killguipid(thisgraph);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'corrtest'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_corrtest(svarguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'archtest'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_archtest(svarguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'normalitytest'
      if cm.running==0;
         cm.running = 1;
         if cm.r>0;
            ind = 1;
         else;
            ind = 0;
         end;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_normalitytest(svarguidlg,cm,curr_ini,ind);
         cm.running = 0;
      end;
   case 'constancytest'
      if cm.running==0;
         cm.running = 1;
         if rank(cm.beta'*cm.beta,1.0e-004)==cm.r;
            ind = 1;
            rind = 0;
            if (cm.r==0)|(cm.dfbeta==0);
               ind = 0;
               rind = 1;
            end;
            aind = 0;
            if cm.restrict(1,1)==1;
               aind = 1;
            end;
            show_constancytest(svarguidlg,cm.beta,cm,curr_ini,ind,aind,rind);
         else;
            myrank = rank(cm.beta'*cm.beta,1.0e-004);
            txt = ['The estimated beta matrix is badly scaled. The rank of beta''*beta is ' num2str(myrank(1,1)) ', i.e. less than ' num2str(cm.r) ', when a tolerance level of 1.0e-004 is used. Parameter constancy tests will not be calculated. Please consider some other restrictions.'];
            about(txt,'warning','Badly identified beta matrix',120,350);
         end;
         cm.running = 0;
      end;
   case 'lagorder'
      if cm.running==0;
         cm.running = 1;
         Sigma_theta = cm.Sigma_theta;
         ind = 0;
         if cm.dfbeta>0;
            ind = 1;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_lagordertest(svarguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'firstdiffs'
      if cm.running==0;
         cm.running = 1;
         Sigma_theta = cm.Sigma_theta;
         ind = 0;
         if cm.dfbeta>0;
            ind = 1;
         end;
         show_firstdiffstests(svarguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,cm.restrict(1,3));
         cm.running = 0;
      end;
   case 'gctest'
      if cm.running==0;
         cm.running = 1;
         Sigma_theta = cm.Sigma_theta;
         ind = 0;
         if cm.dfbeta>0;
            ind = 1;
         end;
         aind = 0;
         if cm.restrict(1,1)==1;
            aind = 1;
         end;
         show_gctest(svarguidlg,cm.beta,cm.alpha,cm.Psi,Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'multigctest'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         %
         this_graph = select_mgc('init',cm,cm.beta,cm.Sigma_theta,cm.restrict(1,1),ind,cm.restrict(1,3));
         mgc = get(this_graph,'UserData');
         set(mgc.show,'UserData','done');
         killguipid(this_graph);
         set(svarguidlg,'Visible','on');
         drawnow;
         cm.running = 0;
      end;
   case 'savetc'
      if cm.running==0;
         cm.running = 1;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         [y_t,perm_y,trans_y] = permcomp(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.x0,cm.x1,cm.x1_r,cm.x2,cm.d,cm.dcoint,cm.p,cm.pe,cm.q1,cm.q0);
         myfile = [cm.outputdir 'trans.comp-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(trans_y);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving transitory components to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '              tc_' cm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),trans_y(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'savecoint'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,2)==1;
            CR = cm.beta'*cm.Z1t;
         else;
            CR = normbeta(cm.beta)'*cm.Z1t;
         end;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         myfile = [cm.outputdir 'coint-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myR myT] = size(CR);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving cointegration relations to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myR;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '               ci_' num2str(i,'%1.0f')];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),CR(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'saveresid'
      if cm.running==0;
         cm.running = 1;
         addrest = 'rest-';
         if cm.dfbeta==0;
            addrest = '';
         end;
         myfile = [cm.outputdir 'epsilon-' addrest num2str(cm.r,'%1.0f') '.data'];
         [savefile,savepath] = uiputfile(myfile,'Save As');
         drawnow;
         if isnumeric(savefile)==0;
            [myN myT] = size(cm.epsilon);
            myt = '';
            for i=1:myT;
               myt = strvcat(myt,getdate(cm.samplebeg,cm.sampleend,cm.freq,i/myT));
            end;
            setlogtext(['Saving residuals to ' savefile '...']);
            savefile = strcat(savepath,savefile);
            fid = fopen(savefile,'wt');
            myprintstr = '';
            firststr = ' DATE  ';
            prt_str = ['%' num2str(2*(curr_ini.exportdata+3),'%0.0f') '.' num2str(curr_ini.exportdata+3,'%0.0f') 'f'];
            for i=1:myN;
               myprintstr = [myprintstr '   ' prt_str];
               firststr = [firststr '             eps_' cm.myvars(i,:)];
            end;
            fprintf(fid,[firststr '\n']);
            for i=1:myT;
               fprintf(fid,['%s ' myprintstr '\n'],myt(i,:),cm.epsilon(:,i)');
            end;
            fclose(fid);
         end;
         cm.running = 0;
      end;
   case 'viewdata'
      if cm.running==0;
         cm.running = 1;
         if matlabnumber>=6.0;
            myfile = {'*.txt;*.data;*.wk1;*.xls;*.mat;*.csv', 'Data Files (*.txt, *.data, *.wk1, *.xls, *.mat, *.csv)'; ...
                    '*.txt;*.data', 'Text Files (*.txt, *.data)'; ...
                    '*.mat', 'MatLab MAT Files (*.mat)'; ...
                    '*.wk1', 'Lotus 1-2-3 Spreadsheets (*.wk1)'; ...
                    '*.xls', 'Excel Spreadsheets (*.xls)'; ...
                    '*.csv', 'Comma Separated Value Files (*.csv)'; ...
                    '*.*', 'All Files (*.*)'};
         else;
            if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1);
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               basedir = getpath(strtrim(afile));
               file_fltr = 'Data Files (*.txt, *.data, *.wk1, *.mat, *.csv)|*.txt;*.data;*.wk1;*.mat;*.csv|Text Files (*.txt, *.data)|*.txt;*.data|MatLab MAT Files (*.mat)|*.mat|Lotus 1-2-3 Spreadsheets (*.wk1)|*.wk1|Comma Separated Value Files (*.csv)|*.csv|All Files (*.*)|*.*|';
               best_pos = 1;
               title_txt = 'Open';
            else;
               fix_fig = findobj('Tag','StructuralVAR');
               model = get(fix_fig,'UserData');
               afile = get(model.endogenous,'String');
               afile = strtrim(afile);
               if fileexist(afile)==1;
                  adir = getpath(afile);
                  if strcmp(computer,'PCWIN')==1;
                     if strcmp(adir(1,size(adir,2)),'\')==0;
                        adir = [adir '\'];
                     end;
                  else;
                     if strcmp(adir(1,size(adir,2)),'/')==0;
                        adir = [adir '/'];
                     end;
                  end;
               else;
                  adir = cm.outputdir;
               end;
               myfile = [adir '*.txt;*.data;*.wk1;*.mat;*.csv'];
            end;
         end;
         if (iscompiled==1)&(iswinnt==1)&(curr_ini.opendialog==1)&(matlabnumber<6.0);
            [openfile,openpath] = fileopen(basedir,file_fltr,best_pos,title_txt);
         else;
            if matlabnumber>=6;
               [openfile,openpath] = uigetfile(myfile,'Open',getpath(cm.endofile));
            else;
               [openfile,openpath] = uigetfile(myfile,'Open');
            end;
         end;
         drawnow;
         if isnumeric(openfile)==0;
            openfile = strcat(openpath,openfile);
            set(svarguidlg,'Visible','off');
            drawnow;
            textgui(openfile,['View Data - ' openfile],[(realscrsz(3)-(realscrsz(4)-80))/2 1 realscrsz(4)-80 realscrsz(4)-54],'Data',25);
            set(svarguidlg,'Visible','on');
            drawnow;
         end;
         cm.running = 0;
      end;
   case 'amat'
      if cm.running==0;
         cm.running = 1;
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         if isfield(cm,'signshock')==1;
            H = change_ir_signs(cm.signshock,cm.orig_B_0,cm.n);
         else;
            H = eye(cm.n);
         end;
         std_A = vecinv(sqrt(abs(diag(cm.Sigma_A))),cm.n);
         %
         for i=1:cm.n;
            for j=1:cm.n;
               if abs(cm.A(i,j))<1.0e-008;
                  std_A(i,j) = 0;
               end;
            end;
         end;
         %
         ctext = strvcat(' ',['THE A-MATRIX (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showa(cm.A*H,cm.myvars,cm.myshocks),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showa(std_A,cm.myvars,cm.myshocks),' ','t-ratios:',' ',showa(inf2nan((cm.A*H)./std_A),cm.myvars,cm.myshocks),' ');
         set(svarguidlg,'Visible','off');
         drawnow;
         textgui(ctext,'A-Matrix',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'bmat'
      if cm.running==0;
         cm.running = 1;
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         if isfield(cm,'signshock')==1;
            H = change_ir_signs(cm.signshock,cm.orig_B_0,cm.n);
         else;
            H = eye(cm.n);
         end;
         std_B = vecinv(sqrt(abs(diag(cm.Sigma_B0_adj))),cm.n);
         %
         for i=1:cm.n;
            for j=1:cm.n;
               if abs(cm.orig_B_0(i,j))<1.0e-008;
                  std_B(i,j) = 0;
               end;
            end;
         end;
         %
         ctext = strvcat(' ',['THE B-MATRIX (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showa(cm.orig_B_0*H,cm.myvars,cm.myshocks),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showa(std_B,cm.myvars,cm.myshocks),' ','t-ratios:',' ',showa(inf2nan((cm.orig_B_0*H)./std_B),cm.myvars,cm.myshocks),' ');
         set(svarguidlg,'Visible','off');
         drawnow;
         textgui(ctext,'B-Matrix',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'longrun_gir'
      if cm.running==0;
         cm.running = 1;
         set(svarguidlg,'Visible','off');
         drawnow;
         %
         [lr_gir,lr_gir_std] = longrun_gir_fcn(cm.alpha,cm.beta,cm.Psi,cm.Omega,cm.Sigma_theta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,cm.B_0,cm.Sigma_B0);
         %
         for i=1:cm.n;
            for j=1:cm.n;
               if abs(lr_gir_std(i,j))<1.0e-005;
                  lr_gir_std(i,j) = 0;
                  lr_gir(i,j) = mystr2num(strtrim(num2str(lr_gir(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
               end;
            end;
         end;
         %
         %
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         ctext = strvcat(' ',['LONG RUN GENERALIZED IMPULSE RESPONSES (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(lr_gir,cm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(lr_gir_std,cm.myvars),' ','t-ratios:',' ',showc(inf2nan(lr_gir./lr_gir_std),cm.myvars),' ');
         drawnow;
         textgui(ctext,'Long Run GIRs',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         %
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'cmat'
      if cm.running==0;
         cm.running = 1;
         addrest = ' restricted';
         if cm.dfbeta==0;
            addrest = '';
         end;
         if cm.r<cm.n;
            C = cmat(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,cm.Sigma_theta);
         else;
            C = cmat_stat(cm.Psi,cm.alpha,cm.beta,cm.epsilon,cm.Z1,cm.Z2,cm.p,cm.d,1,cm.Sigma_theta);
         end;
         ctext = strvcat(' ',['THE C-MATRIX (for' addrest ' rank=' num2str(cm.r,'%1.0f') ')']);
         numchar = size(ctext,2);
         addcorr = '';
         for i=1:numchar;
            addcorr = strcat(addcorr,'=');
         end;
         C_std = C(:,cm.n+1:2*cm.n);
         C_t = C(:,1:cm.n);
         for i=1:cm.n;
            for j=1:cm.n;
               if abs(C_std(i,j))<1.0e-005;
                  C_std(i,j) = 0;
                  C_t(i,j) = mystr2num(strtrim(num2str(C_t(i,j),['%0.' num2str(curr_ini.decimals,'%0.0f') 'f'])));
               end;
            end;
         end;
         %
         ctext = strvcat(ctext,addcorr,' ');
         ctext = strvcat(ctext,showc(C_t,cm.myvars),' ');
         ctext = strvcat(ctext,'Estimated asymptotic standard errors:',' ',showc(C_std,cm.myvars),' ','t-ratios:',' ',showc(inf2nan(C_t./C_std),cm.myvars),' ');
         set(svarguidlg,'Visible','off');
         drawnow;
         textgui(ctext,'C-Matrix',[(realscrsz(3)-700)/2 (realscrsz(4)-500)/2 700 500],'Results',38);
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'weakexo'
      if cm.running==0;
         cm.running = 1;
         if cm.restrict(1,1)==0;
            aind = 0;
         else;
            aind = 1;
         end;
         ind = 1;
         if cm.dfbeta==0;
            ind = 0;
         end;
         show_weakexo(svarguidlg,cm.beta,cm.alpha,cm.Sigma_theta,cm,curr_ini,ind,aind);
         cm.running = 0;
      end;
   case 'shortdyn'
      if cm.running==0;
         cm.running = 1;
         Sigma_theta = cm.Sigma_theta;
         shortstr = shortrun(cm.alpha,cm.Psi,cm.Omega,cm.beta,cm.Z1,cm.Z2,cm.p,cm.pe,cm.d,cm.deterselect,cm.q0,cm.q1,cm.evars,cm.xvars0,cm.xvars1,cm.deter,cm.restvec,1,cm.outputfile,1,Sigma_theta);
         if cm.dfbeta>0;
            myshortstr = shortstr(2,:);
            oldsize = size(myshortstr,2);
            myshortstr = strrep(myshortstr,'for rank=','for restricted rank=');
            newsize = size(myshortstr,2);
            myshortstr = myshortstr(1,1:newsize-11);
            kind = findstr(myshortstr,')');
            if isempty(kind)==0;
               shortstr(2,:) = myshortstr;
               shortstr(3,:) = strrep(shortstr(3,:),'=           ','============');
            end;
         end;
         set(svarguidlg,'Visible','off');
         drawnow;
         textgui(shortstr,'Short Run Dynamics',[(realscrsz(3)-800)/2 (realscrsz(4)-500)/2 800 500],'Results',38);
         set(svarguidlg,'Visible','on');
         cm.running = 0;
      end;
   case 'continue';
      if cm.running==0;
         cm.running = 1;
         set(svar.ploberger,'UserData',thisexit);
         if cm.estimated==0;
            if curr_ini.sillyq==0;
               txt = 'Are you sure you don''t want to estimate a structural model?';
               answer = query(txt,'question',120,'Identification and Estimation',350);
            else;
               answer = 'Yes';
            end;
            if strcmp(answer,'Yes');
               cm.estimated = 1;
            end;
         end;
         if cm.estimated==1;
            set(svar.continue,'UserData','done');
            set(svarguidlg,'UserData',svar);
         else;
            set(svarguidlg,'UserData',svar);
         end;
         cm.running = 0;
      end;
   case 'wayout'
      if cm.running==0;
         cm.running = 1;
         set(svar.ploberger,'UserData',thisexit);
         if cm.estimated==0;
            if curr_ini.sillyq==0;
               txt = 'Are you sure you don''t want to estimate a structural model?';
               answer = query(txt,'question',120,'Identification and Estimation',350);
            else;
               answer = 'Yes';
            end;
            if strcmp(answer,'Yes');
               cm.estimated = 1;
            end;
         end;
         if cm.estimated==1;
            set(svar.continue,'UserData','done');
            if strcmp(get(svar.residgraph,'UserData'),'done')==1;
               delete(svarguidlg);
            end;
         else;
            set(svarguidlg,'UserData',svar);
         end;
         cm.running = 0;
      end;
end;

if strcmp(selector,'init')==1;
   set(svarguidlg,'Visible','on');
   if strcmp(cm.deterdel,'')==0;
      deterdelinfo = ['Due to linear dependence, the following deterministic variables were deleted: ' cm.deterdel '.'];
      set(svar.model_info,'TooltipString',deterdelinfo);
      set(svar.model_header,'TooltipString',deterdelinfo);
   end;
end;

if nargout==1;
   myhandle = svarguidlg;
end;

curr_ini.images = button_images;
curr_ini.model = cm;
set(svar.viewdata,'UserData',curr_ini);

if thisexit==0;
   waitfor(svar.continue,'UserData','done');
end;

%
% end of svargui.m
%
